package org.cn3.morpho.data;

import java.io.File;
import java.io.FileNotFoundException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;


public class GenericMetaDataManager {

	private final static String DEFAULT_RECORD_NAME_COLUMN = "NeuronName";
	
	private File file;
	private Connection conn = null;
	//private List<String[]> data;
	//private Map<String,Integer> rowIndexMap;
	//private Map<String,Integer> columnIndexMap;
	
	public GenericMetaDataManager(String filename) throws Exception{
		setFile(new File(filename));
		importData(file);
	}
	public GenericMetaDataManager(File file) throws Exception{
		setFile(file);
		importData(file);
	}
	public void close(){
		try{
			if (conn != null){
				conn.close();
			}
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	private void setFile(File file) throws FileNotFoundException{
		this.file = file;
		if (!file.exists()){
			throw new FileNotFoundException("File "+file+" not found");
		}else if (file.isDirectory()){
			throw new FileNotFoundException(file+" is a directory");
		}
	}

/*	private void importData(File file) throws IOException{
		FileReader freader = null;
	    BufferedReader reader = null;
	    data = new ArrayList<String[]>();
	    String[] row;
	    int lineNum = 0;
	    int numCols = -1;

	    rowIndexMap = new HashMap<String,Integer>();
	    columnIndexMap = new HashMap<String,Integer>();
	    
	    try{
		    freader = new FileReader(file);
		    reader = new BufferedReader(freader);

			String dataString = "";
			
			while ( (dataString=reader.readLine()) != null ) {
				lineNum++;
				if (!StringUtils.isEmpty(dataString)){
					row = dataString.split(""+separator);
					if (numCols == -1){
						numCols = row.length;
						for (int i = 0; i < numCols; i++){
							columnIndexMap.put(row[i], i);
						}
					}
					else if (row.length != numCols){
						throw new IOException("Error on line "+lineNum+". Expected "+numCols+" columns, found "+row.length);
					}
					else{
						rowIndexMap.put(row[0], lineNum-1);
						data.add(dataString.substring(row[0].length()+1).split(""+separator));
					}
				}
			}
			
		} catch (IOException ioe){
			throw ioe;
		} finally{
			if (reader != null){
				reader.close();
			}
			if (freader != null){
				freader.close();
			}
		}

	}
	*/
	private void importData(File file) throws Exception{
		// load the driver into memory
		Class.forName("org.relique.jdbc.csv.CsvDriver");

		// create a connection. The first command line parameter is assumed to
		//  be the directory in which the .csv files are held
		conn = DriverManager.getConnection("jdbc:relique:csv:" + file.getParent());
	}
	
	public Set<String> getRecordNamesByMetaData(List<MetaDataSelection> metadataSelections) throws Exception{
		return getRecordNamesByMetaData(DEFAULT_RECORD_NAME_COLUMN, metadataSelections);
	}
	public Set<String> getRecordNamesByMetaData(String recordNameColumn, List<MetaDataSelection> metadataSelections) throws Exception{
		Set<String> neuriteNames = new TreeSet<String>();
		Statement stmt = conn.createStatement();
		StringBuffer queryStr = new StringBuffer();
		queryStr.append("Select "+recordNameColumn+" from "+file.getName().substring(0, file.getName().length()-4)+" WHERE ");
		for (MetaDataSelection mds : metadataSelections){
			queryStr.append(mds.getName()+" in (");
			for (int i = 0; i < mds.getValues().size(); i++){
				if (i > 0){
					queryStr.append(",");
				}
				queryStr.append("'"+mds.getValues().get(i)+"'");
			}
		}

		ResultSet results = stmt.executeQuery(queryStr.toString());

		// dump out the results
		while (results.next()){
			neuriteNames.add(results.getString(recordNameColumn));
		}
	      
		return neuriteNames;
	}

}
