package org.cn3.morpho.data;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.cn3.morpho.data.persistence.NeuronDAO;
import org.cn3.morpho.domain.NeuronMetadataComplete;
import org.cn3.morpho.resources.MorphoDataProperties;
import org.cn3.morpho.utils.StringUtils;


public class MetaDataUpdater {

	public static void main(String[] args){
		try{
			update();
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	public static void update() throws Exception{
		// Get meta data file location
		String metaDataFile = MorphoDataProperties.getInstance().getMetaDataFilename();
		List<NeuronMetadataComplete> neurons = NeuronDAO.getInstance().getNeuronInfos();
		NeuronMetadataComplete neuron;
		StringBuffer sb = new StringBuffer();
		
		// Read existing data - to protect non-NeuroMorpho meta data if it exists
		Map<String,NeuronMetadataComplete> metaData = MetaDataManager.getExtendedCellMetaDataFromFile(metaDataFile);
		
		// Write header
		sb.append("#Name,Class2,Species,BrainRegion,Archive,Class1,Class3,Region2,Region3,Gender,Strain,AgeClass,Protocol,ExperimentCondition\n");
		
		// Write database derived meta data to the string buffer
		Set<String> nmSet = new HashSet<String>();
		for (int i = 0; i < neurons.size(); i++){
			neuron = neurons.get(i);
			if (neuron.getName().equals("oi24rpy1-1")){
				System.out.println("Added oi24rpy1-1");
			}
			if (!neuron.getArchive().equalsIgnoreCase("Chiang") &&
					!neuron.getArchive().equalsIgnoreCase("OpenWorm")){
				appendNeuron(sb, neuron);
			}
			nmSet.add(neuron.getName());
		}
		
		// Write non-database meta-data that was in the old meta-data file
		sb.append("\n");
		for (String name : metaData.keySet()){
			if (!nmSet.contains(name) && !name.equals(MetaDataManager.CELL_TYPE_LIST)){
				appendNeuron(sb, metaData.get(name));
			}
		}
		System.out.println("MetaDataFile: "+metaDataFile);
		DataWriter.writeStringToFile(metaDataFile, sb.toString());
	}
	
	private static void appendNeuron(StringBuffer sb, NeuronMetadataComplete neuron){
		sb.append("\""+neuron.getName()+"\",");
		sb.append("\""+neuron.getCellType()+"\",");
		sb.append("\""+neuron.getAnimal()+"\",");
		sb.append("\""+neuron.getBrainRegion()+"\",");
		sb.append("\""+neuron.getArchive()+"\",");
		//Extended
		sb.append("\""+StringUtils.nullAsEmpty(neuron.getClass1())+"\",");
		sb.append("\""+StringUtils.nullAsEmpty(neuron.getClass3())+"\",");
		sb.append("\""+StringUtils.nullAsEmpty(neuron.getRegion2())+"\",");
		sb.append("\""+StringUtils.nullAsEmpty(neuron.getRegion3())+"\",");
		sb.append("\""+StringUtils.nullAsEmpty(neuron.getGender())+"\",");
		sb.append("\""+StringUtils.nullAsEmpty(neuron.getStrain())+"\",");
		sb.append("\""+StringUtils.nullAsEmpty(neuron.getAgeClassification())+"\",");
		sb.append("\""+StringUtils.nullAsEmpty(neuron.getProtocol())+"\",");
		sb.append("\""+StringUtils.nullAsEmpty(neuron.getExperimentCondition()+"\""));
		sb.append("\n");
	}
}
