package org.cn3.morpho.data.persistence;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

import org.cn3.morpho.resources.MorphoProjectProperties;


public class ConnectionFactory {

	private static Map<String,Connection> connMap = new HashMap<String,Connection>();
	public static final String DEFAULT_NEURON_CONNECTION_CLASS = "com.mysql.jdbc.Driver";
	
	protected static Connection getConnection(String dbBase) throws Exception{
		try{
			if (!connMap.containsKey(dbBase)){
		        Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
		        // set this to a MS Access DB you have on your machine
		        
		        String filename = MorphoProjectProperties.getInstance().getResourceDirectory()+"sequenceData-"+dbBase+".mdb";
		        //System.out.println(filename);
		        String db = "jdbc:odbc:Driver={Microsoft Access Driver (*.mdb)};DBQ=";
		        db += filename.trim() + ";DriverID=22;READONLY=true}"; // add on to the end 
		        // now we can get the connection from the DriverManager
		        Connection conn = DriverManager.getConnection(db,"","");
		        connMap.put(dbBase,conn);
			}
			return (Connection)connMap.get(dbBase);
		} catch (Exception e){
			System.out.println(dbBase);
			throw e;
		}
	}

	protected static Connection getNeuronConnection(String dbBase) throws Exception{
		return getNeuronConnection(dbBase, DEFAULT_NEURON_CONNECTION_CLASS);
	}
	protected static Connection getNeuronConnection(String dbBase, String connClass) 
	throws IOException,SQLException,ClassNotFoundException{
		try{
			if (!connMap.containsKey(dbBase)){
				MorphoProjectProperties props = MorphoProjectProperties.getInstance();
		        Class.forName(connClass);
		        
		        //String filename = MorphoProjectProperties.getInstance().getResourceDirectory()+dbBase+".mdb";
		        String db = "jdbc:mysql://"+props.getDatabaseServer();
		        //String db = "jdbc:mysql://Neuron-218A.krasnow.gmu.edu/NeuMO";
		        // now we can get the connection from the DriverManager
		        Connection conn = DriverManager.getConnection(db, props.getDatabaseUsername(), props.getDatabasePassword());
		        connMap.put(dbBase,conn);
			}
			return (Connection)connMap.get("Neuron");
		} catch (ClassNotFoundException e){
			System.out.println(dbBase);
			throw e;
		} catch (SQLException e){
			System.out.println(dbBase);
			throw e;
		}
	}

	protected static boolean destroyNeuron() throws Exception{
		return destroy("Neuron");
	}

	protected static boolean destroy(String dbBase) throws Exception{
		if (connMap.containsKey(dbBase)){
			Connection conn = (Connection)connMap.remove(dbBase);
			if (conn != null && !conn.isClosed()){
				conn.close();
				conn = null;
				return true;
			}
		}
		return false;
	}
	
}
