package org.cn3.morpho.domain;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

/**
 * 
 * @author gillette
 *
 * @param <T>
 * @param <O>
 * 
 * The parent class for various types of trees, including SwcTreeNode and BinaryTreeNode
 */
public abstract class AbstractTreeNode<T extends AbstractTreeNode<T,O>,O> implements TreeNode<T,O>, Comparable<T> {

//	protected T node;
    protected O data;
    //protected List children;
    protected List<T> children;
    protected T parent;
    protected long branchOrder;

    protected int branchSize;
    protected int height;
    protected int totalHeight;
    protected float averageHeight;
    protected int degree;

    public AbstractTreeNode(){
        data = null;
        parent = null;
        children = new ArrayList<T>();
        branchSize = 1;
    }
    public AbstractTreeNode(O d){
        data = d;
        parent = null;
        children = new ArrayList<T>();
        branchSize = 1;
    }
    public void setParent(T p){
    	setParent(p, true);
    }
    public void setParent(T p, boolean addChild){
        parent = p;
        /*
        if (p != null && addChild){
        	if (!p.getChildren().contains(node)){
        		p.addChildNode(node, false);
        		p.updateBranchSize();
        	}
        }
        */
    }

    public void addChildNode(T n){
    	children.add(n);
    }
    /*
    public void addChildNode(T n){
    	addChildNode(n, true);
    }
    public void addChildNode(T n, boolean setParent){
    	if (setParent){
    		n.setParent(node, false);
    	}
    	childrenMap.put(n.getData(), n);
    	children.add(n);
    	StandardTreeNode.updateBranchStatisticsUp(this);
    }
    */
    
    public void setData(O d) {
        data = d;
    }
    public O getData(){
        return data;
    }
    
    public List<T> getChildren(){
        return children;
    }
    public void setChildren(List<T> children){
        this.children = children;
    }

    public String toString(){
        return ""+data;
    }
    
    public int getBranchSize(){
        return branchSize;
    }
    
    public int getBranchSizeRecursively(){
    	int size = 1;
    	
        for (T child : children){
     	   size += child.getBranchSizeRecursively();
        }
    	return size;
    }

    public int getBranchSizeIteratively(){
    	int size = 1;
    	if (this.hasChildren()){
	    	Stack<T> stack = new Stack<T>();
	    	stack.push(this.getChildren().get(0).getParent());
	    	T node;
	    	while (!stack.isEmpty()){
	    		node = stack.pop();
	    		size++;
	    		if (node.hasChildren()){
			        for (T child : node.getChildren()){
			     	   stack.push(child);
			        }
	    		}
	    	}
    	}
    	return size;
    }

    public void updateBranchSize(){
       branchSize = 1;
       for (T node : children){
    	   branchSize += node.getBranchSize();
       }
       
       if (parent != null){
          parent.updateBranchSize();
       }
    }

    public boolean hasChildren(){
       if (children == null || children.size() == 0){
          return false;
       }
       else{
          return true;
       }
    }

    public void detach(){
       T p = parent;
       if (p != null){
	       p.getChildren().remove(this);
//    	   StandardTreeNode.updateBranchStatisticsUp(p);
	       parent = null;
       }
    }

    public String toStringLargeThenSmall(){
       String str = data.toString();
       List<T> list = children;
       while (!list.isEmpty()){
    	   int largest = 0;
    	   int size = 0;
	       for (int i = 0; i < list.size(); i++){
	    	   if (list.get(i).getBranchSize() > size){
	    		   size = list.get(i).getBranchSize();
	    		   largest = i;
	    	   }
	       }
	       str.concat(list.get(largest).toStringLargeThenSmall());
	       list.remove(largest);
       }
       return str;
    }

    public String toStringSmallThenLarge(){
        String str = data.toString();
        List<T> list = children;
        while (!list.isEmpty()){
     	   int smallest = 0;
     	   int size = 10000;
 	       for (int i = 0; i < list.size(); i++){
 	    	   if (list.get(i).getBranchSize() < size){
 	    		   size = list.get(i).getBranchSize();
 	    		   smallest = i;
 	    	   }
 	       }
 	       str.concat(list.get(smallest).toStringSmallThenLarge());
 	       list.remove(smallest);
        }
        return str;
    }

    public int compareTo(T node){
    	if (branchSize != node.getBranchSize()){
    		return branchSize - node.getBranchSize();	
    	}
    	else if (height != node.getHeight()){
    		return height - node.getHeight();
    	}
    	else if (averageHeight != node.getAverageHeight()){
    		return (Math.round(averageHeight - node.getAverageHeight()));
    	}
    	else {
    		return degree - node.getDegree();
    	}
    }
    public float getAverageHeight() {
		return averageHeight;
	}
	protected void setAverageHeight(float averageHeight) {
		this.averageHeight = averageHeight;
	}
	public int getHeight() {
		return height;
	}
	protected void setHeight(int height) {
		this.height = height;
	}
	public int getDegree() {
		return degree;
	}
	protected void setDegree(int degree) {
		this.degree = degree;
	}
	public long getBranchOrder() {
		return branchOrder;
	}
	public void setBranchOrder(long branchOrder) {
		this.branchOrder = branchOrder;
	}
	public int getTotalHeight() {
		return totalHeight;
	}
	protected void setTotalHeight(int totalHeight) {
		this.totalHeight = totalHeight;
	}
	public T getParent() {
		return parent;
	}
	public boolean hasParent(){
		return parent != null;
	}
	protected void setBranchSize(int branchSize) {
		this.branchSize = branchSize;
	}
	public boolean isLeaf(){
		return children == null || children.isEmpty();
	}

	
	public static void updateStatisticsDown(AbstractTreeNode<?,?> node){
    	AbstractTreeNode<?,?> current;
    	Stack<AbstractTreeNode<?,?>> assembleStack = new Stack<AbstractTreeNode<?,?>>();
    	Stack<AbstractTreeNode<?,?>> branchStack = new Stack<AbstractTreeNode<?,?>>();
    	int branchSize, maxHeight, totalHeight, degree;
    	assembleStack.push(node);
    	branchStack.push(node);
    	while (!assembleStack.isEmpty()){
    		current = assembleStack.pop();
    		if (current.hasParent()){
    			current.setBranchOrder(current.getParent().getBranchOrder()+1);
    		}else{
    			current.setBranchOrder(0);
    		}
    		for (AbstractTreeNode<?,?> child : current.getChildren()){
    			assembleStack.push(child);
    			branchStack.push(child);
    		}
    	}
    	assembleStack = null;
    	while (!branchStack.isEmpty()){
    		current = branchStack.pop();
    		branchSize = 1;
    		degree = totalHeight = maxHeight = 0;
    		if (current.hasChildren()){
	    		for (AbstractTreeNode<?,?> child : current.getChildren()){
	    			branchSize += child.getBranchSize();
	    			degree += child.getDegree();
	    			maxHeight = child.getHeight() + 1;
	    			// totalLength is the total length of the children plus 1 for each child below (branchSize)
	    			totalHeight += child.getTotalHeight()
	    				+ child.getDegree();
	    		}
    		}
    		else{
    			degree++;
    		}
    		current.setBranchSize(branchSize);
    		current.setDegree(degree);
    		current.setHeight(maxHeight);
    		current.setTotalHeight(totalHeight);
    		// branchSize-1 so this node is not included
    		current.setAverageHeight((float)totalHeight/degree);
    	}
    	branchStack = null;
    }

    public static void updateBranchStatisticsUp(AbstractTreeNode<?,?> node){
    	AbstractTreeNode<?,?> current = node;
    	int branchSize, maxHeight, totalHeight, degree;
    	while (current != null){
    		branchSize = 1;
    		degree = totalHeight = maxHeight = 0;
    		if (current.hasChildren()){
	    		for (AbstractTreeNode<?,?> child : current.getChildren()){
	    			branchSize += child.getBranchSize();
	    			degree += child.getDegree();
	    			maxHeight = child.getHeight() + 1;
	    			// totalLength is the total length of the children plus 1 for each child below (branchSize)
	    			totalHeight += child.getTotalHeight()
	    				+ child.getDegree();
	    		}
    		}
    		else{
    			degree++;
    		}
    		current.setBranchSize(branchSize);
    		current.setDegree(degree);
    		current.setHeight(maxHeight);
    		current.setTotalHeight(totalHeight);
    		// branchSize-1 so this node is not included
    		current.setAverageHeight((float)totalHeight/degree);
    		
   			current = current.getParent();
    	}
    }
    
}
