package org.cn3.morpho.domain;

import java.util.Arrays;
import java.util.List;

/**
 * 
 * @author gillette
 * Contains data about the neuron as well as various potential types of processes.
 * Dendrites of non-pyramidal cells can be stored as "basal" trees.
 *
 */
public class Soma<T extends AbstractTreeNode<T,?>> {

	private NeuronMetadata neuron;
	private NeuronProcess<T> somaRoot;
	private NeuronProcess<T> axonalTree;
	// Used for non-apical trees. 
	// Multiple dendrites may be combined with artificial nodes into one dendritic tree
	private NeuronProcess<T> basalDendriteTree;
	private NeuronProcess<T> apicalDendriteTree;
	private int numDendriteStems;
	
	public static enum ArborType{
		Axon("Axon","axon"), 
		ApicalDendrite("ApicalDendrite","Apical","apical"), 
		Dendrite("Dendrite","BasalDendrite","dendrite","basal","Basal");
		
		private final List<String> names;
		ArborType(String ...names){
			this.names = Arrays.asList(names);
		}
		public List<String> getNames() {
	        return names;
	    }
		
		public static ArborType find(String name) {
		    for (ArborType arborType : ArborType.values()) {
		        if (arborType.getNames().contains(name)) {
		            return arborType;
		        }
		    }
		    return null;
		}
	}
	
	/**
	 * List objects are for when more than one tree that exists for a dendrite
	 * type should be represented as separate.
	 **/
	private List<T> basalDendriteTrees;
	private List<T> apicalDendriteTrees;
	
	public NeuronProcess<T> getSomaRoot() {
		return somaRoot;
	}
	public void setSomaRoot(NeuronProcess<T> somaRoot) {
		this.somaRoot = somaRoot;
	}
	public NeuronProcess<T> getApicalDendriteTree() {
		return apicalDendriteTree;
	}
	public boolean hasApicalDendriteTree(){
		return apicalDendriteTree != null && apicalDendriteTree.getTreeHead() != null;
	}
	public boolean hasApicalDendriteTrees(){
		return apicalDendriteTrees != null && apicalDendriteTrees.size() > 0;
	}
	public void setApicalDendriteTree(NeuronProcess<T> apicalDendriteTree) {
		this.apicalDendriteTree = apicalDendriteTree;
	}
	public NeuronProcess<T> getAxonalTree() {
		return axonalTree;
	}
	public boolean hasAxonalTree(){
		return axonalTree != null && axonalTree.getTreeHead() != null;
	}
	public void setAxonalTree(NeuronProcess<T> axonalTree) {
		this.axonalTree = axonalTree;
	}
	public NeuronProcess<T> getBasalDendriteTree() {
		return basalDendriteTree;
	}
	public BinaryTreeNode getBasalDendriteTree(int index) {
		if (basalDendriteTrees != null && basalDendriteTrees.size() > index){
			return (BinaryTreeNode)basalDendriteTrees.get(index);
		}
		else{
			return null;
		}
	}
	public boolean hasBasalDendriteTree(){
		return basalDendriteTree != null && basalDendriteTree.getTreeHead() != null;
	}
	public boolean hasBasalDendriteTrees(){
		return basalDendriteTrees != null && basalDendriteTrees.size() > 0;
	}
	public void setBasalDendriteTree(NeuronProcess<T> basalDendriteTree) {
		this.basalDendriteTree = basalDendriteTree;
	}
	public List<T> getApicalDendriteTrees() {
		return apicalDendriteTrees;
	}
	public void setApicalDendriteTrees(List<T> apicalDendriteTrees) {
		this.apicalDendriteTrees = apicalDendriteTrees;
	}
	public List<T> getBasalDendriteTrees() {
		return basalDendriteTrees;
	}
	public void setBasalDendriteTrees(List<T> basalDendriteTrees) {
		this.basalDendriteTrees = basalDendriteTrees;
	}
	public NeuronMetadata getNeuron() {
		return neuron;
	}
	public void setNeuron(NeuronMetadata neuron) {
		this.neuron = neuron;
	}
	public int getNumDendriteStems() {
		return numDendriteStems;
	}
	public void setNumDendriteStems(int numDendriteStems) {
		this.numDendriteStems = numDendriteStems;
	}
	
}
