package org.cn3.morpho.domain;

public class SwcDataNode implements Comparable<SwcDataNode> {

	public static final int TYPE_SOMA = 1;
	public static final int TYPE_AXON = 2;
	public static final int TYPE_DENDRITE = 3;
	public static final int TYPE_APICAL = 4;
	public static final int TYPE_UNKNOWN = 8;
	
	public static int getTypeFromString(String string){
		if (string.equalsIgnoreCase("Axon")){
			return TYPE_AXON;
		}
		else if (string.equalsIgnoreCase("ApicalDendrite")){
			return TYPE_APICAL;
		}
		else if (string.equalsIgnoreCase("Dendrite") || string.equalsIgnoreCase("BasalDendrite")){
			return TYPE_DENDRITE;
		}
		else if (string.equalsIgnoreCase("Soma")){
			return TYPE_SOMA;
		}
		else{
			return TYPE_UNKNOWN;
		}
	}
	
	private int nodeId;
	private int parentId;
	private int type;
	private double x;
	private double y;
	private double z;
	private double radius;
	
	private SwcSecondaryData secondaryData;

	public SwcDataNode(){}
	public SwcDataNode(int nodeId){
		this.nodeId = nodeId;
	}
	public SwcDataNode(int nodeId, int parentId){
		this.nodeId = nodeId;
		this.parentId = parentId;
	}
	public SwcDataNode(int nodeId, int parentId, int type){
		this.nodeId = nodeId;
		this.parentId = parentId;
		this.type = type;
	}
	public SwcDataNode(double x, double y, double z){
		this.x = x;
		this.y = y;
		this.z = z;
	}
	
	public int getNodeId() {
		return nodeId;
	}
	public void setNodeId(int nodeId) {
		this.nodeId = nodeId;
	}
	public int getParentId() {
		return parentId;
	}
	public void setParentId(int parentId) {
		this.parentId = parentId;
	}
	public int getType() {
		return type;
	}
	public void setType(int type) {
		this.type = type;
	}
	
	public String toString(){
		return ("SwcDataNode [nodeId: "+nodeId+"; parentId: "+parentId+"; type: "+type+"]");
	}
	public String positionString(){
		return ("(X: "+x+"; Y: "+y+"; Z: "+z+")");
	}
	public double getX() {
		return x;
	}
	public void setX(double x) {
		this.x = x;
	}
	public double getY() {
		return y;
	}
	public void setY(double y) {
		this.y = y;
	}
	public double getZ() {
		return z;
	}
	public void setZ(double z) {
		this.z = z;
	}
	public SwcSecondaryData getSecondaryData() {
		return secondaryData;
	}
	public void setSecondaryData(SwcSecondaryData secondaryData) {
		this.secondaryData = secondaryData;
	}
	public double getDiameter() {
		return radius*2;
	}
	public double getRadius() {
		return radius;
	}
	public double getRadiusSquared() {
		return radius*radius;
	}
	public void setRadius(double radius) {
		this.radius = radius;
	}
	
	public boolean equals(SwcDataNode data){
		return (data.getNodeId() == this.getNodeId());
	}

	public int compareTo(SwcDataNode node){
		//return nodeId - node.getNodeId();
		return (nodeId > node.getNodeId() ? 1 : nodeId == node.getNodeId() ? 0 : -1);
	}
	
	public SwcDataNode copy(){
		SwcDataNode copy = new SwcDataNode();
		copy.setNodeId(nodeId);
		copy.setParentId(parentId);
		copy.setX(x);
		copy.setY(y);
		copy.setZ(z);
		copy.setRadius(radius);
		if (secondaryData != null){
			copy.setSecondaryData(secondaryData.copy());	
		}
		copy.setType(type);
		return copy;
	}

}
