package org.cn3.morpho.domain;

import java.util.HashMap;
import java.util.Map;
import java.util.Stack;


//public class SwcTreeNode extends StandardTreeNode {
public class SwcTreeNode extends AbstractTreeNode<SwcTreeNode,SwcDataNode> {

//	private SwcDataNode swcData;
//	protected List children;
//	protected List<SwcTreeNode> children;
//	protected StandardTreeNode parent;
	
    protected long level;

    protected int branchSize;
    protected int height;
    protected int totalHeight;
    protected float averageHeight;
    protected int degree;
	
    public SwcTreeNode(){
    	this.data = new SwcDataNode();
    }
	public SwcTreeNode(SwcDataNode data){
		this.data = data;
	}
	public SwcTreeNode(int nodeId, int parentId){
		data = new SwcDataNode();
		data.setNodeId(nodeId);
		data.setParentId(parentId);
	}
	public SwcTreeNode(int nodeId, int parentId, double x, double y, double z){
		data = new SwcDataNode();
		data.setNodeId(nodeId);
		data.setParentId(parentId);
		data.setX(x);
		data.setY(y);
		data.setZ(z);
	}
	public SwcTreeNode(double x, double y, double z){
		data = new SwcDataNode();
		data.setX(x);
		data.setY(y);
		data.setZ(z);
	}
	public SwcTreeNode(double x, double y, double z, double radius){
		data = new SwcDataNode();
		data.setX(x);
		data.setY(y);
		data.setZ(z);
		data.setRadius(radius);
	}
	public SwcTreeNode(double x, double y, double z, double radius, int type){
		data = new SwcDataNode();
		data.setX(x);
		data.setY(y);
		data.setZ(z);
		data.setRadius(radius);
		data.setType(type);
	}

    public void setParent(SwcTreeNode p){
    	setParent(p, true);
    }
    public void setParent(SwcTreeNode p, boolean addChild){
    	if (parent != null) parent.children.remove(this);
        parent = p;
        if (p != null && addChild){
        	if (!p.getChildren().contains(this)){
        		p.addChildNode(this, false);
        		//p.updateBranchSize();
        	}
        }
    }
/*
    public void addChildNode(T n){
    	children.add(n);
    }
  */  
    public void addChildNode(SwcTreeNode n){
    	addChildNode(n, true);
    }
    public void addChildNode(SwcTreeNode n, boolean setParent){
    	if (setParent){
    		n.setParent(this, false);
    	}
    	//childrenMap.put(n.getData(), n);
    	children.add(n);
    	//AbstractTreeNode.updateBranchStatisticsUp(this);
    }
    
	public int getNodeId() {
		return data.getNodeId();
	}
	public void setNodeId(int nodeId) {
		data.setNodeId(nodeId);
	}
	public int getParentId() {
		return data.getParentId();
	}
	public void setParentId(int parentId) {
		data.setParentId(parentId);
	}
	public int getType() {
		return data.getType();
	}
	public void setType(int type) {
		data.setType(type);
	}
	public int getDegree() {
		return degree;
	}
	protected void setDegree(int degree) {
		this.degree = degree;
	}
	
	public String toString(){
		return ("SwcTreeNode [nodeId: "+getNodeId()+"; parentId: "+getParentId()+"; type: "+getType()+"]");
	}
	public double getX() {
		return data.getX();
	}
	public void setX(double x) {
		data.setX(x);
	}
	public double getY() {
		return data.getY();
	}
	public void setY(double y) {
		data.setY(y);
	}
	public double getZ() {
		return data.getZ();
	}
	public void setZ(double z) {
		data.setZ(z);
	}
	public SwcSecondaryData getSecondaryData() {
		return data.getSecondaryData();
	}
	public void setSecondaryData(SwcSecondaryData secondaryData) {
		data.setSecondaryData(secondaryData);
	}
	public double getDiameter() {
		return data.getDiameter();
	}
	public double getRadius() {
		return data.getRadius();
	}
	public double getRadiusSquared() {
		return data.getRadiusSquared();
	}
	public void setRadius(double radius) {
		data.setRadius(radius);
	}
	
	public void reorderNodeIds(){
		reorderNodeIds(1);
	}

	/**
	 *  For multiple independent trees or for recursion (not currently used)
	 * @param initialId
	 * @return nextId (in the event that another tree is grouped with this one)
	 */
	public int reorderNodeIds(int initialId){
		setNodeId(initialId);
		int currId = initialId;
		Stack<SwcTreeNode> nodeStack = new Stack<SwcTreeNode>();
		nodeStack.push(this);
		SwcTreeNode node;
		do{
			node = nodeStack.pop();
			node.setNodeId(currId);
			if (children.size() > 0){
				for (SwcTreeNode child : node.getChildren()){
					child.setParentId(currId);
					nodeStack.push(child);
				}
			}
			currId++;
		} while(!nodeStack.isEmpty());
		return currId;
	}
	
	public boolean equals(Object o){
		if (o != null && o.getClass().equals(this.getClass())){
			SwcTreeNode node = (SwcTreeNode)o;
			return (node.getNodeId() == this.getNodeId());
		}
		return false;
	}

	public int compareTo(SwcTreeNode node){
		return getNodeId() - node.getNodeId();
	}

	public SwcTreeNode getChild(int index){
		return children.get(index);
	}
	
	public SwcTreeNode copy(){
		Stack<SwcTreeNode> stack = new Stack<SwcTreeNode>();
		Map<SwcTreeNode,SwcTreeNode> nodeMap = new HashMap<SwcTreeNode,SwcTreeNode>();
		SwcTreeNode node, nodeCopy, childCopy, rootCopy = new SwcTreeNode(getData().copy());
		stack.push(this);
		nodeMap.put(this, rootCopy);
		while (!stack.isEmpty()){
			node = stack.pop();
			nodeCopy = nodeMap.remove(node);
			if (node.hasChildren()){
				for (SwcTreeNode child : node.getChildren()){
					childCopy = new SwcTreeNode(child.getData().copy());
					nodeCopy.addChildNode(childCopy);
					stack.push(child);
					nodeMap.put(child, childCopy);
				}
			}
		}

		return rootCopy;
	}
	/*
	public static void updateStatisticsDown(SwcDataNode node){
    	SwcDataNode current;
    	LinkedStack<SwcDataNode> assembleStack = new LinkedStack<SwcDataNode>();
    	LinkedStack<SwcDataNode> branchStack = new LinkedStack<SwcDataNode>();
    	int branchSize, maxHeight, totalHeight, degree;
    	assembleStack.push(node);
    	branchStack.push(node);
    	while (!assembleStack.isEmpty()){
    		current = assembleStack.pop();
    		for (StandardTreeNode<SwcDataNode> child : current.getChildren()){
    			assembleStack.push(child);
    			branchStack.push(child);
    		}
    	}
    	assembleStack = null;
    	while (!branchStack.isEmpty()){
    		current = branchStack.pop();
    		branchSize = 1;
    		degree = totalHeight = maxHeight = 0;
    		if (current.hasChildren()){
	    		for (StandardTreeNode<SwcDataNode> child : current.getChildren()){
	    			branchSize += child.getBranchSize();
	    			degree += child.getDegree();
	    			maxHeight = child.getHeight() + 1;
	    			// totalLength is the total length of the children plus 1 for each child below (branchSize)
	    			totalHeight += child.getTotalHeight()
	    				+ child.getDegree();
	    		}
    		}
    		else{
    			degree++;
    		}
    		current.setBranchSize(branchSize);
    		current.setDegree(degree);
    		current.setHeight(maxHeight);
    		current.setTotalHeight(totalHeight);
    		// branchSize-1 so this node is not included
    		current.setAverageHeight((float)totalHeight/degree);
    	}
    	branchStack = null;
    }

    public static void updateBranchStatisticsUp(StandardTreeNode node){
    	StandardTreeNode current = node, child;
    	int branchSize, maxHeight, totalHeight, degree;
    	while (current != null){
    		branchSize = 1;
    		degree = totalHeight = maxHeight = 0;
    		if (current.hasChildren()){
	    		for (Iterator<StandardTreeNode> it = current.getChildren().iterator(); it.hasNext();){
	    			child = (StandardTreeNode)it.next();
	
	    			branchSize += child.getBranchSize();
	    			degree += child.getDegree();
	    			maxHeight = child.getHeight() + 1;
	    			// totalLength is the total length of the children plus 1 for each child below (branchSize)
	    			totalHeight += child.getTotalHeight()
	    				+ child.getDegree();
	    		}
    		}
    		else{
    			degree++;
    		}
    		current.setBranchSize(branchSize);
    		current.setDegree(degree);
    		current.setHeight(maxHeight);
    		current.setTotalHeight(totalHeight);
    		// branchSize-1 so this node is not included
    		current.setAverageHeight((float)totalHeight/degree);
    		
   			current = (StandardTreeNode)current.getParent();
    	}
    }
*/
	
	/**
	 * Returns this node's nearest ancestor that has more than one child or has no
	 * parent, whichever comes first; null if current node is root
	 * @return SwcTreeNode
	 */
	public SwcTreeNode getParentNode(){
		SwcTreeNode node = this.getParent();
		while (node != null && node.getChildren().size() == 1){
			node = node.getParent();
		}
		return node;
	}

	/**
	 * Returns this node's nearest child that has more than one child or has no
	 * children, whichever comes first (returns self if self has multiple children)
	 * @return SwcTreeNode
	 */
	public SwcTreeNode getChildNode(){
		SwcTreeNode node = this;
		while (node.getChildren().size() == 1){
			node = node.getChild(0);
		}
		return node;
	}
}
