package org.cn3.morpho.resources;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Map;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cn3.morpho.domain.NeuronMetadata;


public class MorphoProjectProperties {

	public static final String DATASOURCE_DB = "db";
	public static final String DATASOURCE_FILE = "file";
	public static final String DATABASE_SERVER = "database.server";
	public static final String DATABASE_USERNAME = "database.username";
	public static final String DATABASE_PASSWORD = "database.password";
	
	private static MorphoProjectProperties instance = null;
	private static Properties projectProperties = null;

	public static MorphoProjectProperties getInstance() throws IOException{
		if (instance == null){
			instance = new MorphoProjectProperties();
		}
		return instance;
	}
	
	private MorphoProjectProperties() throws IOException{
		projectProperties = new Properties();
		projectProperties.load(new FileInputStream("resources/conf/project.properties"));
	}
	
	public Properties getBlastMorphoProperties() throws Exception{
		return projectProperties;
	}
	
	public String getWorkingDir() throws IOException{
		return projectProperties.getProperty(MorphoConstants.WORKING_LOC);
	}
	public String getResourceDirectory() throws IOException{
		return getWorkingDir()+"/"+projectProperties.getProperty(MorphoConstants.RESOURCES_LOC);
	}
	public String getDataSource() throws IOException{
		return projectProperties.getProperty(MorphoConstants.DATASOURCE);
	}
	public String getDatabaseServer() throws IOException{
		return projectProperties.getProperty(DATABASE_SERVER);
	}
	public String getDatabaseUsername() throws IOException{
		return projectProperties.getProperty(DATABASE_USERNAME);
	}
	public String getDatabasePassword() throws IOException{
		return projectProperties.getProperty(DATABASE_PASSWORD);
	}
	public NeuronMetadata getNeuronFromSequence(Map<String,NeuronMetadata> cellTypeMap, String sequence){
		Pattern p = Pattern.compile("(.*)\\-(Dendrite|BasalDendrite|ApicalDendrite|Axon)");
		Matcher m = p.matcher(sequence);
		if (!m.matches()){
			System.out.println("no match");
			return null;
		}
		return (NeuronMetadata)cellTypeMap.get(m.group(1));
	}
	
}
