package org.cn3.morpho.scripts;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.text.NumberFormat;
import java.util.Stack;
import java.util.regex.Matcher;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.SWCReader;
import org.cn3.morpho.domain.SwcTreeNode;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 * Rewrites the swc file with all x, y, and z values flipped (newVal = -origVal).
 * 
 * java org.cn3.morpho.scripts.SwcNeuronFlipper -i "C:/Documents and Settings/gillette/My Documents/MentalFlossVirtualModel/SWC Files/DG gc axon.swc" -o "C:/Documents and Settings/gillette/My Documents/MentalFlossVirtualModel/SWC Files/gc-flipped.swc"
 * 
 */
public class SwcNeuronFlipper {

	public static void mirrorAndWrite(File swcFile, File outputFile) throws Exception{
		mirrorAndWrite(swcFile, outputFile, true, true, true);
	}
	public static void mirrorAndWrite(File swcFile, File outputFile, boolean x, boolean y, boolean z) throws Exception{
		BufferedReader reader = new BufferedReader(new FileReader(swcFile));

		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(2);
		nf.setGroupingUsed(false);
		
		String dataString = "";
		SwcTreeNode node;
		Matcher m;
		int type, nodeId, parentId;
		
		StringBuffer sb = new StringBuffer();
		
		// 1. Assembles axon, basal trees, and apical trees (non-binary).
		while ( (dataString=reader.readLine()) != null ) { 
			if (dataString.startsWith("#")){
				sb.append(dataString+"\n");
			}
			else{
				m = SWCReader.swcDataPattern.matcher(dataString);
				//System.out.println(dataString);
				
		        if (m.matches()){
		        	node = new SwcTreeNode();
		        	nodeId = Integer.parseInt(m.group(1));
		        	node.setNodeId(nodeId);
		        	type = Integer.parseInt(m.group(2));
		        	node.setType(type);
		        	node.setX(Double.parseDouble(m.group(3)));
		        	node.setY(Double.parseDouble(m.group(4)));
		        	node.setZ(Double.parseDouble(m.group(5)));
		        	node.setRadius(Double.parseDouble(m.group(6)));
		        	parentId = Integer.parseInt(m.group(7));
		        	node.setParentId(parentId);
		        	
		        	if (x){
		        		node.setX(-node.getX());
		        	}
		        	if (y){
		        		node.setY(-node.getY());
		        	}
		        	if (z){
		        		node.setZ(-node.getZ());
		        	}
		
					sb.append(node.getNodeId()+" "+node.getType() 
							+ " "+nf.format(node.getX())+" "+nf.format(node.getY())+" "+nf.format(node.getZ())
							+ " "+node.getRadius()+" "+node.getParentId()+"\n");
		        }
		        else{
		        	System.out.println(dataString);
		        }
	        }
		}
		DataWriter.writeStringToFile(outputFile, sb.toString());
	}
	
	public static void mirror(SwcTreeNode root, boolean x, boolean y, boolean z){
		Stack<SwcTreeNode> stack = new Stack<SwcTreeNode>();
		stack.push(root);
		SwcTreeNode node;
		while (!stack.isEmpty()){
			node = stack.pop();
			if (x){
				node.setX(-node.getX());
			}
			if (y){
				node.setY(-node.getY());
			}
			if (z){
				node.setZ(-node.getZ());
			}
			if (node.hasChildren()){
				for (int i = 0; i < node.getChildren().size(); i++){
					stack.push((SwcTreeNode)node.getChildren().get(i));
				}
			}
		}
	}
	
	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);
			String filename = config.getString("input");
			String outFilename = config.getString("output");
			
			File inputFile = new File(filename), outputFile = new File(outFilename);
			if (inputFile.isDirectory()){
				if (!outputFile.isDirectory()){
					outputFile = new File(inputFile.getAbsolutePath()+"/AxesFlipped/");
					if (!outputFile.exists()){
						outputFile.mkdir();
					}
				}
				File[] files = inputFile.listFiles();
				for (int i = 0; i < files.length; i++){
					inputFile = files[i];
					if (!inputFile.isDirectory()&&
							(inputFile.getName().endsWith(".swc")||inputFile.getName().endsWith(".SWC"))){
						mirrorAndWrite(inputFile, new File(outputFile.getAbsolutePath()+"/"+inputFile.getName()));
					}
				}
			}
			else{
				mirrorAndWrite(inputFile, outputFile);
			}
			
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;
        
        opt = new FlaggedOption("input")
            .setStringParser(JSAP.STRING_PARSER)
            .setRequired(true)
            .setShortFlag('i') 
            .setLongFlag("input");
        jsap.registerParameter(opt);

        opt = new FlaggedOption("output")
	        .setStringParser(JSAP.STRING_PARSER)
	        .setRequired(true)
	        .setShortFlag('o') 
	        .setLongFlag("output");
	    jsap.registerParameter(opt);

        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (@SuppressWarnings("rawtypes")
			java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + SwcNeuronFlipper.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}

	
}