package org.cn3.morpho.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class StringUtils {

	public static String getNeuronNameFromSwcFile(String swcFilename){
		Matcher m = Pattern.compile("(.*)\\.CNG?\\.swc").matcher(swcFilename);
		if (m.matches()){
			return m.group(1);
		}
		else{
			m = Pattern.compile("(.*)\\.swc").matcher(swcFilename);
			if (m.matches()){
				return m.group(1);
			}
		}
		return null;
	}

	public static String insertBreaksAtLength(String string, int chars){
		return insertBreaksAtLength(string, chars, "\n");
	}

	public static String insertBreaksAtLength(String string, int chars, String breakString){
		StringBuffer sb = new StringBuffer();
		int end;
		for (int i = 0; i < string.length(); i+=chars){
			end = i + chars;
			if (end > string.length()){
				end = string.length();
			}
			if (i > 0){
				sb.append(breakString);
			}
			sb.append(string.subSequence(i, end));
		}
		return sb.toString();
	}

	public static boolean isEmpty(String string){
		return (string == null || string.equals(""));
	}
	
	public static void printList(List<Object> list){
		for (int i = 0; i < list.size(); i++){
			System.out.println(i+": "+list.get(i));
		}
	}

	private static Pattern arborP 
		= Pattern.compile("(.*)\\-(ApicalDendrite|BasalDendrite|Dendrite|Axon)");
	public static String getNeuronNameFromSequence(String sequenceName){
		Matcher m = arborP.matcher(sequenceName);
		if (m.matches()){
			return m.group(1);
		}
		return null;
	}

	public static String getArborizationType(String sequenceName){
		Matcher m = arborP.matcher(sequenceName);
		if (m.matches()){
			return m.group(2);
		}
		return null;
	}

	public static String nullAsEmpty(String str){
		return (str == null ? "" : str);
	}
	
	public static String insertLeadingSpaces(int maxLength, int number){
		String numStr = String.valueOf(number);
		int zeros = maxLength - numStr.length();
		for (int i = 0; i < zeros; i++){
			numStr = " "+numStr;
		}
		return numStr;
	}

	public static String insertLeadingZeros(int maxLength, int number){
		String numStr = String.valueOf(number);
		int zeros = maxLength - numStr.length();
		for (int i = 0; i < zeros; i++){
			numStr = "0"+numStr;
		}
		return numStr;
	}

	public static List<String> intersection(List<String> list1, List<String> list2){
		return new ArrayList<String>(
				intersectionSub(new TreeSet<String>(list1),new TreeSet<String>(list2)));
	}
	public static Set<String> intersection(Set<String> set1, Set<String> set2){
		return intersectionSub(new TreeSet<String>(set1),new TreeSet<String>(set2));
	}
	private static Set<String> intersectionSub(Set<String> set1, Set<String> set2){
		for(String str : set1){
		    if(!set2.remove(str))
		    	set1.remove(str);
		}
		return set1;
	}

	public static List<String> union(List<String> list1, List<String> list2){
		TreeSet<String> newSet = new TreeSet<String>(list1);
		newSet.addAll(list2);
		return new ArrayList<String>(newSet);
	}
	public static Set<String> union(Set<String> set1, Set<String> set2){
		TreeSet<String> newSet = new TreeSet<String>(set1);
		newSet.addAll(set2);
		return newSet;
	}
	
}
