package org.cn3.morpho.utils;

import java.util.List;
import java.util.Stack;

import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.SwcDataNode;
import org.cn3.morpho.domain.SwcTreeNode;

public class SwcDataUtils {

	public static double getDistance(SwcDataNode node1, SwcDataNode node2){
		double xDist = node1.getX() - node2.getX();
		double yDist = node1.getY() - node2.getY();
		double zDist = node1.getZ() - node2.getZ();
		
		return Math.sqrt(xDist*xDist + yDist*yDist + zDist*zDist);
	}
	
	public static double getAngle(SwcDataNode parent, SwcDataNode node1, SwcDataNode node2){
		SwcDataNode v1 = getVector(parent, node1), v2 = getVector(parent, node2);
		normalizeVector(v1);
		normalizeVector(v2);
		return Math.acos(dotProduct(v1,v2));
	}
	
	public static SwcDataNode getVector(SwcDataNode base, SwcDataNode end){
		SwcDataNode v = new SwcDataNode();
		v.setX(end.getX() - base.getX());
		v.setY(end.getY() - base.getY());
		v.setZ(end.getZ() - base.getZ());
		return v;
	}
	
	public static void normalizeVector(SwcDataNode v){
		double length = Math.sqrt(v.getX()*v.getX() + v.getY()*v.getY() + v.getZ()*v.getZ());
		v.setX(v.getX() / length);
		v.setY(v.getY() / length);
		v.setZ(v.getZ() / length);
	}
	
	public static double dotProduct(SwcDataNode v1, SwcDataNode v2){
		return v1.getX()*v2.getX() + v1.getY()*v2.getY() + v1.getZ()*v2.getZ();
	}

	public static SwcDataNode crossProduct(SwcDataNode middle, SwcDataNode point1, SwcDataNode point2){
		SwcDataNode v1 = getVector(middle,point1);
		SwcDataNode v2 = getVector(middle,point2);
		return crossProduct(v1,v2);
	}

	public static SwcDataNode crossProduct(SwcDataNode v1, SwcDataNode v2){
		SwcDataNode cp = new SwcDataNode();
		cp.setX(v1.getY()*v2.getZ() - v1.getZ()*v2.getY());
		cp.setY(v1.getZ()*v2.getX() - v1.getX()*v2.getZ());
		cp.setZ(v1.getX()*v2.getY() - v1.getY()*v2.getX());
		return cp;
	}
	
	/**
	 *  For multiple independent trees or for recursion (not currently used)
	 * @param initialId
	 * @return nextId (in the event that another tree is grouped with this one)
	 */
	public static int reorderNodeIds(SwcTreeNode root, int initialId){
		root.setNodeId(initialId);
		int currId = initialId;
		Stack<SwcTreeNode> nodeStack = new Stack<SwcTreeNode>();
		nodeStack.push(root);
		SwcTreeNode node, swcChild;
		do{
			node = nodeStack.pop();
			node.setNodeId(currId);
			if (node.getChildren().size() > 0){
				for (SwcTreeNode child : node.getChildren()){
					swcChild = (SwcTreeNode)child;
					swcChild.setParentId(currId);
					nodeStack.push(swcChild);
				}
			}
			currId++;
		} while(!nodeStack.isEmpty());
		return currId;
	}
	
	public static SwcDataNode getPointAlongVector(SwcDataNode node, SwcDataNode vector, double distance){
		SwcDataNode newPoint = new SwcDataNode();
		newPoint.setX(vector.getX()*distance+node.getX());
		newPoint.setY(vector.getY()*distance+node.getY());
		newPoint.setZ(vector.getZ()*distance+node.getZ());
		return newPoint;
	}

	public static SwcDataNode getBisectionPoint(SwcDataNode node1, SwcDataNode node2){
		SwcDataNode bisectionNode = new SwcDataNode();
		bisectionNode.setX(getBisectionPoint(node1.getX(),node2.getX()));
		bisectionNode.setY(getBisectionPoint(node1.getY(),node2.getY()));
		bisectionNode.setZ(getBisectionPoint(node1.getZ(),node2.getZ()));
		return bisectionNode;
	}
	public static double getBisectionPoint(double x1, double x2){
		return (x2-x1)/2 + x1;
	}
	public static SwcDataNode getBisectionVector(SwcDataNode middle, SwcDataNode node1, SwcDataNode node2){
		SwcDataNode v1 = getVector(middle, node1);
		normalizeVector(v1);
		SwcDataNode v2 = getVector(middle, node2);
		normalizeVector(v2);
		SwcDataNode p1 = getPointAlongVector(middle,v1,1);
		SwcDataNode p2 = getPointAlongVector(middle,v2,1);
		SwcDataNode bisectPoint = getBisectionPoint(p1, p2);
		SwcDataNode finalVector = getVector(bisectPoint, middle);
		normalizeVector(finalVector);
		return finalVector;
	}
	
	public static void insertSwcDataNodes(BinaryTreeNode tree){
		insertSwcDataNodes(tree,1);
	}
	public static void insertSwcDataNodes(BinaryTreeNode tree, int nodeType){
		// Ensure each node in the tree has SwcDataNode
		int i = 0;
		List<BinaryTreeNode> nodes = BinaryTreeUtils.createNodeList(tree);
		for (BinaryTreeNode node : nodes){
			node.setData(new SwcDataNode(i++));
			node.getSwcData().setType(nodeType);
		}
	}
	
}
