package org.cn3.ns.analysis;

import java.io.File;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.MetaDataManager;
import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.DistributionBean;
import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.morpho.utils.ClassificationUtils;
import org.cn3.ns.data.DataConverter;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.resources.DataProperties;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

public class AsymmetryPerClass {
	private final static String PARAM_SPECIFIC_CELL_TYPE = "specific-cell-type";
	private final static String PARAM_RANDOMS = "randoms";
	
	public static void main(String[] args){
		try{
			JSAPResult config = readParameters(args);
			
			//findRandomAsymmetry(config);
			findArborizationAsymmetry(config);
			
		} catch (Exception e){
			e.printStackTrace();
		}
		
	}
	
	private static class AsymmetryBean{
		private double averageAsymmetry;
		private double caulescence;
		private double minDegreeAsymmetry;
		//private double sizeWeightedAsymmetry;
		private double dfWeightedAsymmetry;
		public double getAverageAsymmetry() {
			return averageAsymmetry;
		}
		public void setAverageAsymmetry(double averageAsymmetry) {
			this.averageAsymmetry = averageAsymmetry;
		}
		public double getDfWeightedAsymmetry() {
			return dfWeightedAsymmetry;
		}
		public void setDfWeightedAsymmetry(double dfWeightedAsymmetry) {
			this.dfWeightedAsymmetry = dfWeightedAsymmetry;
		}
		public double getMinDegreeAsymmetry() {
			return minDegreeAsymmetry;
		}
		public void setMinDegreeAsymmetry(double minDegreeAsymmetry) {
			this.minDegreeAsymmetry = minDegreeAsymmetry;
		}
		public double getCaulescence() {
			return caulescence;
		}
		public void setCaulescence(double caulescence) {
			this.caulescence = caulescence;
		}
/*		public double getSizeWeightedAsymmetry() {
			return sizeWeightedAsymmetry;
		}
		public void setSizeWeightedAsymmetry(double sizeWeightedAsymmetry) {
			this.sizeWeightedAsymmetry = sizeWeightedAsymmetry;
		}*/
	}
	
	private static void findArborizationAsymmetry(JSAPResult config) throws Exception{

		DataProperties props = DataProperties.getInstance();
		String sequenceName;
		String dataset = "NM_SpurLen2D";
		String order = "STL";
		int mapping = NSConstants.MAPPING_MODE_TOPO_ACT;
		boolean specificCellClass = config.getBoolean(PARAM_SPECIFIC_CELL_TYPE);
		
		Map<String,Integer> lengthMap = NSDataReader.loadLengthDatabaseFromFastaFile(mapping, dataset, order);
		List<Integer> seqLengths = NSDataReader.loadUniqueSequenceLengths(lengthMap);

		Map<String,NeuronMetadata> cellMetaData = MetaDataManager.getCellMetaData();
		NeuronMetadata neuron;
		String neuronName;
		FastaReader fr = new FastaReader(DataProperties.getInstance().getFastaFilename(dataset, mapping, order));

		Map<String,List<DistributionBean>> arbTypeLocMap = new HashMap<String,List<DistributionBean>>();
		Map<String,List<DistributionBean>> cellTypeLocMap = new HashMap<String,List<DistributionBean>>();
		Map<String,List<DistributionBean>> labLocMap = new HashMap<String,List<DistributionBean>>();
		Map<String,List<DistributionBean>> cellTypeArbTypeLocMap = new HashMap<String,List<DistributionBean>>();
		Map<String,List<DistributionBean>> groupLocMap = new HashMap<String,List<DistributionBean>>();
		Map<String,List<DistributionBean>> lengthLocMap = new HashMap<String,List<DistributionBean>>();
		
		List<DistributionBean> distBeanList;
		for (int i = 0; i < seqLengths.size(); i++){
			distBeanList = new ArrayList<DistributionBean>();
			lengthLocMap.put("ArbLen"+seqLengths.get(i), distBeanList);
			lengthLocMap.put("ArbLen"+seqLengths.get(i), distBeanList);
			for (int j = 0; j < 3; j++){
				distBeanList.add(new DistributionBean());
			}
		}
		
		DistributionBean distBean = new DistributionBean();
		String arborizationType, cellTypeArbType, group;
		StringBuffer fullResults = new StringBuffer();
		BinaryTreeNode tree;
		fullResults.append("Name,Length,ArborType,AvgAsymmetry,CutoffAsymmetry,DFAsymmetry,Caulescence,MaxBranchOrder\n");
		AsymmetryBean asymBean;
		

		while (fr.next()){
			// Load sequenceName
			sequenceName = fr.getSequenceName();
			// Get Neuron data
			neuronName = NeuriteSequenceStringUtils.getNeuronNameFromSequence(sequenceName);
			neuron = cellMetaData.get(neuronName);
			
			arborizationType = NeuriteSequenceStringUtils.getArborizationType(sequenceName);
			if (!specificCellClass){
				neuron.setCellType(ClassificationUtils.reclassifyCellTypes(
					neuron.getCellType()));
				arborizationType = ClassificationUtils
					.reclassifyArborizationTypes(arborizationType);
			}
			cellTypeArbType = neuron.getCellType()+"."+arborizationType;
			group = neuron.getCellType()+"."+arborizationType+"."+neuron.getLabName();
			
			// Get distance for sequence / tree analysis
			
			tree = DataConverter.fastaStringToTree(fr.getSequence());
			/*if (sequenceName.equals("Renshawcell1-Dendrite")){
				System.out.println(fr.getSequence()+" "+tree.getTreeAsymmetry());
			}*/
			
			asymBean = new AsymmetryBean();
			asymBean.setAverageAsymmetry(tree.getTreeAsymmetry());
			asymBean.setMinDegreeAsymmetry(tree.getTreeAsymmetry(
					BinaryTreeNode.ASYMMETRY_WEIGHT_DEGREE_ABOVE_3));
			asymBean.setDfWeightedAsymmetry(tree.getTreeAsymmetry(
					BinaryTreeNode.ASYMMETRY_WEIGHT_DF));
			asymBean.setCaulescence(tree.getAsymmetry().getCaulescence());

			fullResults.append(sequenceName);
			fullResults.append(","+fr.getSequenceLength());
			fullResults.append(","+arborizationType);
			fullResults.append(","+asymBean.getAverageAsymmetry());
			fullResults.append(","+asymBean.getMinDegreeAsymmetry());
			fullResults.append(","+asymBean.getDfWeightedAsymmetry());
			fullResults.append(","+asymBean.getCaulescence());
			fullResults.append(","+tree.getMaxOrderBranch().getBranchOrder());
			fullResults.append("\n");
			distBean.addData(asymBean.getAverageAsymmetry());
			
			//addToClassTypeDist(seqLenDistMap,seqLenMap.get(sequenceName),asymmetry);
			
			addToClassTypeDist(arbTypeLocMap,arborizationType,asymBean);
			addToClassTypeDist(cellTypeLocMap,neuron.getCellType(),asymBean);
			addToClassTypeDist(labLocMap,neuron.getLabName(),asymBean);
			addToClassTypeDist(cellTypeArbTypeLocMap,cellTypeArbType,asymBean);
			addToClassTypeDist(groupLocMap,group,asymBean);
			addToClassTypeDist(lengthLocMap,"ArbLen"+fr.getSequenceLength(),asymBean);
			
		}
		fr.close();

		String filepath = props.getResultsDir()+"asymmetry/";
		File file = new File(filepath);
		if (!file.exists()){
			file.mkdirs();
		}
		filepath += dataset+"-";
		DataWriter.writeStringToFile(
				filepath+"asymmetry.csv",
				fullResults.toString());
		
		StringBuffer sb = new StringBuffer();
		sb.append("Stats for "
				+config.getString(NSConstants.PARAM_TRAVERSAL)
				+config.getString(NSConstants.PARAM_ORDER)
				+": \r\n");
		sb.append("Mean: "+distBean.getMean()+"\r\n");
		sb.append("Std Dev: "+distBean.getStandardDeviation()+"\r\n");
		sb.append("1st%: "+distBean.getPercentile(.01)+"\r\n");
		sb.append("5th%: "+distBean.getPercentile(.05)+"\r\n");
		sb.append("50th%: "+distBean.get50thPercentile()+"\r\n");
		sb.append("95th%: "+distBean.get95thPercentile()+"\r\n");
		sb.append("99th%: "+distBean.get99thPercentile()+"\r\n");
		DataWriter.writeStringToFile(
				filepath+"asymmetrySummary.csv",
				sb.toString());
		
		sb = new StringBuffer();
		writeClassTypeDist(sb,arbTypeLocMap,"Arborization Type");
		writeClassTypeDist(sb,cellTypeLocMap,"Cell Type");
		writeClassTypeDist(sb,labLocMap,"Lab");
		writeClassTypeDist(sb,cellTypeArbTypeLocMap,"CellType.ArbType");
		writeClassTypeDist(sb,groupLocMap,"CellType.ArbType.Lab");
		DataWriter.writeStringToFile(
				filepath+"asymmetrySummary-perClass.csv",
				sb.toString());

		sb = new StringBuffer();
		writeClassTypeDist(sb,lengthLocMap,"Length");
		DataWriter.writeStringToFile(
				filepath+"asymmetrySummary-perLength.csv",
				sb.toString());

		/*
		sb = new StringBuffer();
		sb.append("Length,Mean Locality,Std Dev\r\n");
		List lenList = new LinkedList(seqLenDistMap.keySet());
		Collections.sort(lenList);
		for (int i = 0; i < lenList.size(); i+=2){
			distBean = (DistributionBean)seqLenDistMap.get(lenList.get(i));
			sb.append(lenList.get(i)+","+distBean.getMean()+","+distBean.getStandardDeviation()+"\r\n");
		}
		if (metric == METRIC_FROM_SEQUENCE){
			DataWriter.writeStringToFile(
					PropertyUtils.getResultsDir()+"locality/localityFromSeq-perLength-rad"+radius+"-"+traversal+order+".csv",
					sb.toString());
		}
		else{
			DataWriter.writeStringToFile(
					PropertyUtils.getResultsDir()+"locality/localityFromTree-perLength-rad"+radius+"-"+traversal+order+".csv",
					sb.toString());
		}*/
	}
	
	private static void addToClassTypeDist(Map<String,List<DistributionBean>> locMap, String key, AsymmetryBean asymBean){
		List<DistributionBean> distBeanList;
		if (!locMap.containsKey(key)){
			distBeanList = new ArrayList<DistributionBean>();
			locMap.put(key, distBeanList);
			for (int i = 0; i < 3; i++){
				distBeanList.add(new DistributionBean());
			}
		}
		distBeanList = locMap.get(key);
		distBeanList.get(0).addData(asymBean.getAverageAsymmetry());
		distBeanList.get(1).addData(asymBean.getMinDegreeAsymmetry());
		distBeanList.get(2).addData(asymBean.getDfWeightedAsymmetry());
	}

	private static void writeClassTypeDist(StringBuffer sb, Map<String,List<DistributionBean>> locMap, String classType){
		DistributionBean distBean;
		sb.append(classType+",N,Mean Asymmetry,Std Dev,MinDegreeAsymmetry,DFAsymmetry\r\n");
		List<DistributionBean> distBeanList;
		for (String className : locMap.keySet()){
			distBeanList = locMap.get(className);
			distBean = distBeanList.get(0);
			sb.append(className+","+distBean.getCount()+","+distBean.getMean()+","+distBean.getStandardDeviation());
			distBean = distBeanList.get(1);
			sb.append(","+distBean.getMean());
			distBean = distBeanList.get(2);
			sb.append(","+distBean.getMean()+"\r\n");
		}
		sb.append("\r\n");
	}

	private static void findRandomAsymmetry(JSAPResult config) throws Exception{
		
		DataProperties props = DataProperties.getInstance();
		String baseDir = props.getFastaDir()+"Random/Method3/SizeLTS/";
		FastaReader fr;
	
		DistributionBean distBean, minDegreeDistBean, dfDistBean;
		String sequenceName, fastaFilename;
		BinaryTreeNode tree;
		
		List<Integer> lengthList = NSDataReader.loadUniqueSequenceLengths(NSConstants.MAPPING_MODE_TOPO_CA);
		int length;
		NumberFormat nf = NumberFormat.getInstance();
		double asymmetry;
		
		StringBuffer meansSb = new StringBuffer();
		meansSb.append("Length,Mean Asymmetry,Std Dev,MinDegreeAsymmetry,DFAsymmetry\r\n");
		StringBuffer allSb = new StringBuffer();
		allSb.append("Tree,Length,Asymmetry,MinDegreeAsymmetry,DFAsymmetry\r\n");
		for (int i = 0; i < lengthList.size(); i+=2){
			distBean = new DistributionBean();
			minDegreeDistBean = new DistributionBean();
			dfDistBean = new DistributionBean();
			length = lengthList.get(i);
			System.out.println("Rand len "+length);
			for (int j = 1; j <= 100; j++){
				// Load sequenceName
				sequenceName = "LEN"+length+"NUM"+j+"LTS.fasta";
				
				// Load fasta file / actual sequence
				fastaFilename = baseDir+sequenceName;
				fr = new FastaReader(fastaFilename);
				fr.next();
				// Get asymmetry
				tree = DataConverter.fastaStringToTree(fr.getSequence());
				asymmetry = tree.getTreeAsymmetry();
				distBean.addData(asymmetry);
				minDegreeDistBean.addData(tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_DEGREE_ABOVE_3));
				dfDistBean.addData(tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_DF));
				allSb.append(sequenceName+","+length+","+asymmetry+","
						+tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_DEGREE_ABOVE_3)+","
						+tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_DF)+"\r\n");
				fr.close();
			}
			meansSb.append(length+",");
			meansSb.append(nf.format(distBean.getMean())+",");
			meansSb.append(nf.format(distBean.getStandardDeviation())+",");
			meansSb.append(nf.format(minDegreeDistBean.getMean())+",");
			meansSb.append(nf.format(dfDistBean.getMean())+"\r\n");
		}
		DataWriter.writeStringToFile(
				props.getResultsDir()+"asymmetry/RandomM3AsymmetryMeans.csv",
				meansSb.toString());
		DataWriter.writeStringToFile(
				props.getResultsDir()+"asymmetry/RandomM3Asymmetry.csv",
				allSb.toString());
		
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;
        
        opt = new FlaggedOption(PARAM_SPECIFIC_CELL_TYPE)
			.setStringParser(JSAP.BOOLEAN_PARSER)
			.setDefault("false")
			.setRequired(false) 
			.setShortFlag('c') 
			.setLongFlag(PARAM_SPECIFIC_CELL_TYPE);
        jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_RANDOMS)
			.setStringParser(JSAP.BOOLEAN_PARSER)
			.setDefault("false")
			.setRequired(false) 
			.setShortFlag('R') 
			.setLongFlag(PARAM_RANDOMS);
        jsap.registerParameter(opt);

    	JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + AsymmetryPerClass.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;

	}
}
