package org.cn3.ns.analysis;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.MetaDataManager;
import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.morpho.utils.ClassificationUtils;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.resources.DataProperties;
import org.cn3.ns.resources.ProjectProperties;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

/**
 * Creates a file with all A nodes, their tree size, and their partition, per class
 * @author gillette
 *
 */
public class PartitionAnalysis {

	public static void main(String[] args){
		try{
//			JSAPResult config = readParameters(args);
	//		int mapping = config.getInt(Constants.PARAM_MAPPING_MODE);
			int mapping = 2;
			List<String> arborizations = NSDataReader.loadSequenceList(mapping);
			Map<String,NeuronMetadata> cellData = MetaDataManager.getCellMetaData();

			// Variable preparation
			String arborizationFileBase = DataProperties.getInstance().getFastaDir()+"Arborization/Mapping"+mapping+"/Size/";
			String fastaFilename;
			NeuronMetadata neuron;
			FastaReader fr;
			
			int degree;
			double partition;
			String[] subtrees;
			Object[] dataObs;
			Map<String,Object> classDataMap = new HashMap<String,Object>();
			String arborType, cellType, arbTypeCellType;
			Set<String> cellTypes = new TreeSet<String>();
			Set<String> arbTypeCellTypes = new TreeSet<String>();
			Set<String> arborTypes = new TreeSet<String>();
			
			// Get partition data from all arbors
			for (String arborization : arborizations){
				
				// Get neuron data for arborization's lab - used to find file; + classification
				neuron = (NeuronMetadata)cellData.get(NeuriteSequenceStringUtils.getNeuronNameFromSequence(arborization));
				neuron.setCellType(ClassificationUtils.reclassifyCellTypes(neuron.getCellType()));
				arborType = ClassificationUtils.reclassifyArborizationTypes(
						NeuriteSequenceStringUtils.getArborizationType(arborization));
				cellType = neuron.getCellType();
				arbTypeCellType = arborType+"-"+neuron.getCellType();
				cellTypes.add(cellType);
				arborTypes.add(arborType);
				arbTypeCellTypes.add(arbTypeCellType);
				
				// Generate filename and retrieve sequence
				fastaFilename = arborizationFileBase + neuron.getLabName() 
					+ "/" + arborization + ".lts.fasta";
				// Get sequence
				fr = new FastaReader(fastaFilename);
				fr.next();
				
				for (int j = 0; j < fr.getSequence().length(); j++){
					if (fr.getSequence().charAt(j) == 'A'){
						subtrees = findChildTrees(fr.getSequence(), j);
						degree = subtrees[0].length()+subtrees[1].length()+2;
						partition = ((double)(degree-2*(subtrees[1].length()+1)))/(degree-2);

						dataObs = new Object[2];
						dataObs[0] = new Integer(degree);
						dataObs[1] = new Double(partition);
						addDataToClass(classDataMap,"All",dataObs);
						addDataToClass(classDataMap,cellType,dataObs);
						addDataToClass(classDataMap,arborType,dataObs);
						addDataToClass(classDataMap,arbTypeCellType,dataObs);
					}
				}
				fr.close();

			}

			System.gc();
			
			// Write to file
			int maxDataSize = 0, size;
			StringBuffer sb = new StringBuffer();
			Set<String> allClasses = new LinkedHashSet<String>();
			allClasses.add("All");
			allClasses.addAll(arborTypes);
			allClasses.addAll(cellTypes);
			allClasses.addAll(arbTypeCellTypes);
			for (String className : allClasses){
				sb.append(className+",,");
				size = ((List)classDataMap.get(className)).size();
				if (size > maxDataSize){
					maxDataSize = size;
				}
			}
			sb.append("\r\n");
			for (String className : allClasses){
				sb.append("Degree,Partition,");
			}
			sb.append("\r\n");
			List<Object> dataList;
			//NumberFormat nf = NumberFormat.getInstance();
			for (int i = 0; i < maxDataSize; i++){
				for (String className : allClasses){
					dataList = (List)classDataMap.get(className);
					if (i < dataList.size()){
						dataObs = (Object[])dataList.get(i);
						sb.append(dataObs[0]+","+dataObs[1]+",");
					}
					else{
						sb.append(",,");
					}
				}
				sb.append("\r\n");
			}
			
			DataWriter.writeStringToFile(
					ProjectProperties.getInstance().getResultsDir()+"partitionAnalysis.csv", sb.toString());
			
		} catch (Exception e){
			e.printStackTrace();
		}

		
	}
	
	private static void addDataToClass(Map<String,Object> dataMap, String key, Object[] data){
		if (!dataMap.containsKey(key)){
			dataMap.put(key, new ArrayList<Object[]>());
		}
		List<Object[]> list = (List<Object[]>)dataMap.get(key);
		list.add(data);
	}

	private static String[] findChildTrees(String treeString, int nodeIndex){
		if (treeString.charAt(nodeIndex)!='A' && treeString.charAt(nodeIndex)!='G'){
			return null;
		}
		String[] subtrees = new String[2];
		subtrees[0] = findSubtree(treeString, nodeIndex+1);
		subtrees[1] = findSubtree(treeString, nodeIndex+subtrees[0].length()+1);
		return subtrees;
	}
	
	private static String findSubtree(String treeString, int nodeIndex){
		int tees = 0, currIndex = nodeIndex;
		char chr;
		while (tees < 1){
			chr = treeString.charAt(currIndex++);
			if (chr=='A'){
				tees--;
			}
			else if (chr=='T'){
				tees++;
			}
		}
		return treeString.substring(nodeIndex, currIndex);
	}
	
}
