package org.cn3.ns.generation;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.CountDownLatch;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.motif.analysis.MotifFinder;
import org.cn3.ns.resources.DataProperties;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.MappingUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 * INCOMPLETE
 *	Generate 20 (or #sets) surrogates for each real sequence fully constrained by real sequence monomers
 *	-n --numSurrogates | required
 *	-o --order (LTS)
 *	-b --baseline (8)
 *	--maxSequenceLength (0)
 *	--debug (false)
 *	java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.generation.MABaselineGenerator -n 20 -M 4 --maxSequenceLength 1200
 *
 */
public class MultipleAlignmentBaselineGenerator {
	
	public static final String PARAM_NUM_SURROGATES = "numSurrogates";
	
	private static final String PARAM_MAX_SEQUENCE_LENGTH = "maxSequenceLength";

	private static final int DEFAULT_MAX_SEQUENCE_LENGTH = 0;
	
	private static int numSurrogates;
	private static int method;
	private static int mapping = NSConstants.MAPPING_MODE_TOPO_ACT;
	private static boolean uniformApproximation = false;
	private static int debug = 0;
	
	// Shared between threads
	private static List<String> sequenceNames;
	private static List<Integer> removedSequences;
	private static Map<Integer,Map<String,Number>> realTreeMotifCounts;
	private static List<String>[] motifsByLength;
	private static int[] firsts;
	private static Map<String,Integer> lengthMap;
	private static int numProcessors;
	private static int numGeneratedSequences;
	private static CountDownLatch latch;
	private static Set<String> motifs;
	private static String outputFilename;
	private static Map<Object,List<String>> completedSequencesForK;
	
	public static void main(String args[]) throws Exception{
		JSAPResult config = readParameters(args);
		method = config.getInt(NSConstants.PARAM_BASELINE);
		numSurrogates = config.getInt(PARAM_NUM_SURROGATES);
		int maxSequenceLength = config.getInt(PARAM_MAX_SEQUENCE_LENGTH);
		numGeneratedSequences = numSurrogates;

		debug = config.getInt(NSConstants.PARAM_DEBUG);
		NodeSequenceBinaryTreeGenerator.debugLevel = debug;
		
		System.out.println("Initializing...");
		
		StringBuffer sb;
		
		Integer sequenceId;
		
		int length, lengthIndex;
		String sequenceName;
		
		String allSeqFastaFile = DataProperties.getInstance().getFastaFilename("NM", mapping, "LTS");

		// Get sequence names and map them to unique ids
		sequenceNames = NSDataReader.loadSequenceListFromFastaFile(allSeqFastaFile);
		Map<String,Integer> sequenceIdMap = new HashMap<String,Integer>();
		for (int j = 0; j < sequenceNames.size(); j++){
			sequenceIdMap.put(sequenceNames.get(j), new Integer(j));
		}
		
		System.out.println("Loading tree size data");
		// Loading tree sizes represented in database
		List<Integer> sequenceLengths = NSDataReader.loadUniqueSequenceLengths(mapping);

		// Loading sequence:length Map
		lengthMap = NSDataReader.loadLengthDatabase(mapping);
		
		// Loading map keyed by seq length, values are lists of sequence ids of seqs of that length
		Map<Integer,List<Integer>> lengthArborsMap = loadTreeSizeArborizationsMap(sequenceIdMap);

		// Load real tree n-mer counts into sequence id keyed map
		System.out.println("Loading real tree motif counts");
		
		// Remove lengths that are too big
		removedSequences = new ArrayList<Integer>();
		if (maxSequenceLength > 0){
			System.out.println("Removing very large sequences (larger than "+maxSequenceLength+")");
			for (lengthIndex = sequenceLengths.size()-1; ((Integer)sequenceLengths.get(lengthIndex)).intValue() > maxSequenceLength; lengthIndex--){
				length = ((Integer)sequenceLengths.get(lengthIndex)).intValue();
				removedSequences.addAll(lengthArborsMap.remove(new Integer(length)));
				sequenceLengths.remove(lengthIndex);
			}
		}
		
		// Remove sequences that are no longer in the database
		sb = new StringBuffer();
		for (int i = 0; i < sequenceNames.size(); i++){
			sequenceName = (String)sequenceNames.get(i);
			sequenceId = (Integer)sequenceIdMap.get(sequenceName);
			if (!realTreeMotifCounts.containsKey(sequenceId)){
				removedSequences.add(sequenceId);
				sb.append(sequenceName+"  ");
			}
		}
		System.out.println("Sequences no longer in db: "+sb.toString());
		
		// Add flagged neurons to removedSequences lists
		List<String> flaggedSequences = NSDataReader.getFlaggedSequences();
		for (int i = 0; i < flaggedSequences.size(); i++){
			removedSequences.add(sequenceIdMap.get(flaggedSequences.get(i)));
		}

		// Loop through constraint lengths
		/*StringBuffer everything = new StringBuffer();
		everything.append("NumSurrogates:"+numSurrogates+"\n");*/
		if (!new File(outputFilename).exists()){
			DataWriter.writeStringToFile(outputFilename,"NumSurrogates:"+numSurrogates+"\n"); 
		}
		int k = 1;

		String neuriteName, baselineSequenceLTS, baselineSequenceSTL;
		int degree, cCount;
		BinaryTreeNode baselineTreeRoot;
		Map<String,Number> motifCounts;
		
		// LOOP THROUGH SEQUENCES
		FastaReader fr = new FastaReader(allSeqFastaFile);
		while (fr.next()){
			neuriteName = fr.getNeuron().getName();
			motifCounts = MotifFinder.findMotifs(fr.getSequence(),1,1);
			cCount = motifCounts.get("C").intValue();
			degree = fr.getSequenceLength()+1;
			
			// GENERATE BASELINES
			baselineTreeRoot = 
				NodeSequenceBinaryTreeGenerator.generateRandomMonomerConstrainedHardingTreeByDegree(
					degree, cCount);
			
			baselineSequenceLTS = MappingUtils.traverseTreeToString(baselineTreeRoot,
					NSConstants.MAPPING_MODE_TOPO_ACT, "LTS");
			baselineSequenceSTL = MappingUtils.traverseTreeToString(baselineTreeRoot,
					NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
			
			// STORE BASELINES
			
		}

	}
	
	private static synchronized void appendToFile(String string) throws IOException{
		DataWriter.appendStringToFile(outputFilename, string);
	}
	
	// Maps sequence lengths to lists of sequence IDs associated with sequences of that length
	private static Map<Integer,List<Integer>> loadTreeSizeArborizationsMap
	(Map<String,Integer> sequenceIdMap) throws Exception{
		Map<Integer,List<String>> treeSizeArborMapTmp = NSDataReader.loadTreeSizeArborizationsMap(mapping);
		Map<Integer,List<Integer>> treeSizeArborMap = new HashMap<Integer,List<Integer>>();
		Integer seqLength;
		List<String> listOfSequences;
		List<Integer> listOfLengths;
		for (Iterator<Integer> it = treeSizeArborMapTmp.keySet().iterator(); it.hasNext();){
			seqLength = it.next();
			listOfSequences = treeSizeArborMapTmp.get(seqLength);
			listOfLengths = new ArrayList<Integer>();
			for (int i = 0; i < listOfSequences.size(); i++){
				listOfLengths.add(sequenceIdMap.get(listOfSequences.get(i)));
				//System.out.println("ltsam "+list1.get(i)+" " +sequenceIdMap.get(list1.get(i)));
			}
			treeSizeArborMap.put(seqLength, listOfLengths);
		}
		return treeSizeArborMap;
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;
        
	    opt = new FlaggedOption(NSConstants.PARAM_BASELINE)
        .setStringParser(JSAP.INTEGER_PARSER)
        .setRequired(false)
        .setDefault(""+NSConstants.DISTRIBUTION_NODE_TYPE_ASSEMBLED3)
        .setShortFlag('b') 
        .setLongFlag(NSConstants.PARAM_BASELINE);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(PARAM_NUM_SURROGATES)
        .setStringParser(JSAP.INTEGER_PARSER)
        .setRequired(true)
        .setShortFlag('n') 
        .setLongFlag(PARAM_NUM_SURROGATES);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(PARAM_MAX_SEQUENCE_LENGTH)
	    	.setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault(""+DEFAULT_MAX_SEQUENCE_LENGTH)
	        .setShortFlag(JSAP.NO_SHORTFLAG) 
	        .setLongFlag(PARAM_MAX_SEQUENCE_LENGTH);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(NSConstants.PARAM_DEBUG)
    	.setStringParser(JSAP.INTEGER_PARSER)
        .setRequired(false)
        .setDefault("0")
        .setShortFlag(JSAP.NO_SHORTFLAG) 
        .setLongFlag(NSConstants.PARAM_DEBUG);
	    jsap.registerParameter(opt);

    	JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (@SuppressWarnings("rawtypes")
			java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + MultipleAlignmentBaselineGenerator.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}

}
