package org.cn3.ns.motif.analysis;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.domain.DistributionBean;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.motif.generation.ConstraintsDataReader;
import org.cn3.ns.motif.generation.MotifConstants;
import org.cn3.ns.resources.DataProperties;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.MappedBinaryTreeUtils;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 * Simply writes a file containing each sequence and its kmer frequencies
 * Reals filename format: KmerFreqs-Size%order%-K%k%.csv
 * Surrogates filename format: KmerFreqs-Size%order%[(RandSet%randomSet%|RandSets%randomSets%%distSummaryType%)K%randomK%-B%method%-T%threshold%%thresholdType%]-K%k%.csv
 * Reals run
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator --startK 1 --endK 4 -o STL --outputDir ~/results/
 *
 * Baseline sequences
 * filename format: KmerFreqs-Size%order%-K%k%.csv
 *
 * 5/16/14 Run on NM_SpurLen2D, getting raw kmer counts
 *
 * 2/28/14 Run on NM_SpurLen2D
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -D NM_SpurLen2D -o STL --startK 1 --endK 1 --outputDir ~/results/RawKmerFreqs/ --sourceTraversal STL
 * 
 * 2/19/14 Run on NM_SpurLen5
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -D NM_SpurLen5 -o STL --startK 1 --endK 1 --outputDir ~/results/RawKmerFreqs/ --sourceTraversal STL
 *  
 *
 * Method 3 (Uniform dist)
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o LTS -T 0.0 -m 3 -S 0 -K 0 --randomSets 20 --randomK 0 --startK 1 --endK 1 --outputDir ~/results/
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o LTS -T 0.0 -m 3 -S 0 -K 0 --randomSets 20 --randomK 1 --startK 2 --endK 2 --outputDir ~/results/
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o LTS -T 0.0 -m 3 -S 0 -K 0 --randomSets 20 --randomK 2 --startK 3 --endK 3 --outputDir ~/results/
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o STL -T 0.0 -m 3 -S 0 -K 0 --randomSets 20 --randomK 2 --startK 3 --endK 3 --outputDir ~/results/ --sourceTraversal LTS
 * Method 8 (kmer assembly)
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o LTS -T 0.0 -m 8 -S 0 -K 0 --randomSets 20 --randomK 0 --startK 1 --endK 1 --outputDir ~/results/ --singleSurrogateFile true
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o LTS -T 0.0 -m 8 -S 0 -K 0 --randomSets 20 --randomK 1 --startK 2 --endK 2 --outputDir ~/results/ --singleSurrogateFile true
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o LTS -T 0.0 -m 8 -S 0 -K 0 --randomSets 20 --randomK 2 --startK 3 --endK 3 --outputDir ~/results/ --singleSurrogateFile true
 * 
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o LTS --sourceTraversal LTS -T 0.0 -m 8 -S 0 -K 0 --randomSet 4 --randomSets 40 --randomK 2 --startK 3 --endK 3 --outputDir ~/results/ --singleSurrogateFile true
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o STL --sourceTraversal LTS -T 0.0 -m 8 -S 0 -K 0 --randomSet 4 --randomSets 40 --randomK 2 --startK 3 --endK 3 --outputDir ~/results/ --singleSurrogateFile true
 * 
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o LTS -T 0.0 -m 2 -S 0 -K 0 --randomSet 9 --randomK 0 --startK 2 --endK 2 --outputDir ~/results/
 *
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o LTS -T 0.0 -m 8 -S 0 -K 0 --randomSets 40 --randomK 2 --startK 3 --endK 3 --outputDir ~/results/ --singleSurrogateFile true
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o STL -T 0.0 -m 8 -S 0 -K 0 --randomSets 40 --randomK 2 --startK 3 --endK 3 --outputDir ~/results/ --singleSurrogateFile true
 * 
 * Checking kmers of one traversal order of virtual tree-sequence constrained by k-1-mers of the other traversal order 
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o LTS -T 0.0 -m 8 -S 0 -K 0 --randomSets 40 --setNum 2 --randomK 2 --startK 3 --endK 3 --outputDir ~/results/ --singleSurrogateFile true --sourceTraversal STL
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o STL -T 0.0 -m 8 -S 0 -K 0 --randomSets 40 --setNum 2 --randomK 2 --startK 3 --endK 3 --outputDir ~/results/ --singleSurrogateFile true --sourceTraversal LTS
 * Standard deviations
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o LTS -T 0.0 -m 8 -S 0 -K 0 --randomSets 40 --setNum 2 --randomK 2 --startK 3 --endK 3 --outputDir ~/results/ --singleSurrogateFile true --sourceTraversal STL --randomDistMode 2
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.motif.analysis.KmerFrequencyCalculator -o STL -T 0.0 -m 8 -S 0 -K 0 --randomSets 40 --setNum 2 --randomK 2 --startK 3 --endK 3 --outputDir ~/results/ --singleSurrogateFile true --sourceTraversal LTS --randomDistMode 2 
 * 
 */
public class KmerFrequencyCalculator {
	
	private static final String PARAM_RANDOM_SET = "randomSet";
	private static final String PARAM_RANDOM_SETS = "randomSets";
	private static final String PARAM_RANDOM_DIST_MODE = "randomDistMode";
	private static final String PARAM_RANDOM_K = "randomK";
	private static final String PARAM_START_K = "startK";
	private static final String PARAM_END_K = "endK";
	private static final String PARAM_OUTPUT_DIR = "outputDir";
	private static final String PARAM_SOURCE_TRAVERSAL = "sourceTraversal";
	
	private static final int DEFAULT_END_K = 4;
	
	private static final int DIST_MODE_MEAN = 1;
	private static final int DIST_MODE_SD = 2;
	
	private static double threshold;
	private static int kmerThresholdType = MotifConstants.KMER_THRESHOLD_NONE;
	private static int seqThresholdType = MotifConstants.SEQ_THRESHOLD_WHOLE;
	private static String thresholdTypeString = null;
	private static int method;
	private static String dataset;
	private static String order;
	private static int randomSet = 0;
	private static int randomSets = 0;
	private static int distributionMode = 1;
	private static int randomK = 0;
	private static int mapping = NSConstants.MAPPING_MODE_TOPO_ACT;
	private static int endK = DEFAULT_END_K;
//	private static int debug = 0;
	private static int startK = 0;
	private static int surrogateSetNum = -1;
	private static String sourceTraversal;
	private static boolean retraverse = false;
	
	private static List<String>[] removedSequences;
	private static List<String>[] motifsByLength;

	@SuppressWarnings("unchecked")
	public static void main(String args[]) throws Exception{
		JSAPResult config = readParameters(args);
		surrogateSetNum = randomSet = config.getInt(PARAM_RANDOM_SET);
		randomSets = config.getInt(PARAM_RANDOM_SETS);
		distributionMode = config.getInt(PARAM_RANDOM_DIST_MODE);
		randomK = config.getInt(PARAM_RANDOM_K);
		startK = config.getInt(PARAM_START_K);
		endK = config.getInt(PARAM_END_K);
		String outputDir = config.getString(PARAM_OUTPUT_DIR);
		
		threshold = config.getDouble(MotifConstants.PARAM_THRESHOLD);
		seqThresholdType = config.getInt(MotifConstants.PARAM_SEQ_THRESHOLD_TYPE);
		if (seqThresholdType != MotifConstants.SEQ_THRESHOLD_WHOLE){
			kmerThresholdType = config.getInt(MotifConstants.PARAM_KMER_THRESHOLD_TYPE);
		}
		thresholdTypeString = MotifConstants.getThresholdTypeString(kmerThresholdType, seqThresholdType);
		method = config.getInt(NSConstants.PARAM_RANDOM_METHOD);
		dataset = config.getString(NSConstants.PARAM_DATASET);
		order = config.getString(NSConstants.PARAM_ORDER);
		sourceTraversal = order;
		if (config.userSpecified(PARAM_SOURCE_TRAVERSAL)){
			sourceTraversal = config.getString(PARAM_SOURCE_TRAVERSAL);
			if (!sourceTraversal.equals(order)){
				retraverse = true;
			}
		}
		
		System.out.println("Initializing...");
		
		removedSequences = new List[endK+1];
		for (int i = 0; i <= endK; i++){
			removedSequences[i] = new ArrayList<String>();
		}

		// Initializing motifs
		motifsByLength = new List[endK+1];
		for (int i = 1; i <= endK; i++){
			if (i > 1){
				motifsByLength[i] = NeuriteSequenceStringUtils.generatePossibleACTKmers(i,order);
			}
			else{
				motifsByLength[1] = new ArrayList<String>();
				motifsByLength[1].add("C");
			}
		}

		// Initializing string buffers for output
		StringBuffer[] sb = new StringBuffer[endK+1];
		for (int k = startK; k <= endK; k++){
			sb[k] = new StringBuffer();
			sb[k].append("Sequence");
			for (int j = 0; j < motifsByLength[k].size(); j++){
				sb[k].append(","+motifsByLength[k].get(j));
			}
			sb[k].append("\n");
		}

		// Load real tree n-mer counts into sequence id keyed map
		System.out.println("Loading real tree motif counts");
		Map<String,Map<String,Number>> treeMotifFreqs = null;
		if (randomSets > 0){
			treeMotifFreqs = getVirtualTreeMotifFreqSummary();
		}
		else{
			treeMotifFreqs = loadRealTreeMotifCounts(motifsByLength);
		}

		Map<String,Number> motifCounts;

		// Determine whether nmers are constrained
		String kmer;
		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(3);
		
		// Loop through sequences and k to calculate whether a given real sequence was constrained
		for (String sequenceName : treeMotifFreqs.keySet()){
			motifCounts = treeMotifFreqs.get(sequenceName);
			
			for (int k = startK; k <= endK; k++){
				// Start new line in output file
				sb[k].append(sequenceName);
				
				// Loop through motifs at k and determine the sequence's kmer frequencies
				for (int i = 0; i < motifsByLength[k].size(); i++){
					// Get distribution of k+1-mer counts across virtual sequences
					kmer = motifsByLength[k].get(i);
					// Get the average kmer count
					//System.out.println(kmer+" "+motifCounts.get(kmer));
					sb[k].append(","+nf.format(motifCounts.get(kmer)));
				}
				sb[k].append("\n");
				
			}
		}
		
 		// Write results to a csv file
		String baseFilename = outputDir+"/"+
			dataset+"-KmerFreqs-Size"+order;
		if (randomSets > 0){
			String distSummaryType;
			if (distributionMode == 1){
				distSummaryType = "Mean";
			}
			else{
				distSummaryType = "SD";
			}
			if (randomSet > -1){
				baseFilename += "RandSets"+randomSets+distSummaryType+"K"+randomK+"-B"+method+sourceTraversal+"_Set"+randomSet;
			}
			else{
				baseFilename += "RandSets"+randomSets+distSummaryType+"K"+randomK+"-B"+method+sourceTraversal+"-T"+threshold+thresholdTypeString;
			}
		}
		else if (randomSet > 0){
			baseFilename += "RandSet"+randomSet+"K"+randomK+"-B"+method+sourceTraversal+"-T"+threshold+thresholdTypeString;
		}
		String filename;
		for (int k = startK; k <= endK; k++){
			filename = baseFilename + "-K"+k+".csv";
			DataWriter.writeStringToFile(filename, sb[k].toString());
			System.out.println("Wrote to "+filename);
		}

	}
	
	// Gets mean and sd of kmer frequency distribution for each surrogate set (by real sequence) and kmer
	private static Map<String,Map<String,Number>> getVirtualTreeMotifFreqSummary() throws Exception{
		Map<Integer,Map<String,List<String>>> virtualSequencesPerK = null;
		Map<String,Number> kmerFreqMap;
		Map<String,Map<String,Number>> treeMotifFreqs = null;
		Map<String,Map<String,DistributionBean>> treeMotifDists = new HashMap<String,Map<String,DistributionBean>>();
		Map<String,DistributionBean> kmerDistMap;
		DistributionBean distBean;
		// Load surrogates
		virtualSequencesPerK = ConstraintsDataReader.loadSurrogates(surrogateSetNum,sourceTraversal,method,randomSets,false);

		Map<String,List<String>> virtualSequencesBySeq = virtualSequencesPerK.get(randomK);
		String sequence;
		// Add data from set to distributions for each real sequence
		for (String seqName : virtualSequencesBySeq.keySet()){
			List<String> sequences = virtualSequencesBySeq.get(seqName);
			kmerDistMap = new HashMap<String,DistributionBean>();
			treeMotifDists.put(seqName, kmerDistMap);
	
			// Set up distributions for each kmer
			for (int k = startK; k <= endK; k++){
				for (int j = 0; j < motifsByLength[k].size(); j++){
					kmerDistMap.put(motifsByLength[k].get(j), new DistributionBean());
				}
			}
			// Go through surrogate set (for current real sequence) and fill in kmer frequency distributions
			for (int i = 0; i < sequences.size(); i++){
				sequence = sequences.get(i);
				if (retraverse) sequence = MappedBinaryTreeUtils.reorderSequence(sequence,order);
				kmerFreqMap = MotifFinder.findMotifFrequencies(sequence,startK,endK);
				for (int k = startK; k <= endK; k++){
					for (String kmer : motifsByLength[k]){
						distBean = kmerDistMap.get(kmer);
						if (!kmerFreqMap.containsKey(kmer)){
							distBean.addData(0);
						}
						else{
							distBean.addData(kmerFreqMap.get(kmer));
						}
					}
				}
			}
		}
		
		// Get summary data from distributions and put in treeMotifFreqs
		treeMotifFreqs = new HashMap<String,Map<String,Number>>();
		for (String seqName : treeMotifDists.keySet()){
			kmerFreqMap = new HashMap<String,Number>();
			treeMotifFreqs.put(seqName, kmerFreqMap);
			kmerDistMap = treeMotifDists.get(seqName);
			for (String kmer : kmerDistMap.keySet()){
				distBean = kmerDistMap.get(kmer);
				switch (distributionMode){
				case DIST_MODE_SD:
					kmerFreqMap.put(kmer, distBean.getStandardDeviation());
					break;
				case DIST_MODE_MEAN:
				default:
					kmerFreqMap.put(kmer, distBean.getMean());
				}
			}
		}
		return treeMotifFreqs;
	}
	
	private static Map<String,Map<String,Number>> loadRealTreeMotifCounts(
			List<String>[] motifsByLength) throws Exception{
		Map<String,Map<String,Number>> realTreeMotifCounts = new HashMap<String,Map<String,Number>>();
		FastaReader fr = new FastaReader();
		Map<String,Number> motifCounts;

		String fastaFilename = DataProperties.getInstance().getFastaArborDir(dataset)+"Mapping"+mapping+"/Size/"+dataset+"ArborSize"+order+"Map"+mapping+".fasta";
		System.out.println("Reading "+fastaFilename);
		fr.setFile(fastaFilename);
		while (fr.next()){
			motifCounts = MotifFinder.findMotifFrequencies(fr.getSequence(),1,endK);
			for (int k = startK; k <= endK; k++){
				for (int j = 0; j < motifsByLength[k].size(); j++){
					if (!motifCounts.containsKey(motifsByLength[k].get(j))){
						motifCounts.put(motifsByLength[k].get(j), 0);
					}
				}
			}
			realTreeMotifCounts.put(fr.getSequenceName(),
				motifCounts);
		}
		return realTreeMotifCounts;
	}


	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;

	    opt = new FlaggedOption(PARAM_START_K)
    	.setStringParser(JSAP.INTEGER_PARSER)
        .setRequired(false)
        .setDefault("1")
        .setShortFlag(JSAP.NO_SHORTFLAG) 
        .setLongFlag(PARAM_START_K);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_END_K)
       	.setStringParser(JSAP.INTEGER_PARSER)
        .setRequired(false)
        .setDefault(""+DEFAULT_END_K)
        .setShortFlag(JSAP.NO_SHORTFLAG) 
        .setLongFlag(PARAM_END_K);
	    jsap.registerParameter(opt);
	    
	    opt = new FlaggedOption(PARAM_RANDOM_SET)
    	.setStringParser(JSAP.INTEGER_PARSER)
        .setRequired(false)
        .setDefault("-1")
        .setShortFlag(JSAP.NO_SHORTFLAG) 
        .setLongFlag(PARAM_RANDOM_SET);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(PARAM_RANDOM_SETS)
    	.setStringParser(JSAP.INTEGER_PARSER)
        .setRequired(false)
        .setDefault("0")
        .setShortFlag(JSAP.NO_SHORTFLAG) 
        .setLongFlag(PARAM_RANDOM_SETS);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(PARAM_RANDOM_DIST_MODE)
    	.setStringParser(JSAP.INTEGER_PARSER)
        .setRequired(false)
        .setDefault(""+DIST_MODE_MEAN)
        .setShortFlag(JSAP.NO_SHORTFLAG)
        .setLongFlag(PARAM_RANDOM_DIST_MODE);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(PARAM_RANDOM_K)
    	.setStringParser(JSAP.INTEGER_PARSER)
        .setRequired(false)
        .setDefault("0")
        .setShortFlag(JSAP.NO_SHORTFLAG) 
        .setLongFlag(PARAM_RANDOM_K);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(NSConstants.PARAM_DATASET)
	        .setStringParser(JSAP.STRING_PARSER)
	        .setDefault(NSConstants.DEFAULT_DATASET)
	        .setRequired(false)
	        .setShortFlag('D') 
	        .setLongFlag(NSConstants.PARAM_DATASET);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(NSConstants.PARAM_ORDER)
        .setStringParser(JSAP.STRING_PARSER)
        .setDefault(NSConstants.DEFAULT_ORDER)
        .setRequired(false)
        .setShortFlag('o') 
        .setLongFlag(NSConstants.PARAM_ORDER);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(PARAM_OUTPUT_DIR)
    	.setStringParser(JSAP.STRING_PARSER)
        .setRequired(false)
        .setDefault("C:/Documents and Settings/gillette/My Documents/research/MotifAnalysis/PercentileRanks/")
        .setShortFlag('O') 
        .setLongFlag(PARAM_OUTPUT_DIR);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(MotifConstants.PARAM_THRESHOLD)
        .setStringParser(JSAP.DOUBLE_PARSER)
        .setDefault("0")
        .setRequired(false)
        .setShortFlag('T') 
        .setLongFlag(MotifConstants.PARAM_THRESHOLD);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(MotifConstants.PARAM_METHOD)
	        .setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault(""+NSConstants.DEFAULT_BASELINE)
	        .setShortFlag('m') 
	        .setLongFlag(MotifConstants.PARAM_METHOD);
	    jsap.registerParameter(opt);
	
	    opt = new FlaggedOption(MotifConstants.PARAM_KMER_THRESHOLD_TYPE)
	    	.setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault(""+MotifConstants.KMER_THRESHOLD_ABSOLUTE)
	        .setShortFlag('K') 
	        .setLongFlag(MotifConstants.PARAM_KMER_THRESHOLD_TYPE);
	    jsap.registerParameter(opt);
	
	    opt = new FlaggedOption(MotifConstants.PARAM_SEQ_THRESHOLD_TYPE)
	    	.setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault(""+MotifConstants.SEQ_THRESHOLD_STRICT)
	        .setShortFlag('S') 
	        .setLongFlag(MotifConstants.PARAM_SEQ_THRESHOLD_TYPE);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(PARAM_SOURCE_TRAVERSAL)
    	.setStringParser(JSAP.STRING_PARSER)
        .setRequired(false)
        .setDefault(""+NSConstants.DEFAULT_ORDER)
        .setShortFlag(JSAP.NO_SHORTFLAG) 
        .setLongFlag(PARAM_SOURCE_TRAVERSAL);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(NSConstants.PARAM_DEBUG)
    	.setStringParser(JSAP.INTEGER_PARSER)
        .setRequired(false)
        .setDefault("0")
        .setShortFlag(JSAP.NO_SHORTFLAG) 
        .setLongFlag(NSConstants.PARAM_DEBUG);
	    jsap.registerParameter(opt);

    	JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (@SuppressWarnings("rawtypes")
			java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + KmerFrequencyCalculator.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}

}
