package org.cn3.ns.motif.analysis;

import java.text.NumberFormat;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.cn3.ns.data.FastaReader;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 * Calculate motif data for a given input fasta file
 *
 */
public class MotifAnalyzer {

	private static final String PARAM_INPUT = "input";
	private static final String PARAM_MIN_LENGTH = "minMotif";
	private static final String PARAM_MAX_LENGTH = "maxMotif";
	
	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);
			String inputFile = config.getString(PARAM_INPUT);
			int minMotifLength = config.getInt(PARAM_MIN_LENGTH);
			int maxMotifLength = config.getInt(PARAM_MAX_LENGTH);
			
			FastaReader fr = new FastaReader(inputFile);
			Map globalMotifCounter = new HashMap();
			Map motifMap;
			String motif;
			long currCount, globalCount;
			long[] totalCounts = new long[maxMotifLength+1];
			for (int i = minMotifLength; i <= maxMotifLength; i++){
				totalCounts[i] = 0;
			}
			while (fr.next()){
				motifMap = MotifFinder.findMotifs(fr.getSequence(),minMotifLength,maxMotifLength);
				for (Iterator it = motifMap.keySet().iterator(); it.hasNext();){
					motif = (String)it.next();
					if (!globalMotifCounter.containsKey(motif)){
						globalMotifCounter.put(motif, new Integer(0));
					}
					globalCount = ((Long)globalMotifCounter.get(motif)).intValue();
					currCount = ((Integer)motifMap.get(motif)).intValue();
					globalMotifCounter.put(motif,new Long(currCount+globalCount));
					totalCounts[motif.length()] += currCount;
				}
			}

			double frequency;
			NumberFormat nf = NumberFormat.getInstance();
			nf.setMaximumFractionDigits(3);
			for (Iterator it = globalMotifCounter.keySet().iterator(); it.hasNext();){
				motif = (String)it.next();
				globalCount = ((Integer)globalMotifCounter.get(motif)).intValue();
				frequency = ((double)globalCount)/totalCounts[motif.length()];
				System.out.println(motif+": "+nf.format(frequency));
			}

		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;
        
	    opt = new FlaggedOption(PARAM_INPUT)
	    .setStringParser(JSAP.STRING_PARSER)
	    .setRequired(true)
	    .setShortFlag('i') 
	    .setLongFlag(PARAM_INPUT);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(PARAM_MIN_LENGTH)
	    .setStringParser(JSAP.INTEGER_PARSER)
	    .setRequired(false)
	    .setDefault("1")
	    .setShortFlag('n') 
	    .setLongFlag(PARAM_MIN_LENGTH);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(PARAM_MAX_LENGTH)
	    .setStringParser(JSAP.INTEGER_PARSER)
	    .setRequired(false)
	    .setDefault("1")
	    .setShortFlag('x') 
	    .setLongFlag(PARAM_MAX_LENGTH);
	    jsap.registerParameter(opt);

	    JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + MotifAnalyzer.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}
}
