package org.cn3.ns.motif.generation;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cn3.morpho.data.DataWriter;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.data.persistence.GACTCompressor;
import org.cn3.ns.resources.DataProperties;
import org.cn3.ns.resources.NSConstants;

import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

public class ConstraintsResultDeleter {

	public static Map loadVirtualSequencesAcrossSets(int sets, 
			String order, int method, double threshold, int kmerThresholdType, int seqThresholdType) throws Exception{
		return loadVirtualSequencesAcrossSets(false, true, sets, order, method, threshold, kmerThresholdType, seqThresholdType, false);
	}
	public static Map loadVirtualSequencesAcrossSets(int sets, 
			String order, int method, double threshold, int kmerThresholdType, int seqThresholdType, boolean doubleLineBreak) throws Exception{
		return loadVirtualSequencesAcrossSets(false, true, sets, order, method, threshold, kmerThresholdType, seqThresholdType, doubleLineBreak);
	}
	public static Map loadVirtualSequencesAcrossSets(boolean keyById, boolean uncompress, int sets,
			String order, int method, double threshold, int kmerThresholdType, int seqThresholdType, boolean doubleLineBreak)
	throws Exception{
		List sequenceMapList = new ArrayList();
		Map virtualSequencesPerK;
		File file;
		for (int setNum = 1; setNum <= sets; setNum++){
			file = new File(MotifConstants.getConstrainedTreesFilename(order, method, setNum, threshold, kmerThresholdType, seqThresholdType));
			virtualSequencesPerK = ConstraintsResultDeleter.loadSequencesByRealSequence(file, keyById, uncompress, doubleLineBreak);
			if (virtualSequencesPerK != null){
				sequenceMapList.add(virtualSequencesPerK);
			}
		}
		return ConstraintsResultDeleter.reorganizeMapsIntoSequenceLists(sequenceMapList);
	}
	
	/**
	 * 
	 * @param sequenceMaps (List of Map[k,Map[sequence name or id,virtualSequence]])
	 * @return Map[k,Map[sequence name or id,List(virtualSequences)]]
	 */
	public static Map reorganizeMapsIntoSequenceLists(List sequenceMaps){
		Map newMap = new HashMap();
		Map mainMap, mapForK, newMapForK;
		Integer k;
		Object key;
		List virtualSequenceList;
		
		// Loop through map from each virtual sequence set
		for (int i = 0; i < sequenceMaps.size(); i++){
			mainMap = (Map)sequenceMaps.get(i);
			// Loop through each k represented in the map
			for (Iterator it = mainMap.keySet().iterator(); it.hasNext();){
				k = (Integer)it.next();
				// Create a map of sequences for k if it doesn't exist
				if (!newMap.containsKey(k)){
					newMap.put(k, new HashMap());
				}
				mapForK = (Map)mainMap.get(k);
				newMapForK = (Map)newMap.get(k);
				// Loop through sequences in mapForK
				for (Iterator it2 = mapForK.keySet().iterator(); it2.hasNext();){
					key = it2.next();
					if (!newMapForK.containsKey(key)){
						newMapForK.put(key, new ArrayList());
					}
					// Add virtualSequence into list for the k/sequence pair
					virtualSequenceList = (List)newMapForK.get(key);
					virtualSequenceList.add(mapForK.get(key));
				}
			}
		}
		return newMap;
	}
	
	public static Map loadSequencesByRealSequence(File file, boolean keyById, boolean uncompress, boolean doubleLineBreak) throws Exception{
		Map sequenceMap, kSequenceMap = null;
		Integer kKey;
		
		if (file.exists()){
			kSequenceMap = new HashMap();
			List sequenceNames = NSDataReader.loadSequenceList();
			
			FileInputStream fis = null;
			DataInputStream dis = null;
			
			try{
				fis = new FileInputStream(file);
				dis = new DataInputStream(fis);
	
				Object virtualSequence;
				String dataString, sequenceName;
				byte[] compressedSequence;
				// Pattern for sequence id, followed by kmin and kmax constrained 
				Pattern p = Pattern.compile("(\\d+)\\|(\\d)\\-(\\d)");
				Matcher m;
				int idInFile, kmin, kmax;

				// Read id line
				dataString = NSDataReader.readLine(dis, doubleLineBreak);
				
				while (dataString != null){
					m = p.matcher(dataString);
			        
			        if (m.matches()){
			        	// Get sequence info
			        	idInFile = Integer.parseInt(m.group(1));
			        	sequenceName = (String)sequenceNames.get(idInFile);
			        	kmin = Integer.parseInt(m.group(2));
			        	kmax = Integer.parseInt(m.group(3));
			        	
			        	// Get and decompress virtual sequence
			        	virtualSequence = compressedSequence = GACTCompressor.getByteSequence(dis);
			        	if (uncompress){
				        	virtualSequence = GACTCompressor.decompressSequence(compressedSequence);
			        	}
			        	// Clear the line
			        	if (doubleLineBreak){
			        		dis.read();
			        		dis.read();
			        	}

			        	// Store virtual sequence
			        	for (int k = kmin; k <= kmax; k++){
			        		kKey = new Integer(k);
			        		if (!kSequenceMap.containsKey(kKey)){
			        			kSequenceMap.put(kKey, new HashMap());
			        		}
			        		sequenceMap = (Map)kSequenceMap.get(kKey);
			        		if (keyById){
			        			sequenceMap.put(new Integer(idInFile), virtualSequence);
			        		}
			        		else{
			        			sequenceMap.put(sequenceName, virtualSequence);
			        		}
			        	}
			        }
			        dataString = NSDataReader.readLine(dis, doubleLineBreak);
				};
				
			} catch (IOException ioe){
				throw new Exception(ioe);
			} finally{
				if (dis != null){
					dis.close();
				}
				if (fis != null){
					fis.close();
				}
			}
		}
		else{
			System.out.println("File "+file.getName()+" does not exist");
		}
		return kSequenceMap;
	}
	
	private static int setNum;
	private static int method;
	private static double threshold;
	private static String order;
	private static int seqThresholdType;
	private static int kmerThresholdType;
	
	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);
			threshold = config.getDouble(MotifConstants.PARAM_THRESHOLD);
			order = config.getString(NSConstants.PARAM_ORDER);
			method = config.getInt(NSConstants.PARAM_RANDOM_METHOD);
			setNum = config.getInt(MotifConstants.PARAM_SET_NUM);
			seqThresholdType = config.getInt(MotifConstants.PARAM_SEQ_THRESHOLD_TYPE);
			kmerThresholdType = config.getInt(MotifConstants.PARAM_KMER_THRESHOLD_TYPE);

			boolean doubleLineBreak = config.getBoolean(MotifConstants.PARAM_DOUBLE_LINEBREAK);
		
			FileInputStream fis = null;
			DataInputStream dis = null;
			
			File file = new File(
					MotifConstants.getConstrainedTreesFilename(
							order, method, setNum, threshold, kmerThresholdType, seqThresholdType));

			if (file.exists()){
				try{
					fis = new FileInputStream(file);
					dis = new DataInputStream(fis);
		
					String dataString, virtualSequence;
					Pattern p = Pattern.compile("(\\d+)\\|(\\d)\\-(\\d)");
					Matcher m;
					Integer idInFile;
					int kmin, kmax, maxK = 0;
					
					dataString = NSDataReader.readLine(dis, doubleLineBreak);
					
					while (dataString != null){
						//System.out.println(dataString);
						m = p.matcher(dataString);
				        
				        if (m.matches()){
				        	// Get sequence info
				        	idInFile = new Integer(m.group(1));
				        	kmin = Integer.parseInt(m.group(2));
				        	kmax = Integer.parseInt(m.group(3));
				        	
				        	// Get and decompress virtual sequence
				        	virtualSequence = GACTCompressor.decompressSequence(dis);
				        	if (kmin >= maxK){
				        		maxK = kmin;
				        		writeRealVirtualPair(idInFile, kmin, kmax, virtualSequence);
				        	}
				        	
				        }
				        dataString = NSDataReader.readLine(dis, doubleLineBreak);
					};
					
				} catch (IOException ioe){
					throw new Exception(ioe);
				} finally{
					if (dis != null){
						dis.close();
					}
					if (fis != null){
						fis.close();
					}
				}
			}
			else{
				System.out.println("File "+file.getName()+" does not exist");
			}
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	private static void writeRealVirtualPair(
			Integer sequenceId, int kmin, int kmax, String virtualSequence) throws Exception{
		String compressedSequence = GACTCompressor.compressSequenceIntoString(virtualSequence);
		String filename = DataProperties.getInstance().getFastaRandomsDir()+"/Method"+method+
		"/Mapping2/constrainedTrees"+order+"-set"+setNum+"-threshold-"+threshold
			+ MotifConstants.getThresholdTypeString(kmerThresholdType, seqThresholdType)
			+ "Stripped.dat";

		DataWriter.appendStringToFile(filename, 
				sequenceId + "|" + kmin + "-" + kmax + "\n" +
				compressedSequence + "\n");
	}

	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = MotifConstants.getParameters();
        
        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + ConstraintsResultDeleter.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}
}
