package org.cn3.ns.motif.generation;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.MetaDataManager;
import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.DistributionBean;
import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.data.persistence.GACTCompressor;
import org.cn3.ns.generation.NodeSequenceBinaryTreeGenerator;
import org.cn3.ns.motif.analysis.MotifFinder;
import org.cn3.ns.resources.DataProperties;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.MappingUtils;
import org.cn3.ns.utils.NeuriteSequenceStringUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 * Replaced by ConstrainedSurrogateGenerator
 *
 */
public class DetermineSufficientConstraints2 {
	
	private static final boolean WRITE_DATA = true;
	private static boolean useSeconds = true;
	
	//private static final int MAX_COUNT_STORAGE_DEGREE = 18;
	private static final String PARAM_HOUR_CUTOFF = "hour-cutoff";
	private static final String PARAM_MAX_CONSTRAINT = "max-constraint";
	private static final String PARAM_START = "start";
	private static final String PARAM_MAX_PROCESSORS = "max-processors";
	private static final String PARAM_SEQ_LEN_START = "seqLengthStart";
	private static final String PARAM_SEQ_LEN_STOP = "seqLengthStop";
	
	//private static final int ARBORS_PER_CLASS = 400;
	private static final int MAX_SEQUENCE_LENGTH = 900;
	private static final int HOUR_CUTOFF = 7;
	
	private static final int DEFAULT_MAX_CONSTRAINT = 3;

	private static int hourCutoff;
	private static double threshold;
	private static int kmerThresholdType = MotifConstants.KMER_THRESHOLD_NONE;
	private static int seqThresholdType = MotifConstants.SEQ_THRESHOLD_WHOLE;
	private static String thresholdTypeString = null;
	private static int method;
	private static String order;
	private static int setNum;
	private static int mapping = NSConstants.MAPPING_MODE_TOPO_ACT;
	private static int maxConstraint = DEFAULT_MAX_CONSTRAINT;
	private static int startConstraint = 0;
	private static int maxCheck;
	private static int maxProcessors;
	private static int debug = 0;
	private static boolean doubleLineBreak = false;
	private static ConstraintRunnable[] runnables;
	private static String lineBreak;
	
	// Reuse trees when picking up at a new constraint level
	private static Map startSeqMap = new HashMap();

	// Shared between threads
	private static volatile MyArrayList currentTreeList;
	private static List sequenceNames;
	private static List[] removedSequences;
	private static Map realTreeMotifCounts;
	private static MyCCountMap cCountSequenceMap;
	private static List[] motifsByLength;
	private static Set[] virtualFoundForReal;
	private static boolean[][] results;
	private static DistributionBean[] summary;
	private static double[] countThreshold;
	private static File timeRecordFile;
	private static int numThreads;
	private static CountDownLatch latch;

	// match check variables determined at each k
	private static int[] kmerFailCutoff;
	private static double[] sumDiffThreshold;
	
	// Shared benchmark variables
	private static long timeCheck = 1000*60*60; // every hour
	private static volatile long lastCheck, attempts;
	private static int hours = 0;
	
	public static void main(String args[]) throws Exception{
		JSAPResult config = readParameters(args);
		threshold = config.getDouble(MotifConstants.PARAM_THRESHOLD);
		seqThresholdType = config.getInt(MotifConstants.PARAM_SEQ_THRESHOLD_TYPE);
		if (seqThresholdType != MotifConstants.SEQ_THRESHOLD_WHOLE){
			kmerThresholdType = config.getInt(MotifConstants.PARAM_KMER_THRESHOLD_TYPE);
		}
		thresholdTypeString = MotifConstants.getThresholdTypeString(kmerThresholdType, seqThresholdType);
		method = config.getInt(NSConstants.PARAM_RANDOM_METHOD);
		order = config.getString(NSConstants.PARAM_ORDER);
		setNum = config.getInt(MotifConstants.PARAM_SET_NUM);
		hourCutoff = config.getInt(PARAM_HOUR_CUTOFF);
		startConstraint = config.getInt(PARAM_START);
		maxConstraint = config.getInt(PARAM_MAX_CONSTRAINT);
		maxCheck = maxConstraint+1;
		maxProcessors = config.getInt(PARAM_MAX_PROCESSORS);
		debug = config.getInt(NSConstants.PARAM_DEBUG);
		doubleLineBreak = config.getBoolean(MotifConstants.PARAM_DOUBLE_LINEBREAK);
		if (doubleLineBreak){
			lineBreak = "\r\n";
		}
		else{
			lineBreak = "\n";
		}
		
		System.out.println("Initializing...");
		
		StringBuffer sb;
		Integer lengthObj, cCount;
		
		// Multi-core classes
		if (maxProcessors == 0){
			maxProcessors = Runtime.getRuntime().availableProcessors();
		}
		else{
			maxProcessors = Math.min(Runtime.getRuntime().availableProcessors(), maxProcessors);
		}
		ExecutorService executor = Executors.newFixedThreadPool(maxProcessors);
		runnables = new ConstraintRunnable[maxProcessors];

		Integer sequenceId;
		
		int length, lengthIndex;
		countThreshold = new double[maxCheck+1];
		Set motifs;
		removedSequences = new List[maxCheck];
		for (int i = 0; i < maxCheck; i++){
			removedSequences[i] = new ArrayList();
		}

		// Get sequence names and map them to unique ids
		sequenceNames = NSDataReader.loadSequenceList(mapping);
		Map sequenceIdMap = new HashMap();
		for (int j = 0; j < sequenceNames.size(); j++){
			sequenceIdMap.put(sequenceNames.get(j), new Integer(j));
		}
		int databaseSize = sequenceNames.size();
		
		System.out.println("Loading tree size data");
		// Loading tree sizes represented in database
		List sequenceLengths = NSDataReader.loadUniqueSequenceLengths(mapping);

		// Loading map keyed by seq length, values are lists of sequence ids of seqs of that length
		Map lengthArborsMap = loadTreeSizeArborizationsMap(sequenceIdMap);
		
		// Array for each k (n-mer length) of Sets containing seq names for which virtual trees have been found
		results = new boolean[databaseSize][maxCheck];
		summary = new DistributionBean[maxCheck];
		
		// Initializing
		motifsByLength = new List[maxCheck+1];
		kmerFailCutoff = new int[maxCheck+1];
		sumDiffThreshold = new double[maxCheck+1];
		for (int i = 1; i <= maxCheck; i++){
			if (i > 1){
				motifsByLength[i] = NeuriteSequenceStringUtils.generatePossibleACTMotifs(i);
			}
			else{
				motifsByLength[1] = new ArrayList();
				motifsByLength[1].add("C");
			}
			// Determine the number of kmer misses that put the miss percent (for that k) over the threshold 
			if (seqThresholdType == MotifConstants.SEQ_THRESHOLD_CUTOFF){
				kmerFailCutoff[i] = (int)(threshold * motifsByLength[i].size());
			}
			// Determine the total error that is the point of no return for k. 
			else if (seqThresholdType == MotifConstants.SEQ_THRESHOLD_AVG){
				sumDiffThreshold[i] = threshold * motifsByLength[i].size();
				System.out.println("SumDiffThreshold for "+i+": "+sumDiffThreshold[i]);
			}
		}

		// Load real tree n-mer counts into sequence id keyed map
		System.out.println("Loading real tree motif counts");
		realTreeMotifCounts = loadRealTreeMotifCounts(sequenceIdMap, motifsByLength);

		// Load results from prior runs
		System.out.println("Loading completions from prior runs");
		virtualFoundForReal = loadCompletions(results, summary, realTreeMotifCounts);

		// Create file for recording completion times and counts
		String filename = "constraintTimeSet"+setNum+order+"-Thresh"+threshold+thresholdTypeString;
		filename = filename + ".csv";
		timeRecordFile = new File(filename);
		System.out.println("Time Record File: "+filename);
		if (!timeRecordFile.exists()){
			System.out.println("File doesn't exist, creating it");
			useSeconds = true;
			DataWriter.writeStringToFile(timeRecordFile,
				"Sequence,Constraint,MaxMatch,Length,Attempts,Time.sec,Threads\n");
		}
		else{
			FileReader freader = null;
			BufferedReader reader = null;
			String[] header = null;

			try{
				freader = new FileReader(timeRecordFile);
			    reader = new BufferedReader(freader);
			    
				String dataString = "";
				
				if ( (dataString = reader.readLine()) != null ) { 
					header = dataString.split(",");
					if (header[5].equals("Time.min")){
						useSeconds = false;
					}
				}
			
			} catch (IOException ioe){
				throw new Exception(ioe);
			} finally{
				if (reader != null){
					reader.close();
				}
				if (freader != null){
					freader.close();
				}
			}
		}
		
		NumberFormat nf = NumberFormat.getInstance();
		DateFormat dateFmt = DateFormat.getDateTimeInstance();
		nf.setMaximumFractionDigits(2);

		// Remove lengths that are too big
		System.out.println("Removing very large sequences (larger than "+MAX_SEQUENCE_LENGTH+")");
		for (lengthIndex = sequenceLengths.size()-1; ((Integer)sequenceLengths.get(lengthIndex)).intValue() > MAX_SEQUENCE_LENGTH; lengthIndex--){
			length = ((Integer)sequenceLengths.get(lengthIndex)).intValue();
			removedSequences[0].addAll((List)lengthArborsMap.remove(new Integer(length)));
			sequenceLengths.remove(lengthIndex);
		}
		// Remove sequences that are no longer in the database
		sb = new StringBuffer();
		for (int i = 0; i < sequenceNames.size(); i++){
			String sequenceName = (String)sequenceNames.get(i);
			sequenceId = (Integer)sequenceIdMap.get(sequenceName);
			if (!realTreeMotifCounts.containsKey(sequenceId)){
				for (int j = 0; j < maxCheck; j++){
					removedSequences[j].add(sequenceId);
					sb.append(sequenceName+"  ");
				}
			}
		}
		System.out.println("Sequences no longer in db: "+sb.toString());
		
		// Remove sequences that are "outliers" (take too long to process)
		System.out.println("Loading and pulling out \"outliers\"");
		Map outliers = NSDataReader.loadOutliers(mapping, order, method, threshold, 
				thresholdTypeString, 60*hourCutoff);
		Object key;
		sb = new StringBuffer();
		for (Iterator it = outliers.keySet().iterator(); it.hasNext();){
			key = it.next();
			removedSequences[((Integer)outliers.get(key)).intValue()]
			                 .add(sequenceIdMap.get(key));
			sb.append(key+"  ");
		}
		System.out.println("Outliers: "+sb.toString());
		
		// Add flagged neurons to removedSequences lists
		List flaggedSequences = NSDataReader.getFlaggedSequences();
		for (int i = 0; i < flaggedSequences.size(); i++){
			removedSequences[0].add(flaggedSequences.get(i));
		}
		
		// Determine what sequenceLength to start on and which to stop on
		int seqLenStartIndex = 0;
		if (config.userSpecified(PARAM_SEQ_LEN_START)){
			int seqLenStart = config.getInt(PARAM_SEQ_LEN_START);
			int index = -1;
			int seqLen = 0;
			do {
				index++;
				seqLen = ((Integer)sequenceLengths.get(index)).intValue();
			} while (seqLenStart > seqLen);
			seqLenStartIndex = index;
		}
		int seqLenStopIndex = sequenceLengths.size()-1;
		if (config.userSpecified(PARAM_SEQ_LEN_STOP)){
			int seqLenStop = config.getInt(PARAM_SEQ_LEN_STOP);
			int seqLen, index = sequenceLengths.size();
			do {
				index--;
				seqLen = ((Integer)sequenceLengths.get(index)).intValue();
			} while (seqLenStop < seqLen);
			seqLenStopIndex = index;
		}
		System.out.println("Start at length: "+sequenceLengths.get(seqLenStartIndex)+
				"; Stop at length: "+sequenceLengths.get(seqLenStopIndex));
		
		// Loop through constraint lengths
		for (int k = startConstraint; k <= maxConstraint; k++){
			System.out.println("K: "+k+"; Hour Cutoff: "+hourCutoff);
			
			// Set up motifs set for this k
			motifs = new HashSet();
			for (int i = k; i <= maxCheck; i++){
				if (i > 0) motifs.addAll(motifsByLength[i]);
			}
			// Loop through sequence lengths to set the threshold at which the difference count causes a virtual sequence not to match
			for (lengthIndex = seqLenStartIndex; lengthIndex <= seqLenStopIndex; lengthIndex++){
				lengthObj = (Integer)sequenceLengths.get(lengthIndex);
				length = lengthObj.intValue();
				
				// Threshold/2 since it will be +/-
				if (seqThresholdType == MotifConstants.SEQ_THRESHOLD_WHOLE 
						|| seqThresholdType == MotifConstants.SEQ_THRESHOLD_WHOLE_EFF1
						|| seqThresholdType == MotifConstants.SEQ_THRESHOLD_WHOLE_EFF2){
					for (int k2 = k; k2 <= maxCheck; k2++){
						if (k2 == 1){
							// Only dealing with k, so DON'T multiply threshold by 2
							countThreshold[k2] = threshold * (length - k2 + 1);
						}
						else{
							countThreshold[k2] = 2 * threshold * (length - k2 + 1);

							if (seqThresholdType == MotifConstants.SEQ_THRESHOLD_WHOLE_EFF1){
								// Logic for this one is currently unclear
								countThreshold[k2] = countThreshold[k2] * k2;
							}
							else if (seqThresholdType == MotifConstants.SEQ_THRESHOLD_WHOLE_EFF2){
								// Two locations * total affected positions per location
								countThreshold[k2] = countThreshold[k2] * (2 * (2 * k2 - 2));
							}
						}
					}
				}
				if (kmerThresholdType == MotifConstants.KMER_THRESHOLD_ABSOLUTE){
					for (int k2 = k; k2 <= maxCheck; k2++){
						countThreshold[k2] = threshold * (length - k2 + 1);
					}
				}
				
				currentTreeList = new MyArrayList();
				currentTreeList.addAll((List)lengthArborsMap.get(new Integer(length)));
				
				// Remove trees that have already found matching virtual trees at this k
				currentTreeList.removeAll(virtualFoundForReal[k]);
				// Remove trees that take too long
				for (int i = 0; i <= k; i++){
					currentTreeList.removeAll(removedSequences[i]);
				}

				System.out.println("Length: "+length+" ("+(lengthIndex+1)+"/"+sequenceLengths.size()+"); k "+k+"; "+currentTreeList.size()+" trees"
						+"; Started at "+ dateFmt.format(new Date()));
				sb = new StringBuffer();
				/*for (Iterator it = virtualFoundForReal[k].iterator(); it.hasNext();){
					sb.append("  "+it.next());
				}*/
				for (Iterator it = currentTreeList.iterator(); it.hasNext();){
					sb.append("  "+it.next());
				}
				if (currentTreeList.size() > 0){
					System.out.println(sb.toString());
				}

				if (!currentTreeList.isEmpty()){
					
					cCountSequenceMap = new MyCCountMap();
					if (k > 1){
						// Get sequences into lists based on C node counts
						for (int i = 0; i < currentTreeList.size(); i++){
							cCount = (Integer)((Map)realTreeMotifCounts.get(currentTreeList.get(i))).get("C");
							if (!cCountSequenceMap.containsKey(cCount)){
								cCountSequenceMap.put(cCount, new ArrayList());
							}
							((List)cCountSequenceMap.get(cCount)).add(currentTreeList.get(i));
						}
					}
					
					// If starting at constraint > 0, see if we can reuse sequences from prior runs
					if (startConstraint > 0 && k == startConstraint){
						reusePriorSuccesses(length, motifs);
					}
					
					hours = 0;
					lastCheck = new Date().getTime();
					
					// Generate trees until all real trees have a matching virtual tree
					// Only use all cores if k > 0
					if (k > 1){
						numThreads = Runtime.getRuntime().availableProcessors();
						if (maxProcessors > 0 && maxProcessors < numThreads){
							numThreads = maxProcessors;
						}
						System.out.println("numThreads: "+numThreads);
					}
					else{
						numThreads = 1;
					}
	
					// Start thread processes to find constrained trees
					latch = new CountDownLatch(numThreads);
					for (int i = 0; i < numThreads; i++){
						runnables[i] = new ConstraintRunnable(length, k, motifs);
						runnables[i].setThreadNum(i);
						executor.execute(runnables[i]);
					}

					// Wait until all threads are finish
					latch.await();
					System.out.println("Finished all threads");
	
					// Grab summary data from all runs
					attempts = 0;
					for (int i = 0; i < numThreads; i++){
						attempts += runnables[i].getLocalAttempts();
			
						runnables[i] = null;
					}
					// Clean up from threads
					//System.gc();
	
					/* If cCountSequenceMap isn't empty, all threads have exited without finishing 
						due to some internal exception (which will be printed) */
					if (cCountSequenceMap.size() > 0){
						throw new Exception("Thread failure: cCountSequenceMap not empty");
					}
				}
			}
			System.out.println("For "+k+": percent constrained = "+nf.format(100*summary[k].getMean()));
		}
		
		// Write results to a csv file
		writeResults(sequenceNames, results, removedSequences);
		executor.shutdownNow();
		executor = null;
		System.gc();
	}
	
	private static long getTotalAttempts(){
		long totalAttempts = 0;
		for (int i = 0; i < numThreads; i++){
			if (runnables[i] != null){
				totalAttempts += runnables[i].getLocalAttempts();
			}
		}
		return totalAttempts;
	}
	
	private static void reusePriorSuccesses(int length, Set motifs) throws Exception{
		Integer sequenceId;
		Map realMotifCounts, virtualMotifCounts;
		String virtualSequence;
		int k = startConstraint, k2;
		boolean isMatch;
		
		// Loop through trees in this group for those with a match at k==startConstraint
		for (int i = 0; i < currentTreeList.size(); i++){
			sequenceId = (Integer)currentTreeList.get(i);
			if (startSeqMap.containsKey(sequenceId)){
				virtualSequence = (String)startSeqMap.get(sequenceId);
				realMotifCounts = (Map)realTreeMotifCounts.get(sequenceId);
				if (realMotifCounts == null){
					System.out.println(sequenceNames.size()+"; "+sequenceId);
					System.out.println("NULL - "+sequenceId + " " + sequenceNames.get(sequenceId.intValue()));
					currentTreeList.remove(i);
					i--;
				}
				else{
					k2 = k+1;
					// Check from k+1 up to maxCheck
					do {
						virtualMotifCounts = MotifFinder.findMotifs(virtualSequence, k, maxCheck, motifs);

						isMatch = (k2==0) || isMatch(realMotifCounts, virtualMotifCounts, countThreshold[k2], 
								motifsByLength[k2], k2, length-k2+1);
						if (isMatch){
							// A match has been found for this sequence at this k, no need to check it again
							if (k2 < maxCheck){
								virtualFoundForReal[k2].add(sequenceId);
							}
							// Successful constraint
							if (k2 > k){
								results[sequenceId.intValue()][k2-1] = true;
								summary[k2-1].addData(1);
							}
							k2++;
						}
						// If match found at lower level but not for this level, then constraint failed
						else if (k2 > k){
							summary[k2-1].addData(0);
							// results array defaults to "false", no need to set
						}
					} while (isMatch && k2 <= maxCheck);
					
					if (k2 > k){
						currentTreeList.remove(sequenceId);
						// sequenceId is removed, to decrement i to continue
						i--;
	
						// Write real/virtual pair
						writeRealVirtualPair(
								sequenceId,
								k,k2-1,
								virtualSequence);
					}
				}
			}
		}
	}
	
	private static class MyArrayList extends ArrayList {
		static final long serialVersionUID = 1;
		
		public synchronized boolean checkAndRemoveId(Integer sequenceId){
			if (this.contains(sequenceId)){
				// Remove the tree first so others don't use it
				this.remove(sequenceId);
				return true;
			}
			return false;
		}		
	}

	private static class MyCCountMap extends HashMap {
		static final long serialVersionUID = 1;
		
		public synchronized boolean checkAndRemoveId(Integer cCount, Integer sequenceId){
			List cCountSequences;
			if (this.containsKey(cCount)){
				cCountSequences = (List)this.get(cCount);
				if (cCountSequences.contains(sequenceId)){
					cCountSequences.remove(sequenceId);
					if (cCountSequences.size() == 0){
						this.remove(cCount);
					}
					// Remove the tree first so others don't use it
					return true;
				}
			}
			return false;
		}		
	}

	private static class ConstraintRunnable implements Runnable {
		private int threadNum;
		private int k;
		private int length;
		private int degree;
		private Set motifs;
		private long localAttempts;
		
		private int localDebug;
		
		ConstraintRunnable(int length, int k, Set motifs){
			this.k = k;
			this.length = length;
			this.motifs = motifs;
			degree = length + 1;
			localDebug = debug;
		}
		
		public void setThreadNum(int threadNum){
			this.threadNum = threadNum;
		}
		
		public long getLocalAttempts() {
			return localAttempts;
		}

		public void setLocalAttempts(int localAttempts) {
			this.localAttempts = localAttempts;
		}

		private long checkTime() throws Exception{
			long currentTime = new Date().getTime();
			if ((currentTime - lastCheck) > timeCheck){
				lastCheck = currentTime;
				hours++;
				System.out.println(hours + " hours; "+currentTreeList.size() + " remaining sequences; "+getTotalAttempts()+" trees tested");
				// If it's taking too long, move it to the removed list

				if (hours > hourCutoff){
					StringBuffer sb = new StringBuffer();
					long minutes = hours * 60 + (int)(((double)(currentTime - lastCheck))/60000);
					synchronized (currentTreeList){
						for (int i = 0; i < currentTreeList.size(); i++){
							// SequenceName, k, minutes, attempts
							sb.append(sequenceNames.get(((Integer)currentTreeList.get(i)).intValue())
									+","+k+","+length+","+(minutes)+","+getTotalAttempts()+","+setNum+","+numThreads+"\n");
						}
						removedSequences[k].addAll(currentTreeList);
						currentTreeList.removeAll(currentTreeList);
					}
					DataWriter.appendStringToFile(
							DataProperties.getInstance().getOutlierFile(mapping, order, method, threshold, thresholdTypeString),
							sb.toString());
				}
			}
			return currentTime;
		}
		
		private void print(String debugStr){
			System.out.println("Thread "+threadNum+": "+debugStr);
		}
		private void trace(String debugStr){
			debug(debugStr, 2);
		}
		private void debug(String debugStr){
			debug(debugStr, 1);
		}
		private void debug(String debugStr, int level){
			if (localDebug >= level){
				System.out.println("Thread "+threadNum+": "+debugStr);
			}
		}
		
		private Integer getNextCCount(){
			int currentIndex = (int)(cCountSequenceMap.size()*Math.random());
			Integer cCountObj = (Integer)(new ArrayList(cCountSequenceMap.keySet())).get(currentIndex);
			return cCountObj;
		}
		
		public void run(){
			if (k <= 1){
				runSequencesSeparately();
			}
			else{
				runGroupByC();
			}
		}
		// Use when k=0,1
		private void runSequencesSeparately(){
			Integer sequenceId, cCountObj;
			List sequences;
			Map realMotifCounts, virtualMotifCounts = null;
			boolean isMatch;
			String virtualSequence = null;
			int maxFound, realCCount = 0, k2;
			BinaryTreeNode tree;
			
			sequences = new ArrayList(currentTreeList);
			localAttempts = 1;

			try{
				// Loop through the sequences with the same length and %C to see if the generated sequence matches
				for (int i = 0; i < sequences.size(); i++){
					sequenceId = (Integer)sequences.get(i);
					realMotifCounts = (Map)realTreeMotifCounts.get(sequenceId);
					cCountObj = (Integer)realMotifCounts.get("C");
					realCCount = cCountObj.intValue();
					if (realMotifCounts == null){
						System.out.println("NULL - "+sequenceId + " " + sequenceNames.get(sequenceId.intValue()));
						currentTreeList.remove(sequenceId);
						sequences.remove(sequenceId);
						i--;
					}
					else{
						// If not reusing generated trees (k <= 1), generate random tree number and determine its motif counts
						if (k == 0){
							// Generate random tree number and determine its motif counts
							virtualSequence = NodeSequenceBinaryTreeGenerator.generateRandomHardingTreeMap2Sequence(degree, order);
							virtualMotifCounts = MotifFinder.findMotifs(virtualSequence, k, maxCheck, motifs);
						}
						else if (k == 1){
							tree = NodeSequenceBinaryTreeGenerator.generateRandomMonomerConstrainedHardingTreeByDegree(degree, realCCount);
							virtualSequence = MappingUtils.traverseTreeToStringWithMappingMode2(tree, order);
							virtualMotifCounts = MotifFinder.findMotifs(virtualSequence, k, maxCheck, motifs);
						}

						
						k2 = k;
						// Check
						do {
							//trace("k: "+k+"; k2: "+k2);
							isMatch = (k2==0 || 
									  // Generation method assures this
									  (k==1 && k2==1) || 
									  // Make sure C counts match if generating on length and checking on monomers
									  (k==0 && k2==1 && isCMatch(realMotifCounts,virtualMotifCounts)) ||
									  // Use thresholds for other cases
									  (isMatch(realMotifCounts, virtualMotifCounts, countThreshold[k2], 
									motifsByLength[k2], k2, length-k2+1)));
							if (isMatch){
								k2++;
							}
						} while (isMatch && k2 <= maxCheck);
						maxFound = k2-1;
						
						// Thread-safe, operating on local tree list
						if (maxFound >= k && currentTreeList.checkAndRemoveId(sequenceId)){
							// Update results: Write time taken, update summary and result variables, and write the result to file
							updateResults(sequenceId, k, maxFound, virtualSequence);
						}
						// Look to see how long it's been for reporting and to stop if it's taking too long
						if (threadNum == 0) checkTime();
					}
				}
			} catch (Exception e){
				e.printStackTrace();
			} finally {
				//System.out.println("Exiting ConstraintRunnable. Countdown now at "+(latch.getCount()-1));
				latch.countDown();
				debug("Leaving");
			}
		}
		
		// Use when k>1
		public void runGroupByC(){
			Integer sequenceId, cCountObj;
			List sequences;
			Map realMotifCounts, virtualMotifCounts = null;
			boolean isMatch;
			String virtualSequence = null;
			int maxFound, realCCount = 0, k2;
			BinaryTreeNode tree;
			
			// Get stated with a particular cCount list
			cCountObj = getNextCCount();
			sequences = (List)cCountSequenceMap.get(cCountObj);
			realCCount = cCountObj.intValue();
			debug("Starting with cCount: "+realCCount+"; "+sequences.size()+" sequences");
			localAttempts = 0;

			try{
				while (cCountSequenceMap.size() > 0){
					localAttempts++;
					
					// Generate random tree number and determine its motif counts
					tree = NodeSequenceBinaryTreeGenerator.generateRandomMonomerConstrainedHardingTreeByDegree(degree, realCCount);
					virtualSequence = MappingUtils.traverseTreeToStringWithMappingMode2(tree, order);
					virtualMotifCounts = MotifFinder.findMotifs(virtualSequence, k, maxCheck, motifs);
					
					// Update local tree list with current global tree list
					debug("Created virtual tree; real trees left: "+currentTreeList.size());
					
					// Loop through the sequences with the same length and %C to see if the generated sequence matches
					for (int i = 0; i < sequences.size() && cCountSequenceMap.containsKey(cCountObj); i++){
						sequenceId = (Integer)sequences.get(i);
						realMotifCounts = (Map)realTreeMotifCounts.get(sequenceId);
						cCountObj = (Integer)realMotifCounts.get("C");
						realCCount = cCountObj.intValue();
						if (realMotifCounts == null){
							System.out.println("NULL - "+sequenceId + " " + sequenceNames.get(sequenceId.intValue()));
							currentTreeList.remove(sequenceId);
							sequences.remove(sequenceId);
							i--;
						}
						else{
							k2 = k;
							// Check
							do {
								//trace("k: "+k+"; k2: "+k2);
								isMatch = (k2==0 || (k2==1 && k==1)) || isMatch(realMotifCounts, virtualMotifCounts, countThreshold[k2], 
										motifsByLength[k2], k2, length-k2+1);
								if (isMatch){
									k2++;
								}
							} while (isMatch && k2 <= maxCheck);
							maxFound = k2-1;
							
							// Thread-safe, operating on local tree list
							if (maxFound >= k && cCountSequenceMap.checkAndRemoveId(cCountObj, sequenceId)){
								currentTreeList.remove(sequenceId);
								if (currentTreeList.size() == 0 && cCountSequenceMap.size() != 0){
									System.out.println("CurrentTreeList now empty, but cCountSequenceMap isn't;");
									for (Iterator it = cCountSequenceMap.keySet().iterator(); it.hasNext();){
										Integer count = (Integer)it.next();
										System.out.println(count+" has "+((List)cCountSequenceMap.get(count)).size());
									}
								}
								// Update results: Write time taken, update summary and result variables, and write the result to file
								updateResults(sequenceId, k, maxFound, virtualSequence);
							}
						}
					}
					// Look to see how long it's been for reporting and to stop if it's taking too long
					if (threadNum == 0) checkTime();

					// If no more sequences with the current cCount, move on to another
					if (cCountSequenceMap.size() > 0){
						cCountObj = getNextCCount();
						sequences = (List)cCountSequenceMap.get(cCountObj);
						realCCount = cCountObj.intValue();
						debug("Moving on to cCount: "+realCCount+"; "+sequences.size()+" sequences; cCountSequenceMap size: "+cCountSequenceMap.size());
					}
				}
			} catch (Exception e){
				e.printStackTrace();
			} finally {
				//System.out.println("Exiting ConstraintRunnable. Countdown now at "+(latch.getCount()-1));
				latch.countDown();
				debug("Leaving");
			}
		}
		// Update results: Write time taken, update summary and result variables, and write the result to file
		private void updateResults(Integer sequenceId, int k, int maxFound, String virtualSequence) throws Exception{
			// Write time taken
			long currentTime = new Date().getTime();
			long seconds = hours * 3600 + (int)(((double)(currentTime - lastCheck))/1000);
			long minutes = seconds / 60;
			if (WRITE_DATA){
				if (useSeconds){
					DataWriter.appendStringToFile(timeRecordFile, 
							sequenceNames.get(sequenceId.intValue())
							+","+k+","+maxFound+","+length+","+getTotalAttempts()+","+seconds+","+numThreads+"\n");
				}
				else{
					DataWriter.appendStringToFile(timeRecordFile, 
							sequenceNames.get(sequenceId.intValue())
							+","+k+","+maxFound+","+length+","+getTotalAttempts()+","+minutes+","+numThreads+"\n");
				}
			}

			if (minutes > 60){
				print("Found "+sequenceId+" from "+k+" to "+maxFound+"; Took "+minutes +" minutes");
			}
			else if (seconds > 30){
				debug("Found "+sequenceId+" from "+k+" to "+maxFound+"; Took "+minutes +" minutes");
			}
			
			// Go through k values to update tracking data
			for (int j = k; j < maxFound; j++){ // Only hits k if (maxFound > k)
				// A match has been found for this sequence at this k, no need to check it again
				virtualFoundForReal[j].add(sequenceId);
				// Successful constraint
				results[sequenceId.intValue()][j] = true;
				summary[j].addData(1);
			}
			// If max match found at level lower than max check, then constraint failed for leve of max found
			if (maxFound < maxCheck){
				// Next level still needs to know that it is done with this sequence
				virtualFoundForReal[maxFound].add(sequenceId);
				summary[maxFound].addData(0);
			}

			// Write real/virtual pair
			writeRealVirtualPair(
					sequenceId,
					k,maxFound,
					virtualSequence);
		}
	}
	
	
	
	private static void writeResults(List sequenceNames, boolean[][] results, List[] removedSequences)
	throws Exception{
		Map metaData = MetaDataManager.getCellMetaData();
		NeuronMetadata neuron;
		boolean na;
		for (int i = 1; i < maxCheck; i++){
			removedSequences[0].addAll(removedSequences[i]);
		}
		Map sequenceLengthsMap = NSDataReader.loadLengthDatabase(mapping);
		String sequenceName;
		StringBuffer sb = new StringBuffer();
		sb.append("SequenceName,Length,ArborType,CellType");
		for (int j = 0; j <= maxConstraint; j++){
			sb.append(",L"+j);
		}
		sb.append("\n");
		for (int i = 0; i < results.length; i++){
			if (!removedSequences[0].contains(new Integer(i))){
				sequenceName = (String)sequenceNames.get(i);
				sb.append(sequenceNames.get(i));
				sb.append(","+sequenceLengthsMap.get(sequenceName));
				sb.append(","+NeuriteSequenceStringUtils.getArborizationType(sequenceName));
				neuron = (NeuronMetadata)metaData.get(NeuriteSequenceStringUtils.getNeuronNameFromSequence(sequenceName));
				sb.append(","+neuron.getCellType());
				na = false;
				for (int j = 0; j < results[i].length; j++){
					if (removedSequences[j].contains(new Integer(i))) na = true;
					if (na){
						sb.append(",NA");
					}
					else{
						sb.append(","+(results[i][j]?1:0));
					}
				}
				sb.append("\n");
			}
		}
		String filename = "/NMerConstrainResults-set"+setNum+"-thresh"
			+threshold + thresholdTypeString + ".csv";
		DataWriter.writeStringToFile(
				DataProperties.getInstance().getResultsDir()+filename,
				sb.toString());
	}
	
	private static Map loadRealTreeMotifCounts(Map sequenceIdMap, List[] motifsByLength) throws Exception{
		Map realTreeMotifCounts = new HashMap();
		FastaReader fr = new FastaReader();
		Map motifCounts;
		
		fr.setFile(DataProperties.getInstance().getFastaArborDir()+"Mapping"+mapping+"/Size/NMArborSize"+order+"Map"+mapping+".fasta");
		List sequenceList = NSDataReader.loadSequenceList(mapping);
		while (fr.next()){
			// Avoids case of using updated fasta file without updated lengths file
			if (sequenceList.contains(fr.getSequenceName())){
				motifCounts = MotifFinder.findMotifs(fr.getSequence(),1,maxCheck);
				for (int i = 1; i <= maxCheck; i++){
					for (int j = 1; j < motifsByLength[i].size(); j++){
						if (!motifCounts.containsKey(motifsByLength[i].get(j))){
							motifCounts.put(motifsByLength[i].get(j), new Integer(0));
						}
					}
				}
				realTreeMotifCounts.put(sequenceIdMap.get(fr.getSequenceName()),
					motifCounts);
			}
		}
		return realTreeMotifCounts;
	}

	private static boolean isCMatch(Map realCounts, Map virtualCounts){
		int realC, virtualC;
		realC = ((Integer)realCounts.get("C")).intValue();
		virtualC = ((Integer)virtualCounts.get("C")).intValue();
		return realC == virtualC;
	}
	// Determines whether the real and virtual motif counts are sufficiently similar (or match)
	private static boolean isMatch(Map realCounts, Map virtualCounts, double countThreshold, List motifs, int k, int nmersPerSequence){
		int diff, realKmerCount;
		String motif;
		double diffThreshold, sumDiffCountThreshold = nmersPerSequence*sumDiffThreshold[k];
		DistributionBean errors = new DistributionBean();
		// Used either to count number of distinct k-mers missed OR to keep track of total number off across k-mers
		int misses = 0;

		for (int i = 0; i < motifs.size(); i++){
			motif = (String)motifs.get(i);
			// Pull real kmer count 
			realKmerCount = ((Integer)realCounts.get(motif)).intValue();
			// Take the difference between real and virtaul count
			diff = Math.abs(
					realKmerCount - ((Integer)virtualCounts.get(motif)).intValue());
			// Whole sequence
			if (seqThresholdType == MotifConstants.SEQ_THRESHOLD_WHOLE
					|| seqThresholdType == MotifConstants.SEQ_THRESHOLD_WHOLE_EFF1
					|| seqThresholdType == MotifConstants.SEQ_THRESHOLD_WHOLE_EFF2){
				// Just add the count difference to the total count difference (where each count difference is a "miss")
				misses += diff;
				if (misses > countThreshold){
					return false;
				}
			}
			// Absolute: realFreq - virtualFreq
			else if (kmerThresholdType == MotifConstants.KMER_THRESHOLD_ABSOLUTE){
				// Check whether this kmer is a miss
				if (diff > countThreshold){
					misses++;
				}
				// If taking average of kmer errors, add diff to the errors
				if (seqThresholdType == MotifConstants.SEQ_THRESHOLD_AVG){
					errors.addData(diff);
					// If the total diff error will cause the avg to be greater than threshold, fail
					if (errors.getTotal() > sumDiffCountThreshold){
						return false;
					}
				}
				// If strict or using a cutoff, check whether the number of misses means no match
				else if (misses > kmerFailCutoff[k]){
					return false;
				}
			}
			// Relative (much more strict): (realFreq - virtualFreq)/realFreq
			else{
				// Relative threshold requires calculation each time
				diffThreshold = (double)realKmerCount * threshold;
				if (debug == 2){
					System.out.println(motif+" "+(((Integer)realCounts.get(motif)).intValue())+" threshold: "+diffThreshold);
				}
				if (diff > diffThreshold){
					misses++;
				}
				// Can't average relative, so check misses against cutoff
				if (misses > kmerFailCutoff[k]){
					return false;
				}
			}
		}
		
		return true;
	}
	
	// Maps sequence lengths to lists of sequence IDs associated with sequences of that length
	private static Map loadTreeSizeArborizationsMap(Map sequenceIdMap) throws Exception{
		Map treeSizeArborMapTmp = NSDataReader.loadTreeSizeArborizationsMap(mapping);
		Map treeSizeArborMap = new HashMap();
		Object key;
		List list1, list2;
		for (Iterator it = treeSizeArborMapTmp.keySet().iterator(); it.hasNext();){
			key = it.next();
			list1 = (List)treeSizeArborMapTmp.get(key);
			list2 = new ArrayList();
			for (int i = 0; i < list1.size(); i++){
				list2.add(sequenceIdMap.get(list1.get(i)));
				//System.out.println("ltsam "+list1.get(i)+" " +sequenceIdMap.get(list1.get(i)));
			}
			treeSizeArborMap.put(key, list2);
		}
		return treeSizeArborMap;
	}
	private static void writeRealVirtualPair(
			Integer sequenceId, int kmin, int kmax, String virtualSequence) throws Exception{
		if (WRITE_DATA){
			String compressedSequence = GACTCompressor.compressSequenceIntoString(virtualSequence);
			DataWriter.appendStringToFile(MotifConstants.getConstrainedTreesFilename(
					order, method, setNum, threshold, kmerThresholdType, seqThresholdType), 
					sequenceId + "|" + kmin + "-" + kmax + lineBreak +
					compressedSequence + lineBreak);
	/*		DataWriter.appendStringToFile(getConstrainedTreesFilename(), 
					sequenceId+"|"+kmin+"-"+kmax+"\n"+ virtualSequence +"\n");*/
		}
	}
	
	private static Set[] loadCompletions(boolean[][] results, DistributionBean[] summary, Map realTreeMotifCounts)
	throws Exception{
		FileInputStream fis = null;
		DataInputStream dis = null;
		int realCCount = 0, virtualCCount = 0;
		
		Set[] completions = new Set[maxCheck];
		for (int i = 0; i < maxCheck; i++){
			completions[i] = new TreeSet();
			summary[i] = new DistributionBean();
		}

		File file = new File(
				MotifConstants.getConstrainedTreesFilename(
						order, method, setNum, threshold, kmerThresholdType, seqThresholdType));
		if (file.exists()){
			try{
				fis = new FileInputStream(file);
				dis = new DataInputStream(fis);
	
				String dataString = "", virtualSequence;
				Pattern p = Pattern.compile("(\\d+)\\|(\\d)\\-(\\d)");
				Matcher m;
				int id, kmin, kmax;

				dataString = NSDataReader.readLine(dis, doubleLineBreak);
				
				while (dataString != null) {
				    m = p.matcher(dataString);
			        if (m.matches()){
			        	id = Integer.parseInt(m.group(1));
			        	kmin = Integer.parseInt(m.group(2));
			        	kmax = Integer.parseInt(m.group(3));
			        	virtualSequence = GACTCompressor.decompressSequence(dis);
			        	//System.out.println(dataString);
			        	// Make sure only exact monomer matches are used for monomer constrained sequences
			        	if (kmin == 0){
				        	virtualCCount = ((Integer)MotifFinder.findMotifs(virtualSequence,1).get("C")).intValue();
				        	realCCount = ((Integer)((Map)realTreeMotifCounts.get(new Integer(id))).get("C")).intValue();
			        	}
			        	for (int i = kmin; i < kmax && i < maxCheck; i++){
			        		// Constraint is accomplished up to kmax-1 on kmax
			        		results[id][i] = true;
			        		summary[i].addData(1);
			        		if ((kmin != 0 || i != 1) || virtualCCount == realCCount){
			        			completions[i].add(new Integer(id));
			        		}
			        	}
			        	//System.out.println(id);
			        	// Clear the line
			        	if (doubleLineBreak){
			        		dis.read(); dis.read();
			        	}
			        	
			        	if (kmax <= maxConstraint 
			        			&& (kmax != startConstraint || startConstraint == 0)){
			        		results[id][kmax] = false;
			        		summary[kmax].addData(0);
			        		if (kmin == 2 && id == 861){
			        			System.out.println("Setting as a completion");
			        		}
			        		completions[kmax].add(new Integer(id));
			        	}
			        	else if (startConstraint > 0 && kmax == startConstraint){
			        		startSeqMap.put(new Integer(id), virtualSequence);
			        	}
			        }
			        dataString = NSDataReader.readLine(dis, doubleLineBreak);
			    }
				
			} catch (IOException ioe){
				throw new Exception(ioe);
			} finally{
				if (dis != null){
					dis.close();
				}
				if (fis != null){
					fis.close();
				}
			}
		}
		return completions;
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = MotifConstants.getParameters();
        
        FlaggedOption opt;
        
        opt = new FlaggedOption(PARAM_HOUR_CUTOFF)
	        .setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault(""+HOUR_CUTOFF)
	        .setShortFlag('c') 
	        .setLongFlag(PARAM_HOUR_CUTOFF);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_MAX_CONSTRAINT)
        	.setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault(""+DEFAULT_MAX_CONSTRAINT)
	        .setShortFlag('C') 
	        .setLongFlag(PARAM_MAX_CONSTRAINT);
	    jsap.registerParameter(opt);
	    
	    opt = new FlaggedOption(PARAM_START)
	    	.setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault("0")
	        .setShortFlag(JSAP.NO_SHORTFLAG) 
	        .setLongFlag(PARAM_START);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_MAX_PROCESSORS)
	    	.setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault("0")
	        .setShortFlag('p') 
	        .setLongFlag(PARAM_MAX_PROCESSORS);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(NSConstants.PARAM_DEBUG)
	    	.setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault("0")
	        .setShortFlag(JSAP.NO_SHORTFLAG) 
	        .setLongFlag(NSConstants.PARAM_DEBUG);
	    jsap.registerParameter(opt);

	    opt = new FlaggedOption(PARAM_SEQ_LEN_START)
	    	.setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault("0")
	        .setShortFlag(JSAP.NO_SHORTFLAG) 
	        .setLongFlag(PARAM_SEQ_LEN_START);
	    jsap.registerParameter(opt);
	
	    opt = new FlaggedOption(PARAM_SEQ_LEN_STOP)
	    	.setStringParser(JSAP.INTEGER_PARSER)
	        .setRequired(false)
	        .setDefault("0")
	        .setShortFlag(JSAP.NO_SHORTFLAG) 
	        .setLongFlag(PARAM_SEQ_LEN_STOP);
	    jsap.registerParameter(opt);

    	JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + DetermineSufficientConstraints2.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}

}
