package org.cn3.ns.nodeencoding;

import java.util.Arrays;
import java.util.List;

public enum EncodingMode {
	TOPO_ZERO(1, 0, "1", "zero", "TOPO_ZERO", "AT"),
	TOPO_ONE(2, 1, "2", "one", "TOPO_ONE", "ACT"),
	LENGTH_ANGLE(3, 1, "3", "LENGTH_ANGLE"),
	TOPO_GACT(4, 1, "4", "TOPO_GACT"),
	LOCAL_LENGTH_ANGLE(5, 1, "5", "LOCAL_LENGTH_ANGLE"),
	LTB_AOB_SEPARATE(6, 1, "6", "LTB_AOB_SEPARATE"),
	TOPO_TWO(7, 2, "7", "two", "TOPO_TWO"),
	TOPO1_REMOTE_ANGLE(8, 1, "8", "TOPO1_REMOTE_ANGLE");
	
	private final int id;
	private final int topologyLevel;
	private final List<String> values;

    EncodingMode(int id, int topologyLevel, String ...values) {
    	this.id = id;
        this.topologyLevel = topologyLevel;
    	this.values = Arrays.asList(values);
    }

    public int getId(){
    	return id;
    }
    public int getTopologyLevel(){
    	return topologyLevel;
    }
    public List<String> getValues() {
        return values;
    }
	public static EncodingMode find(String name) {
	    for (EncodingMode mmode : EncodingMode.values()) {
	        if (mmode.getValues().contains(name)) {
	            return mmode;
	        }
	    }
	    return null;
	}
	public static EncodingMode find(int id) {
		EncodingMode[] modes = EncodingMode.values();
		int index = id-1;
		if (index < 0 || index >= modes.length){
			return null;
		}
		return EncodingMode.values()[id-1];
	}
	public int getSizeFromLength(int length){
		int size = length;
		for (int i = 0; i < topologyLevel; i++) size = size * 2 + 1;
		return size;
	}
	public int getLengthFromSize(int size){
		int length = size;
		for (int i = 0; i < topologyLevel; i++) length = (length - 1) / 2;
		return length;
	}
}
