package org.cn3.ns.nodeencoding;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Stack;

import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.ns.domain.TraversalOrder;

import static org.cn3.ns.domain.TraversalOrder.LTS;

public abstract class NodeEncoder {

	private EncodingMode encodingMode;

	private final static List<Character> availableCharacters = Arrays.asList(
			'A','R','N','D','C','Q','E','G','H','I','L','K','M',
			'F','P','S','T','W','Y','V','B','J','Z','X');
	public final static int MAX_CHARS = availableCharacters.size();

	abstract public String encodeNode(BinaryTreeNode node) throws Exception;
	abstract public BinaryTreeNode decodeChar(char character, BinaryTreeNode node)
		throws EncodeException;
	
	public static List<Character> getAvailableCharacters(){
		// Make a copy of the
		return new ArrayList<Character>(availableCharacters);
	}
	protected static Character getAvailableCharacter(int index){
		return availableCharacters.get(index);
	}

	public int getSizeFromLength(int length){
		return encodingMode.getSizeFromLength(length);
	}
	public int getLengthFromSize(int size){
		return encodingMode.getLengthFromSize(size);
	}

	protected void setEncodingMode(EncodingMode encodingMode){
		this.encodingMode = encodingMode;
	}
	public BinaryTreeNode sequenceToTree(String sequence) throws EncodeException{
		BinaryTreeNode node, treeHead;
		node = treeHead = new BinaryTreeNode();
    	for (int i = 0; i < sequence.length(); i++){
    		node = decodeChar(sequence.charAt(i), node);
    	}
		return treeHead;
	}

	public String encodeTree(BinaryTreeNode root, String travOrder) throws Exception{
		return encodeTree(root, TraversalOrder.find(travOrder));
	}
	public String encodeTree(BinaryTreeNode root, TraversalOrder travOrder) throws Exception{
    	Stack<BinaryTreeNode> stack = new Stack<BinaryTreeNode>();
    	BinaryTreeNode current;
    	StringBuffer seq = new StringBuffer();
    	stack.push(root);
    	while (!stack.isEmpty()){
    		current = stack.pop();
    		seq.append(encodeNode(current));

    		addChildrenToStack(stack, current, travOrder);
    	}

    	return seq.toString();

	}

	private static void addChildrenToStack(
			Stack<BinaryTreeNode> stack, BinaryTreeNode node, String order) throws Exception {
		addChildrenToStack(stack,node,TraversalOrder.find(order));
	}
	private static void addChildrenToStack(
			Stack<BinaryTreeNode> stack, BinaryTreeNode node, TraversalOrder order) throws Exception{
		BinaryTreeNode right = node.getRight();
		BinaryTreeNode left = node.getLeft();
    	if (right != null){
    		if (left != null){
		    	if (right.getTreeSize() > left.getTreeSize()){
		    		addToStack(stack, right, left, order);
		    	}
		    	else if (left.getTreeSize() > right.getTreeSize()){
		    		addToStack(stack, left, right, order);
		    	}
		    	/* Larger shapeNum generally less asymmetric, less avgHeight, 
		    	 * so smaller shapeNum should go first */
		    	else if (left.getShapeNum().compareTo(right.getShapeNum()) > 0){
    	    		addToStack(stack, right, left, order);
    	    	}
    	    	else if (left.getShapeNum().compareTo(right.getShapeNum()) < 0){
    	    		addToStack(stack, left, right, order);
    	    	}
   	    		// no differences, default
	    		else{
	    			addToStack(stack, left, right, order);
    	    	}	
    		}
    		else{
    			stack.push(right);
    		}
    	}
    	else if (left != null){
    		stack.push(left);
    	}
	}

	private static void addToStack(
    		Stack<BinaryTreeNode> stack, 
    		BinaryTreeNode larger, 
    		BinaryTreeNode smaller,
    		TraversalOrder order){
    	if (order == LTS){
    		stack.push(smaller);
			stack.push(larger);
    	}
    	else{
    		stack.push(larger);
			stack.push(smaller);
    	}
    }
}
