package org.cn3.ns.nodeencoding;

import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.ns.resources.NSConstants;

public class NodeEncoderFactory {

	public static NodeEncoder createNewNodeEncoder(EncodingMode encoding)
	throws Exception{
		NodeEncoder nodeEncoder;
		switch(encoding){
		case TOPO_ZERO:
			nodeEncoder = new Topological0thOrderEncoder();
			break;
		case TOPO_ONE:
			nodeEncoder = new Topological1stOrderEncoder();
			break;
		case LENGTH_ANGLE:
			nodeEncoder = new TreeRelativeLengthAngleTopo1Encoder();
			break;
		case TOPO_GACT:
			nodeEncoder = new JoinedTreeTopo1Encoder();
			break;
		case LOCAL_LENGTH_ANGLE:
			nodeEncoder = new ParentRelativeLengthAngleTopo1Encoder();
			break;
		case LTB_AOB_SEPARATE:
			nodeEncoder = new ParentRelativeLengthAngleTopo1SeparateEncoder();
			break;
			case TOPO1_REMOTE_ANGLE:
				nodeEncoder = new Topology1AndRemoteAngleEncoder();
				break;
		case TOPO_TWO:
			nodeEncoder = new Topological2ndOrderEncoder();
			break;
		default:
			throw new Exception("Unknown mapping mode: "+encoding);
		}
		return nodeEncoder;
	}

	public static NodeEncoder createNewNodeEncoder(BinaryTreeNode tree, EncodingMode encoding)
	throws Exception{
		NodeEncoder nodeEncoder;
		switch(encoding){
		case TOPO_ZERO:
			nodeEncoder = new Topological0thOrderEncoder();
			break;
		case TOPO_ONE:
			nodeEncoder = new Topological1stOrderEncoder();
			break;
		case LENGTH_ANGLE:
			nodeEncoder = new TreeRelativeLengthAngleTopo1Encoder(tree);
			break;
		case TOPO_GACT:
			nodeEncoder = new JoinedTreeTopo1Encoder();
			break;
		case LOCAL_LENGTH_ANGLE:
			nodeEncoder = new ParentRelativeLengthAngleTopo1Encoder();
			break;
		case LTB_AOB_SEPARATE:
			nodeEncoder = new ParentRelativeLengthAngleTopo1SeparateEncoder();
			break;
		case TOPO_TWO:
			nodeEncoder = new Topological2ndOrderEncoder();
			break;
		default:
			throw new Exception("Unknown mapping mode: "+encoding);
		}
		return nodeEncoder;
	}
	
	@Deprecated 
	public static NodeEncoder createNewNodeEncoder(int mappingMode)
	throws Exception{
		NodeEncoder nodeEncoder;
		switch(mappingMode){
		case NSConstants.MAPPING_MODE_TOPO_ACT:
			nodeEncoder = new Topological1stOrderEncoder();
			break;
		case NSConstants.MAPPING_MODE_TOPO_LEN_ANG:
			nodeEncoder = new TreeRelativeLengthAngleTopo1Encoder();
			break;
		case NSConstants.MAPPING_MODE_TOPO_GACT:
			nodeEncoder = new JoinedTreeTopo1Encoder();
			break;
		case NSConstants.MAPPING_MODE_TOPO_LOCAL_LEN_ANG:
			nodeEncoder = new ParentRelativeLengthAngleTopo1Encoder();
			break;
		case NSConstants.MAPPING_MODE_TOPO_LTB_AOB_SEPARATE:
			nodeEncoder = new ParentRelativeLengthAngleTopo1SeparateEncoder();
			break;
		case NSConstants.MAPPING_MODE_TOPO_TWO:
			nodeEncoder = new Topological2ndOrderEncoder();
			break;
		default:
			throw new Exception("Unknown mapping mode: "+mappingMode);
		}
		return nodeEncoder;
	}

	@Deprecated
	public static NodeEncoder createNewNodeEncoder(BinaryTreeNode tree, int mappingMode)
	throws Exception{
		NodeEncoder nodeEncoder;
		switch(mappingMode){
		case NSConstants.MAPPING_MODE_TOPO_ACT:
			nodeEncoder = new Topological1stOrderEncoder();
			break;
		case NSConstants.MAPPING_MODE_TOPO_LEN_ANG:
			nodeEncoder = new TreeRelativeLengthAngleTopo1Encoder(tree);
			break;
		case NSConstants.MAPPING_MODE_TOPO_GACT:
			nodeEncoder = new JoinedTreeTopo1Encoder();
			break;
		case NSConstants.MAPPING_MODE_TOPO_LOCAL_LEN_ANG:
			nodeEncoder = new ParentRelativeLengthAngleTopo1Encoder();
			break;
		case NSConstants.MAPPING_MODE_TOPO_LTB_AOB_SEPARATE:
			nodeEncoder = new ParentRelativeLengthAngleTopo1SeparateEncoder();
			break;
		case NSConstants.MAPPING_MODE_TOPO_TWO:
			nodeEncoder = new Topological2ndOrderEncoder();
			break;
		default:
			throw new Exception("Unknown mapping mode: "+mappingMode);
		}
		return nodeEncoder;
	}
}
