package org.cn3.ns.nodeencoding;

import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.SwcDataNode;
import org.cn3.morpho.domain.SwcSecondaryData;
import org.cn3.morpho.utils.SwcDataUtils;

public class ParentRelativeLengthAngleTopo1SeparateEncoder extends NodeEncoder {

	public ParentRelativeLengthAngleTopo1SeparateEncoder(){
		setEncodingMode(EncodingMode.LTB_AOB_SEPARATE);
	}

	public String encodeNode(BinaryTreeNode node) throws Exception{
		if (node.isLeaf()){
			return "";
		}
		
		double branchAngle, leftLength, rightLength;
    	SwcDataNode parentData, leftData, rightData;
    	double parentLength, parentAngle;
    	String trimer;
    	boolean continuation = false;
    	
		parentData = (SwcDataNode)node.getData();
		leftData = (SwcDataNode)node.getLeft().getData();
		rightData = (SwcDataNode)node.getRight().getData();
		leftLength = SwcDataUtils.getDistance(parentData, leftData);
		rightLength = SwcDataUtils.getDistance(parentData, rightData);
		branchAngle = SwcDataUtils.getAngle(parentData, leftData, rightData);
		
		if (parentData.getSecondaryData() != null){
			parentLength = parentData.getSecondaryData().getEuclideanDistance();
			parentAngle = parentData.getSecondaryData().getRemoteAngleOfBifurcation();
		}
		else{
			parentLength = parentAngle = 0;
		}
		
		leftData.setSecondaryData(new SwcSecondaryData());
		leftData.getSecondaryData().setEuclideanDistance(leftLength);
		leftData.getSecondaryData().setRemoteAngleOfBifurcation(branchAngle);
		rightData.setSecondaryData(new SwcSecondaryData());
		rightData.getSecondaryData().setEuclideanDistance(rightLength);
		rightData.getSecondaryData().setRemoteAngleOfBifurcation(branchAngle);
		
		if (node.getLeft().hasChildren() && node.getRight().hasChildren()){
			trimer = "A";
		}
		else if ((node.getLeft().hasChildren() && !node.getRight().hasChildren())
				||(node.getRight().hasChildren() && !node.getLeft().hasChildren())){
			trimer = "C";
			continuation = true;
		}
		else{
			trimer = "T";
		}
		
		if (leftLength > parentLength && rightLength > parentLength){
			trimer += "Q";
		}
		else if (leftLength < parentLength && rightLength < parentLength){
			trimer += "E";
		}
		else{
			if (continuation){
				if (node.getLeft().hasChildren() && leftLength > parentLength
						|| node.getRight().hasChildren() && rightLength > parentLength){
					trimer += "H";
				}
				else{
					trimer += "I";
				}
			}
			else{
				trimer += "G";
			}
		}
		
		if (branchAngle < parentAngle){
			trimer += "L";
		}
		else{
			trimer += "K";
		}
		return trimer;
	}

	
	public BinaryTreeNode decodeChar(char character, BinaryTreeNode node){
		node.setRight(new BinaryTreeNode());
		node.setLeft(new BinaryTreeNode());

		switch(character){
		case 'A':
		case 'C':
			node.setData(""+character);
			node = node.getLeft();
			break;
		case 'T':
			node.setData("T");
			boolean done = false;
			while (node.getParent() != null && !done){
				if (node.getParent().getData().equals("C") || node.isRight()){
					node = node.getParent();
				}
				else if (node.isLeft()){
					node = node.getParent().getRight();
					done = true;
				}
			}
			break;
		default:
		}
		return node;
	}
	
}
