package org.cn3.ns.nodeencoding;

import java.util.Arrays;
import java.util.Set;
import java.util.TreeSet;

import org.cn3.morpho.domain.BifurcationNode;
import org.cn3.morpho.domain.BinaryTreeNode;

public class Topological2ndOrderEncoder extends NodeEncoder {

	public static final int ENCODE_SIMPLE = 0;
	public static final int ENCODE_COMPLEX = 1;
	public static final int ENCODE_FULL = 3;

	public Topological2ndOrderEncoder(){
		setEncodingMode(EncodingMode.TOPO_TWO);
	}

	//private Set<Character> AType = new TreeSet<Character>(Arrays.asList('W','M','N','H','V','J'));
	private Set<Character> AXXType = new TreeSet<Character>(Arrays.asList('W','M','N','H'));
	//private Set<Character> ATXType = new TreeSet<Character>(Arrays.asList('V','J'));
	//private Set<Character> CXType = new TreeSet<Character>(Arrays.asList('Q','O'));

	private static NodeEncoder firstOrderEncoder = new Topological1stOrderEncoder();
	
	private int complexity = ENCODE_FULL;
	
	public String getComplexityString() {
		switch (complexity){
		case ENCODE_SIMPLE:
			return "Simple";
		case ENCODE_COMPLEX:
			return "Complex";
		case ENCODE_FULL:
		default:
			return "Full";
		}
	}

	public int getComplexity() {
		return complexity;
	}

	public void setComplexity(int complexity) {
		this.complexity = complexity;
	}

	public String encodeNode(BinaryTreeNode node) throws Exception{
		switch(complexity){
		case ENCODE_SIMPLE:
			return encodeNodeSimple(node);
		case ENCODE_COMPLEX:
			return encodeNodeComplex(node);
		case ENCODE_FULL:
		default:
			return encodeNodeFull(node);
		}
	}
	
	public String encodeNodeSimple(BinaryTreeNode node) throws Exception{
		if (node.getLeft() != null && node.getRight() != null){
			String topoLarger, topoSmaller;
			if (node.getLeft().hasChildren() && node.getRight().hasChildren()){
				// A-type
				topoLarger = firstOrderEncoder.encodeNode(node.getLargerSubtree());
				topoSmaller = firstOrderEncoder.encodeNode(node.getSmallerSubtree());
				if (topoLarger.equals("A") || topoLarger.equals("C")){
					if (topoSmaller.equals("T")){
						return "P"; // AA...T or AC...T
					}
					else {
						return "A"; // AA...C, AC...A, AA...A, AC...C
					}
				}
				else{ // topo21 == T
					return "F"; // ATT
				}
			}
			else if ((node.getLeft().hasChildren() && !node.getRight().hasChildren())
					||(node.getRight().hasChildren() && !node.getLeft().hasChildren())){
				// C-type
				topoLarger = firstOrderEncoder.encodeNode(node.getLargerSubtree());
				if (topoLarger.equals("A") || topoLarger.equals("C")){
					return "L"; // CA... or CC...
				}
				else{
					return "I"; // CT
				}
			}
			else{
				return ""; // T node, implicit and therefore adds no info
			}
		}
		return "";
	}

	public String encodeNodeComplex(BinaryTreeNode node) throws Exception{
		if (node.getLeft() != null && node.getRight() != null){
			String topoLarger, topoSmaller;
			if (node.getLeft().hasChildren() && node.getRight().hasChildren()){
				// A-type
				topoLarger = firstOrderEncoder.encodeNode(node.getLargerSubtree());
				topoSmaller = firstOrderEncoder.encodeNode(node.getSmallerSubtree());
				if (topoLarger.equals("A")){
					if (topoSmaller.equals("A")){
						return "W"; // AA...A
					}
					else if (topoSmaller.equals("C")){
						return "M"; // AA...C
					}
					else if (topoSmaller.equals("T")){
						return "V"; // AA...T
					} 
				}
				else if (topoLarger.equals("C")){
					// Should larger/smaller side types be distinguished? AATTC... vs ACTA...
					if (topoSmaller.equals("A")){
						return "M"; // AC...A
					}
					else if (topoSmaller.equals("C")){
						return "H"; // AC...C
					}
					else{
						return "J"; // AC...T
					}
				}
				else{ // ATT
					return "F";
				}
			}
			else if ((node.getLeft().hasChildren() && !node.getRight().hasChildren())
					||(node.getRight().hasChildren() && !node.getLeft().hasChildren())){
				// C-type
				topoLarger = firstOrderEncoder.encodeNode(node.getLargerSubtree());
				if (topoLarger.equals("A")){
					return "Q"; // CA...
				}
				else if (topoLarger.equals("C")){
					return "O"; // CC...
				}
				else{
					return "L"; // CT
				}
			}
			else{
				return ""; // T node, implicit and therefore adds no info
			}
		}
		return "";
	}

	public String encodeNodeFull(BinaryTreeNode node) throws Exception{
		if (node.getLeft() != null && node.getRight() != null){
			String topoLarger, topoSmaller;
			if (node.getLeft().hasChildren() && node.getRight().hasChildren()){
				// A-type
				topoLarger = firstOrderEncoder.encodeNode(node.getLargerSubtree());
				topoSmaller = firstOrderEncoder.encodeNode(node.getSmallerSubtree());
				if (topoLarger.equals("A")){
					if (topoSmaller.equals("A")){
						return "W"; // AA...A
					}
					else if (topoSmaller.equals("C")){
						return "M"; // AA...C
					}
					else if (topoSmaller.equals("T")){
						return "V"; // AA...T
					} 
				}
				else if (topoLarger.equals("C")){
					// Should larger/smaller side types be distinguished? AATTC... vs ACTA...
					if (topoSmaller.equals("A")){
						return "N"; // AC...A
					}
					else if (topoSmaller.equals("C")){
						return "H"; // AC...C
					}
					else{
						return "J"; // AC...T
					}
				}
				else{ // ATT
					return "F";
				}
			}
			else if ((node.getLeft().hasChildren() && !node.getRight().hasChildren())
					||(node.getRight().hasChildren() && !node.getLeft().hasChildren())){
				// C-type
				topoLarger = firstOrderEncoder.encodeNode(node.getLargerSubtree());
				if (topoLarger.equals("A")){
					return "Q"; // CA...
				}
				else if (topoLarger.equals("C")){
					return "O"; // CC...
				}
				else{
					return "L"; // CT
				}
			}
			else{
				return ""; // T node, implicit and therefore adds no info
			}
		}
		return "";
	}

	public BinaryTreeNode decodeChar(char character, BinaryTreeNode node){
		node.setRight(new BinaryTreeNode());
		node.setLeft(new BinaryTreeNode());
		node.setData(""+character);

		switch(character){
		case 'W':
		case 'M':
		case 'N':
		case 'H':
			// AX...X
			node = node.getLeft();
			break;
		case 'V':
		case 'J':
			// ATX
			node.setRight(new BifurcationNode());
			node = node.getLeft();
			break;
		case 'F':
			// ATT
			node.setRight(new BifurcationNode());
			node.setLeft(new BifurcationNode());
			node = climbToNext(node);
			break;
		case 'Q':
		case 'O':
			// CX
			node = node.getLeft();
			break;
		case 'L':
			// CT
			node.setLeft(new BifurcationNode());
			node = climbToNext(node);
			break;
		}
		return node;
	}
	
	private BinaryTreeNode climbToNext(BinaryTreeNode node){
		boolean done = false;
		while (node.getParent() != null && !done){
			if (!AXXType.contains(node.getParent().getData()) || node.isRight()){
				node = node.getParent();
			}
			else if (node.isLeft()){
				node = node.getParent().getRight();
				done = true;
			}
		}
		return node;
	}
}
