package org.cn3.ns.nodeencoding;

import java.util.Arrays;
import java.util.Set;
import java.util.TreeSet;

import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.SwcDataNode;
import org.cn3.morpho.utils.SwcDataUtils;

public class TreeRelativeLengthAngleTopo1Encoder extends NodeEncoder {

	private double medianLength;
	private double medianAngle;

	public TreeRelativeLengthAngleTopo1Encoder(){
		// Without a tree, median length and angle can't be calculated
		medianLength = 0;
		medianAngle = 0;
		setEncodingMode(EncodingMode.LENGTH_ANGLE);
	}
	
	public TreeRelativeLengthAngleTopo1Encoder(BinaryTreeNode tree){
		// #TODO: Calculate median angle and median length
		
	}
	
	public String encodeNode(BinaryTreeNode node) throws Exception{
		if (node.isLeaf()){
			return "";
		}
		
		double branchAngle, leftLength, rightLength;
    	SwcDataNode parentData, leftData, rightData;
    	
    	int code = 0;

		parentData = (SwcDataNode)node.getData();
		leftData = (SwcDataNode)node.getLeft().getData();
		rightData = (SwcDataNode)node.getRight().getData();
		leftLength = SwcDataUtils.getDistance(parentData, leftData);
		rightLength = SwcDataUtils.getDistance(parentData, rightData);
		branchAngle = SwcDataUtils.getAngle(parentData, leftData, rightData);

		if (node.getLeft().hasChildren() && node.getRight().hasChildren()){
			code = 100;
		}
		else if ((node.getLeft().hasChildren() && !node.getRight().hasChildren())
				||(node.getRight().hasChildren() && !node.getLeft().hasChildren())){
			code = 200;
		}
		else{
			code = 300;
		}
		
		if (leftLength > medianLength && rightLength > medianLength){
			code += 20;
		}
		else if (leftLength < medianLength && rightLength < medianLength){
			code += 10;
		}
		else{
			if (code == 200){
				if (node.getLeft().hasChildren() && leftLength > medianLength
						|| node.getRight().hasChildren() && rightLength > medianLength){
					code += 40;
				}
				else{
					code += 50;
				}
			}
			else{
				code += 30;
			}
		}
		
		if (branchAngle < medianAngle){
			code += 1;
		}
		else{
			code += 2;
		}
		
		return decode(code);
	}
	
	private Set<Character> AType = new TreeSet<Character>(Arrays.asList('A','R','N','D','C'));

	/** 
	 * 100: Activation		(A)
	 * 200: Continuation	(C)
	 * 300: Termination		(D)
	 * 10: both lengths smaller 	(L-/L-)
	 * 20: both lengths larger		(L+/L+)
	 * 30: one larger, one smaller	(L+/L-)
	 * 40: bifurcating branch larger, terminating branch smaller (b+/t-)
	 * 50: bifurcating branch smaller, terminating branch larger (b-/t+)
	 * 1: angle smaller (O-)
	 * 2: angle larger	(O+)
	 */ 
	private static String decode(int code){
		switch (code){
		case 111: // A|L-/L-|O-
			return "A";
		case 112: // A|L-/L-|O+
			return "R";
		case 121: // A|L+/L+|O-
			return "N";
		case 122: // A|L+/L+|O+
			return "D";
		case 131: // A|L+/L-|O-
			return "C";
		case 132: // A|L+/L-|O+
			return "Q";
		case 211: // C|b-/t-|O-
			return "E";
		case 212: // C|b-/t-|O+
			return "G";
		case 221: // C|b+/t+|O-
			return "H";
		case 222: // C|b+/t+|O+
			return "I";
		case 241: // C|b+/t-|O-
			return "L";
		case 242: // C|b+/t-|O+
			return "K";
		case 251: // C|b-/t+|O-
			return "M";
		case 252: // C|b-/t+|O+
			return "F";
		case 311: // T|L-/L-|O-
			return "P";
		case 312: // T|L-/L-|O+
			return "S";
		case 321: // T|L+/L+|O-
			return "T";
		case 322: // T|L+/L+|O+
			return "W";
		case 331: // T|L+/L-|O-
			return "Y";
		case 332: // T|L+/L-|O+
			return "V";
		default:
			return "";
		}
	}
	
	public BinaryTreeNode decodeChar(char character, BinaryTreeNode node){
		node.setRight(new BinaryTreeNode());
		node.setLeft(new BinaryTreeNode());
		node.setData(""+character);

		switch(character){
		case 'A': // A
		case 'R':
		case 'N':
		case 'D':
		case 'C':
		case 'Q':
		case 'E': // C
		case 'G':
		case 'H':
		case 'I':
		case 'L':
		case 'K':
		case 'M':
		case 'F':
			node = node.getLeft();
			break;
		case 'T': // T
		case 'P':
		case 'S':
		case 'W':
		case 'Y':
		case 'V':
			boolean done = false;
			while (node.getParent() != null && !done){
				if (!AType.contains(node.getParent().getData()) || node.isRight()){
					node = node.getParent();
				}
				else if (node.isLeft()){
					node = node.getParent().getRight();
					done = true;
				}
			}
			break;
		}
		return node;
	}
}
