package org.cn3.ns.resources;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Map;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cn3.morpho.domain.NeuronMetadata;

public class ProjectProperties {

	public static final String DATASOURCE_DB = "db";
	public static final String DATASOURCE_FILE = "file";
	
	private static ProjectProperties instance = null;
	private static Properties projectProperties = null;
	private static Properties dataProperties = null;

	public static ProjectProperties getInstance() throws IOException{
		if (instance == null){
			instance = new ProjectProperties();
		}
		return instance;
	}
	
	private ProjectProperties() throws IOException{
		projectProperties = new Properties();
		projectProperties.load(new FileInputStream("resources/conf/project.properties"));
	}
	
	public Properties getProjectProperties() throws Exception{
		return projectProperties;
	}
	
	public String getWorkingDir() throws IOException{
		return projectProperties.getProperty(NSConstants.WORKING_LOC);
	}
	public String getResourceDirectory() throws IOException{
		return getWorkingDir()+projectProperties.getProperty(NSConstants.RESOURCES_LOC);
	}
	public String getDataSource() throws IOException{
		return projectProperties.getProperty(NSConstants.DATASOURCE);
	}
	public NeuronMetadata getNeuronFromSequence(Map<String,NeuronMetadata> cellTypeMap, String sequence){
		Pattern p = Pattern.compile("(.*)\\-(Dendrite|BasalDendrite|ApicalDendrite|Axon)");
		Matcher m = p.matcher(sequence);
		if (!m.matches()){
			System.out.println("no match");
			return null;
		}
		return cellTypeMap.get(m.group(1));
	}
	public String getFastaDir() throws Exception{
		return projectProperties.getProperty(NSConstants.FASTA_LOCATION);
	}
	public String getFastaArborDir() throws Exception{
		return getFastaDir()+"Arborization/";
	}
	public String getFastaRandomsDir() throws Exception{
		return projectProperties.getProperty(NSConstants.FASTA_RANDOMS_LOCATION);
	}
	public String getResultsDir() throws Exception{
		return projectProperties.getProperty(NSConstants.RESULTS_LOCATION);
	}
	public String getResultsLookupDir() throws Exception{
		return projectProperties.getProperty(NSConstants.RESULTS_LOOKUP_LOCATION);
	}
}
