package org.cn3.ns.scripts;

import java.io.File;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.cn3.morpho.data.MorphoDataReader;
import org.cn3.morpho.data.SWCReader;
import org.cn3.morpho.data.persistence.NeuronDAO;
import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.DistributionBean;
import org.cn3.morpho.domain.NeuronMetadata;
import org.cn3.morpho.domain.Soma;
import org.cn3.morpho.utils.BinaryTreeUtils;
import org.cn3.morpho.utils.StringUtils;
import org.cn3.ns.resources.DataProperties;

/**
 * 
 * @author gillette
 *
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.AxonAsymmetryTest
 *
 */
public class AxonAsymmetryTest {

	public static void main(String args[]){
		try{
			/*
			NeuronDAO dao = NeuronDAO.getInstance(); 
			
			// Collect neurites from the 2 classes
			Map<String,List<String>> metaDataConstraints = new HashMap<String,List<String>>();
			String[] brainRegions = {"Neocortex"};
			metaDataConstraints.put(NeuronDAO.METATYPE_BRAIN_REGION, Arrays.asList(brainRegions));
			String[] cellTypes = {"Pyramidal cell"};
			metaDataConstraints.put(NeuronDAO.METATYPE_CLASS2, Arrays.asList(cellTypes));
			String[] species = {"Mouse"};
			metaDataConstraints.put(NeuronDAO.METATYPE_SPECIES, Arrays.asList(species));
			List<NeuronMetadata> class1Neurons = dao.getNeuronsByMetadata(metaDataConstraints);

			metaDataConstraints.clear();
			String[] brainRegions2 = {"Neocortex"};
			metaDataConstraints.put(NeuronDAO.METATYPE_BRAIN_REGION, Arrays.asList(brainRegions2));
			String[] cellTypes2 = {"Basket cell","Parvalbumin (PV) containing cell","Chandelier cell","Non-fast spiking"};
			metaDataConstraints.put(NeuronDAO.METATYPE_CLASS2, Arrays.asList(cellTypes2));
			String[] species2 = {"Mouse","Rat"};
			metaDataConstraints.put(NeuronDAO.METATYPE_SPECIES, Arrays.asList(species2));
			List<NeuronMetadata> class2Neurons = dao.getNeuronsByMetadata(metaDataConstraints);

			// Combined neurons from classes
			List<NeuronMetadata> combinedList = new ArrayList<NeuronMetadata>(class1Neurons);
			combinedList.addAll(class2Neurons);
			
			*/
			
			// Get list from SeqList files
			String filename = "/home/tgillett/Domains/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust1-Zoom-DistLim1.txt";
			Set<String> clust1List = MorphoDataReader.loadSet(filename);
			filename = "/home/tgillett/Domains/SeqList-Axon-G3E1Identity-Dims2of7-VII4-Clust3-Zoom-DistLim1.txt";
			Set<String> clust2List = MorphoDataReader.loadSet(filename);
			List<String> combinedList = new ArrayList<String>(clust1List);
			combinedList.addAll(clust2List);
			
			// Instantiate distribution beans
			DistributionBean class1Asymmetry = new DistributionBean();
			DistributionBean class2Asymmetry = new DistributionBean();
			DistributionBean class1CPercent = new DistributionBean();
			DistributionBean class2CPercent = new DistributionBean();
			DistributionBean asymmetryValues = new DistributionBean();
			int cNodes;
			
			Soma<BinaryTreeNode> soma;
			BinaryTreeNode axon;
			DataProperties dataProps = DataProperties.getInstance();
			File swcFile;
			List<BinaryTreeNode> bifurcationList;
			
			// Load each axon
			//for (NeuronMetadata neuronMetadata : combinedList){
			String neuronName;
			for (String neuriteName : combinedList){
				// Determine swc location and read tree
				//swcFile = dataProps.getSwcFileByName(neuronMetadata.getName());
				neuronName = StringUtils.getNeuronNameFromSequence(neuriteName);
				//System.out.println(neuronName);
				swcFile = dataProps.getSwcFileByName(neuronName);
				//System.out.println(swcFile);
				soma = SWCReader.convertSwcToBinarySoma(swcFile);
				axon = soma.getAxonalTree().getTreeHead();
				
				bifurcationList = BinaryTreeUtils.createBifurcationList(axon);
				asymmetryValues.clearData();
				cNodes = 0;
				
				for (BinaryTreeNode node : bifurcationList){
					// Calculate average asymmetry of bifurcations with
					// subtree size at least 1/8 of total size
					if (node.getTreeSize() >= axon.getTreeSize()/4){
						asymmetryValues.addData(node.getPartitionAsymmetry());
						if (node.getPartitionAsymmetry() > 1){
							System.out.println(swcFile+" "+node.getSwcData().getNodeId()+" "+node.getPartitionAsymmetry());
						}
					}
					
					// Count %C nodes
					if (node.getPartitionAsymmetry() == 1){
						cNodes++;
					}
				}
				
				if (clust1List.contains(neuriteName)){
					class1Asymmetry.addData(asymmetryValues.getMean());
					class1CPercent.addData((double)cNodes/(axon.getDegree()-1));
				}else{
					class2Asymmetry.addData(asymmetryValues.getMean());
					class2CPercent.addData((double)cNodes/(axon.getDegree()-1));
				}
				
				if (asymmetryValues.getMean() > 1){
					System.out.println(swcFile+" "+asymmetryValues.getMean());
				}

				/*
				if (class1Neurons.contains(neuronMetadata)){
					class1Asymmetry.addData(asymmetryValues.getMean());
					class1CPercent.addData((double)cNodes/(axon.getDegree()-1));
				}else{
					class2Asymmetry.addData(asymmetryValues.getMean());
					class2CPercent.addData((double)cNodes/(axon.getDegree()-1));
				}
				*/
			}
			NumberFormat nf = NumberFormat.getInstance();
			nf.setMaximumFractionDigits(4);
			System.out.println("Cluster1 N: "+class1Asymmetry.getCount());
			System.out.println("Cluster3 N: "+class2Asymmetry.getCount());
			
			System.out.println("Cluster1 Asymmetry: "+nf.format(class1Asymmetry.getMean())+" +/- "+nf.format(class1Asymmetry.getStandardError()));
			System.out.println("Cluster3 Asymmetry: "+nf.format(class2Asymmetry.getMean())+" +/- "+nf.format(class2Asymmetry.getStandardError()));
			nf.setMaximumFractionDigits(1);
			System.out.println("Cluster1 %C: "+nf.format(100*class1CPercent.getMean())+" +/- "+nf.format(100*class1CPercent.getStandardError())+"%");
			System.out.println("Cluster3 %C: "+nf.format(100*class2CPercent.getMean())+" +/- "+nf.format(100*class2CPercent.getStandardError())+"%");
			
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
}
