package org.cn3.ns.scripts;

import java.io.File;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.MappedBinaryTreeUtils;
import org.cn3.ns.utils.MappingUtils;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 * Reads fasta file and converts each tree-sequence into the specified traversed sequence
 *
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.ConvertTraversalInFasta -i /homes/tgillett/fastaFromSwc/NM/Arborization/Mapping2/Size/NMArborSizeLTSMap2_Min20_Scrambled-randomsB3_1.fasta -o STL
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.ConvertTraversalInFasta -i /homes/tgillett/fastaFromSwc/NM/Arborization/Mapping2/Size/NMArborSizeLTSMap2_Min20_Scrambled-randomsB5_1.fasta -o STL
 *
 */
public class ConvertTraversalInFasta {

	public static final String PARAM_INPUT = "input";

	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);
			
			String inputFilename = config.getString(PARAM_INPUT);
			String order = config.getString(NSConstants.PARAM_ORDER);
			String outputFilename;
			
			File inputFile = new File(inputFilename);
			outputFilename = inputFile.getParent()+"/"+inputFile.getName().replaceAll("(STL|stl|LTS|lts)", order);
			
			FastaReader fr = new FastaReader(inputFile);
			StringBuffer sb = new StringBuffer();
			while (fr.next()){
				sb.append(fr.getHeader()+"\n");
				sb.append(MappedBinaryTreeUtils.reorderSequence(fr.getSequence(), order)+"\n");
			}

			DataWriter.writeStringToFile(outputFilename, sb.toString());
			System.out.println("Wrote file "+outputFilename);
		
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;
        
        opt = new FlaggedOption(PARAM_INPUT)
        	.setStringParser(JSAP.STRING_PARSER)
        	.setRequired(true) 
        	.setShortFlag('i') 
        	.setLongFlag(PARAM_INPUT);
        jsap.registerParameter(opt);

        opt = new FlaggedOption(NSConstants.PARAM_ORDER)
    		.setStringParser(JSAP.STRING_PARSER)
    		.setRequired(false)
    		.setDefault(NSConstants.DEFAULT_ORDER)
    		.setShortFlag('o') 
    		.setLongFlag(NSConstants.PARAM_ORDER);
        jsap.registerParameter(opt);

        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (@SuppressWarnings("rawtypes")
			java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + ConvertTraversalInFasta.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;

	}

}
