package org.cn3.ns.scripts;

import org.cn3.ns.data.FastaReader;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 * Determines the total number of character alignments possible in a database, 
 * excepting any from a sequence aligned against itself.
 * java org.krasnow.cng.working.DatabaseLengthCalculator -I C:/fastaFromSwc/NM/Arborization/Mapping2/Size/NMArborSizeLTSMap2.fasta
 * java org.krasnow.cng.working.DatabaseLengthCalculator -I C:/fastaFromSwc/NM/Arborization/Mapping2/Size/NMArborSizeLTSMap2.MergeShort40.fasta
 * java org.krasnow.cng.working.DatabaseLengthCalculator -I C:/fastaFromSwc/NM/Arborization/Mapping2/Size/NMArborSizeLTSMap2.MergeShort80.fasta
 * java org.krasnow.cng.working.DatabaseLengthCalculator -I C:/fastaFromSwc/NM/Arborization/Mapping2/Size/NMArborSizeLTSMap2.MergeShort40ACR.fasta
 * java org.krasnow.cng.working.DatabaseLengthCalculator -I C:/fastaFromSwc/NM/Arborization/Mapping2/Size/NMArborSizeLTSMap2.MergeShort80ACR.fasta
 */
public class DatabaseLengthCalculator {

	private static final String PARAM_INPUT_FILENAME = "input";
	
	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);
			String inputFilename = config.getString(PARAM_INPUT_FILENAME);
			
			FastaReader fr = new FastaReader(inputFilename);
			long totalLength = 0;
			long diagonal = 0;
			long numSequences = 0;
			while (fr.next()){
				totalLength+=fr.getSequenceLength();
				diagonal += fr.getSequenceLength()*fr.getSequenceLength();
				numSequences++;
			}
			long lowerDiagonal = (totalLength*totalLength - diagonal)/2;
			System.out.println("Total Length: "+totalLength);
			System.out.println("Num sequences: "+numSequences);
			System.out.println("Lower Diagonal: "+lowerDiagonal);

		} catch (Exception e){
			e.printStackTrace();
		}
	}

	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;

        opt = new FlaggedOption(PARAM_INPUT_FILENAME)
        .setStringParser(JSAP.STRING_PARSER)
        .setRequired(true)
        .setShortFlag('I') 
        .setLongFlag(PARAM_INPUT_FILENAME);
        jsap.registerParameter(opt);

        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + DatabaseLengthCalculator.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}
	
}
