package org.cn3.ns.scripts;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.cn3.morpho.data.DataWriter;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.resources.NSConstants;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 *
 * Places all sequences from file 2 that are not in file 1 into their own
 * fasta file (output-file).
 *
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.ExtractNewSequences --file1 /home/tgillett/fastaFromSwc/NM_SpurLen2D/Arborization/Mapping2/Size/NM_SpurLen2DArborSizeSTLMap2.NMO5.6.fasta --file2 /home/tgillett/fastaFromSwc/NM_SpurLen2D/Arborization/Mapping2/Size/NM_SpurLen2DArborSizeSTLMap2.fasta -o /home/tgillett/fastaFromSwc/NM_SpurLen2D/Arborization/Mapping2/Size/NM_SpurLen2DArborSizeSTLMap2.newNMO6.fasta 
 *
 */

public class ExtractNewSequences {

	private static final String PARAM_FILE1 = "file1";
	private static final String PARAM_FILE2 = "file2";
	private static final String PARAM_OUTPUT_FILE = "output-file";
	
	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);
			
			File file1 = new File(config.getString(PARAM_FILE1));
			File file2 = new File(config.getString(PARAM_FILE2)); // the newer/longer file
			String outputFile = config.getString(PARAM_OUTPUT_FILE);
			
			List<String> existingSequences = FastaReader.getQueryNames(file1);
			FastaReader fr = new FastaReader(file2);
			StringBuffer newSeqsSb = new StringBuffer();
			List<String> newSeqs = new ArrayList<String>();
			while (fr.next()){
				if (!existingSequences.contains(fr.getSequenceName())){
					newSeqs.add(fr.getSequenceName());
					newSeqsSb.append("> "+fr.getSequenceName()+"\n");
					newSeqsSb.append(fr.getSequence()+"\n");
				}
			}
			
			DataWriter.writeStringToFile(outputFile, newSeqsSb.toString());
			System.out.println(newSeqs.size()+" new sequences");
			System.out.println("Wrote file "+outputFile);
		}catch (Exception e){
			e.printStackTrace();
		}
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;

        opt = new FlaggedOption(PARAM_FILE1)
			.setStringParser(JSAP.STRING_PARSER)
			.setRequired(true)
			.setShortFlag(JSAP.NO_SHORTFLAG) 
			.setLongFlag(PARAM_FILE1);
	    jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_FILE2)
		.setStringParser(JSAP.STRING_PARSER)
		.setRequired(true)
		.setShortFlag(JSAP.NO_SHORTFLAG) 
		.setLongFlag(PARAM_FILE2);
        jsap.registerParameter(opt);

        opt = new FlaggedOption(PARAM_OUTPUT_FILE)
		.setStringParser(JSAP.STRING_PARSER)
		.setRequired(true)
		.setShortFlag('o') 
		.setLongFlag(PARAM_OUTPUT_FILE);
        jsap.registerParameter(opt);

    	opt = new FlaggedOption(NSConstants.PARAM_MAPPING_MODE)
			.setStringParser(JSAP.INTEGER_PARSER)
			.setRequired(false)
			.setDefault(""+NSConstants.DEFAULT_MAPPING_MODE)
			.setShortFlag('M') 
			.setLongFlag(NSConstants.PARAM_MAPPING_MODE);
        jsap.registerParameter(opt);
        	    
        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (@SuppressWarnings("rawtypes")
			java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + ExtractNewSequences.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}
	
}
