package org.cn3.ns.scripts;

import java.io.File;
import java.util.Set;

import org.cn3.morpho.data.DataWriter;
import org.cn3.ns.data.FastaReader;
import org.cn3.ns.data.NSDataReader;
import org.cn3.ns.resources.NSConstants;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 *
 * java org.cn3.ns.scripts.FastaHeaderCleaner -f "C:/fastaFromSwc/NM/Arborization/Mapping2/Size/NMArborSizeLTSMap2_Min20.fasta"
 *
 */

public class FastaHeaderCleaner {

	public static void cleanFastaHeaders(String fastaFilename)
	throws Exception{
		
		FastaReader fr = new FastaReader(fastaFilename);
		StringBuffer cleanFasta = new StringBuffer();
		while (fr.next()){
			cleanFasta.append(">"+fr.getSequenceName()+"\n"+fr.getSequence()+"\n");
		}
		fr.close();
		
		File fastaFile = new File(fastaFilename);
		fastaFile.renameTo(new File(fastaFilename.replaceFirst("\\.fasta", ".fullHeader.fasta")));
		
		DataWriter.writeStringToFile(fastaFilename, cleanFasta);
	}
	
	private static final String PARAM_FASTA_FILENAME = "fastaFile";
	
	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);
			String fastaFilename = config.getString(PARAM_FASTA_FILENAME);
			
			cleanFastaHeaders(fastaFilename);

		} catch (Exception e){
			e.printStackTrace();
		}
	}

	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;

        opt = new FlaggedOption(PARAM_FASTA_FILENAME)
        .setStringParser(JSAP.STRING_PARSER)
        .setRequired(false)
        .setDefault("")
        .setShortFlag('f') 
        .setLongFlag(PARAM_FASTA_FILENAME);
        jsap.registerParameter(opt);

        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + FastaHeaderCleaner.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}

}
