package org.cn3.ns.scripts;

import org.cn3.morpho.data.DataWriter;
import org.cn3.ns.data.FastaReader;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 * Rewrites fasta file excluding sequences lower than a minimum size
 * Krasnow Cluster example:
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.FastaSequenceExcluder -m 20 -i /homes/tgillett/fastaFromSwc/NM/Arborization/Mapping2/Size/NMArborSizeLTSMap2.fasta -o /homes/tgillett/fastaFromSwc/NM/Arborization/Mapping2/Size/NMArborSizeLTSMap2_Min20.fasta
 *
 */
public class FastaSequenceExcluder {

	public static void main(String[] args){
		try{
			JSAPResult config = readParameters(args);
			String inputFilename = config.getString("input");
			String outputFilename = config.getString("output");
			int minLength = config.getInt("min-length");
			
			StringBuffer sb = new StringBuffer();
			FastaReader fr = new FastaReader(inputFilename);
			while (fr.next()){
				if (fr.getSequenceLength() >= minLength){
					sb.append(fr.getHeader()+"\n"+fr.getSequence()+"\n");
				}
			}
			
			DataWriter.writeStringToFile(outputFilename, sb.toString());
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
        
        FlaggedOption opt;

        opt = new FlaggedOption("input")
        .setStringParser(JSAP.STRING_PARSER)
        .setRequired(true)
        .setShortFlag('i') 
        .setLongFlag("input");
        jsap.registerParameter(opt);

        opt = new FlaggedOption("output")
        .setStringParser(JSAP.STRING_PARSER)
        .setRequired(true)
        .setShortFlag('o') 
        .setLongFlag("output");
        jsap.registerParameter(opt);

        opt = new FlaggedOption("min-length")
        .setStringParser(JSAP.INTEGER_PARSER)
        .setRequired(true)
        .setShortFlag('m') 
        .setLongFlag("min-length");
        jsap.registerParameter(opt);

        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + FastaSequenceExcluder.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}
}
