package org.cn3.ns.scripts;

import java.io.File;

import org.cn3.morpho.domain.BinaryTreeNode;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.data.SwcToNewick;
import org.cn3.ns.data.DataConverter;

import com.martiansoftware.jsap.FlaggedOption;
import com.martiansoftware.jsap.JSAP;
import com.martiansoftware.jsap.JSAPResult;

/**
 * 
 * @author gillette
 * 
 * java org.cn3.ns.scripts.NeuriteSequenceToNewick --sequence "AACTACCCTCCCTAATCATACTCCCCCATCATCTAATACTATCTCCACCTCCCATT" --output "C:\Documents and Settings\gillette\My Documents\research\MotifAnalysis\Figures\RepresentativeNeurites\Axons\oi53rpy1-Axon.newick" --label NODE_TYPE
 * java org.cn3.ns.scripts.NeuriteSequenceToNewick --sequence "ATCATCATCATCCATCCCATCATCCCT" --output "C:\Documents and Settings\gillette\My Documents\research\papers & presentations\PASTA Paper\Paper2 - AlignmentClusteringDomains\Figures\Workspace\Consensus-ApicalNC-STL-G3E1Clust1-Sub100T0.5.newick" --label NONE
 *
 */
public class NeuriteSequenceToNewick extends SwcToNewick{

	private static final String PARAM_INPUT_SEQUENCE = "sequence";
	private static final String PARAM_OUTPUT_FILE = "output";
	private static final String PARAM_LABEL_TYPE = "label";
	
	public static void main(String args[]){
		try{
			JSAPResult config = readParameters(args);

			String sequence = config.getString(PARAM_INPUT_SEQUENCE);
			String output = config.getString(PARAM_OUTPUT_FILE);
			NewickLabel labelType = NewickLabel.valueOf(config.getString(PARAM_LABEL_TYPE));
			
			BinaryTreeNode tree = DataConverter.fastaACTStringToTree(sequence);
			String newickString = SwcToNewick.getNewickString(tree, null, labelType);
			DataWriter.writeStringToFile(new File(output), newickString);
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = new JSAP();
		FlaggedOption opt;

    	opt = new FlaggedOption(PARAM_INPUT_SEQUENCE)
		.setStringParser(JSAP.STRING_PARSER)
		.setRequired(true) 
		.setShortFlag('s')
		.setLongFlag(PARAM_INPUT_SEQUENCE);
        jsap.registerParameter(opt);

    	opt = new FlaggedOption(PARAM_OUTPUT_FILE)
		.setStringParser(JSAP.STRING_PARSER)
		.setRequired(false)
		.setShortFlag('o') 
		.setLongFlag(PARAM_OUTPUT_FILE);
        jsap.registerParameter(opt);

    	opt = new FlaggedOption(PARAM_LABEL_TYPE)
		.setStringParser(JSAP.STRING_PARSER)
		.setRequired(false)
		.setDefault("None")
		.setShortFlag('L') 
		.setLongFlag(PARAM_LABEL_TYPE);
    	opt.setHelp("None\nACT\nNode_Type\nOrder_And_Type\n");
        jsap.registerParameter(opt);
        
        JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (@SuppressWarnings("rawtypes")
			java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + NeuriteSequenceToNewick.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        return config;
	}

	
}
