package org.cn3.ns.scripts;

import java.text.NumberFormat;
import java.util.Map;

import org.cn3.morpho.data.DataWriter;
import org.cn3.morpho.domain.Asymmetry;
import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.morpho.domain.DistributionBean;
import org.cn3.morpho.generation.BinaryTreeGenerator;
import org.cn3.morpho.utils.BinaryTreeUtils;
import org.cn3.ns.generation.NodeSequenceBinaryTreeGenerator;
import org.cn3.ns.motif.analysis.MotifFinder;
import org.cn3.ns.resources.NSConstants;
import org.cn3.ns.utils.MappingUtils;

/**
 * 
 * @author gillette
 * Generates random trees produces with terminal and segmental growth models over various tree sizes 
 * and measures asymmetry, caulescence, %C, and dimer percentile ranks.
 *
 * java -classpath .:lib/JSAP-2.1.jar org.cn3.ns.scripts.TerminalSegmentalComparer
 *
 */
public class TerminalSegmentalComparer {

	private static final int TREES_PER_SIZE = 100;
	private static NumberFormat nf = null;
	//private static final String OUTPUT_DIRECTORY = "C:/Documents and Settings/gillette/My Documents/research/MotifAnalysis/";
	private static final String OUTPUT_DIRECTORY = "/home/tgillett/results/MotifAnalysis/";
	
	public static void main(String[] args){
		try{
			nf = NumberFormat.getInstance();
			nf.setMaximumFractionDigits(4);
			NodeSequenceBinaryTreeGenerator.debugLevel = 0;
			BinaryTreeNode tree;
			String sequence;
			int termDegree, size;
			Map<String,Number> motifs;
			/*StringBuilder terminalFasta = new StringBuilder();
			StringBuilder segmentalFasta = new StringBuilder();
			StringBuilder apicalModelFasta = new StringBuilder();
			StringBuilder axonModelFasta = new StringBuilder();*/
			String[] selectTrimers = {"AAA","AAT","AAC","ACC","ACT","ATA","ATC","CAC","CAT","CCA","CCT","CTA","TCA","TCC","TCT"};
			StringBuilder trimerString = new StringBuilder();
			for (String trimer : selectTrimers){
				trimerString.append(","+trimer);
			}
			
			StringBuilder datSb = new StringBuilder();
			datSb.append("Growth,Length,Asymmetry,Caulescence,PercentC,AA,AC,AT,CA,CC,CT,TA,TC,TT"+trimerString.toString()+"\n");
			
			/*
			Map<String,Integer> lengthMap = NSDataReader.loadLengthDatabaseFromFastaFile(
					NSConstants.MAPPING_MODE_TOPO_ACT, "NM_SpurLen2D", "STL");
			List<Integer> lengths = NSDataReader.loadUniqueSequenceLengths(lengthMap);
			*/

			// Loop through set lengths and generate set of trees
			//for (int length = 20; length < 140; length+=20){
			for (int length = 20; length < 160; length = (int)(length*1.1)){
			//for (int length : lengths){
				if (length < 200){
					System.out.println("Sequence Length "+length);
					termDegree = length+1;
					size = termDegree + length; // size = number of branches
					for (int treeNum = 1; treeNum <= TREES_PER_SIZE; treeNum++){
						System.out.println("Tree "+treeNum);
						
						/** Generate terminal growth **/
						tree = BinaryTreeGenerator.generateTerminalGrowthBinaryTreeByDegree(termDegree);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
		
						//terminalFasta.append(">TerminalL"+length+"N"+treeNum+"\n"+sequence+"\n");
						motifs = MotifFinder.findMotifs(sequence,1,2);
						//System.out.println(motifs.keySet());
						datSb.append("Terminal,"+length+","+
								nf.format(tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM))+","+
								nf.format(tree.getAsymmetry().getCaulescence())+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs));
						datSb.append(addKmers(3, selectTrimers, termDegree, sequence));
						datSb.append("\n");
						
						/** Generate terminal growth with low-order bias (Q=0,S=0.4) **/
						tree = BinaryTreeGenerator.generateQSBinaryTreeBySize(2*termDegree-1, 0, 0.4);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
		
						//terminalFasta.append(">TerminalL"+length+"N"+treeNum+"\n"+sequence+"\n");
						motifs = MotifFinder.findMotifs(sequence,1,2);
						//System.out.println(motifs.keySet());
						datSb.append("Terminal_S0.4,"+length+","+
								nf.format(tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM))+","+
								nf.format(tree.getAsymmetry(BinaryTreeUtils.METRIC_NODES,BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM).getCaulescence())+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs));
						datSb.append(addKmers(3, selectTrimers, termDegree, sequence));
						datSb.append("\n");
						
						/** Generate segmental growth **/
						tree = BinaryTreeGenerator.generateSegmentalGrowthBinaryTreeByDegree(termDegree);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
					
						//segmentalFasta.append(">SegmentalL"+length+"N"+treeNum+"\n"+sequence+"\n");
						motifs = MotifFinder.findMotifs(sequence,1,2);
						datSb.append("Segmental,"+length+","+
								nf.format(tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM))+","+
								nf.format(tree.getAsymmetry().getCaulescence())+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs));
						datSb.append(addKmers(3, selectTrimers, termDegree, sequence));
						datSb.append("\n");
						
						/*
						// Apical model 1 - Terminal attached to segmental branch of highest order
						tree = BinaryTreeGenerator.generateSegmentalGrowthBinaryTreeByDegree(1+(int)(termDegree/2));
						BinaryTreeNode termRoot = BinaryTreeGenerator.generateTerminalGrowthBinaryTreeByDegree(termDegree-(int)(termDegree/2));
						BinaryTreeNode attachment = tree.getMaxOrderBranch();
						attachment.setLeft(termRoot.getLeft());
						attachment.setRight(termRoot.getRight());
						BinaryTreeNode.updateStatisticsDown(tree);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
	
						//apicalModelFasta.append(">ApicalModelL"+length+"N"+treeNum+"\n"+sequence+"\n");
						motifs = MotifFinder.findMotifs(sequence,1,2);
						datSb.append("ApicalModel1,"+length+","+
								tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM)+","+
								tree.getAsymmetry().getCaulescence()+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs)+"\n");
	
						// Axonal model 1 - Terminal growth followed by segmental growth on initial tree
						tree = BinaryTreeGenerator.generateTerminalGrowthBinaryTreeByDegree(1+(int)(termDegree/2));
						BinaryTreeGenerator.growTreeSegmentally(tree, 2*termDegree-1);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
						
						motifs = MotifFinder.findMotifs(sequence,1,2);
						datSb.append("AxonModel1,"+length+","+
								tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM)+","+
								tree.getAsymmetry().getCaulescence()+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs)+"\n");

						// Axonal model 2 - QS model where Q = .1
						tree = BinaryTreeGenerator.generateQSBinaryTreeBySize(size, .1, 0);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
						
						motifs = MotifFinder.findMotifs(sequence,1,2);
						datSb.append("AxonModel2,"+length+","+
								tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM)+","+
								tree.getAsymmetry().getCaulescence()+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs)+"\n");

						// Axonal model 2 - QS model where Q = .2
						tree = BinaryTreeGenerator.generateQSBinaryTreeBySize(size, .2, 0);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
						
						motifs = MotifFinder.findMotifs(sequence,1,2);
						datSb.append("AxonModel3,"+length+","+
								tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM)+","+
								tree.getAsymmetry().getCaulescence()+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs)+"\n");

						// Apical model 2 - QS model where Q = .4 and S = -0.3
						tree = BinaryTreeGenerator.generateQSBinaryTreeBySize(size, .4, -0.3);
						sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
						
						motifs = MotifFinder.findMotifs(sequence,1,2);
						datSb.append("ApicalModel2,"+length+","+
								tree.getTreeAsymmetry(BinaryTreeNode.ASYMMETRY_WEIGHT_UNIFORM, BinaryTreeUtils.METRIC_NODES, Asymmetry.SIZE_MODE_SUM)+","+
								tree.getAsymmetry().getCaulescence()+","+
								nf.format(motifs.get("C").doubleValue()/length));
						datSb.append(addDimers(termDegree,motifs)+"\n");
	*/
					}
				}
			}
			
			// Output results
			String outputFilename = OUTPUT_DIRECTORY+"GrowthProperties_MainTrio_WSelectTrimers2.csv";
			DataWriter.writeStringToFile(outputFilename, datSb.toString());
			System.out.println("Wrote file to "+outputFilename);
			/*
			DataWriter.writeStringToFile(OUTPUT_DIRECTORY+"GrowthPropertiesExt.csv", datSb.toString());
*/
	/*		
			DataWriter.writeStringToFile(OUTPUT_DIRECTORY+"TerminalGrowthSequencesExt.fasta", terminalFasta.toString());
			DataWriter.writeStringToFile(OUTPUT_DIRECTORY+"SegmentalGrowthSequencesExt.fasta", segmentalFasta.toString());
			DataWriter.writeStringToFile(OUTPUT_DIRECTORY+"ApicalModelGrowthSequencesExt.fasta", apicalModelFasta.toString());
*/
			//DataWriter.writeStringToFile(OUTPUT_DIRECTORY+"AllModelGrowthProperties.csv", datSb.toString());
//			DataWriter.writeStringToFile(OUTPUT_DIRECTORY+"AxonModelGrowthSequences.fasta", axonModelFasta.toString());

		}catch (Exception e){
			e.printStackTrace();
		}
	}

	private static DistributionBean[] dists = new DistributionBean[9];
	private static String addDimers(int termDegree, Map<String,Number> treeMotifs) throws Exception{
		StringBuilder sb = new StringBuilder();
		String[] dimers = {"AA","AC","AT","CA","CC","CT","TA","TC","TT"};
		BinaryTreeNode tree;
		String sequence;
		Map<String,Number> surrogateMotifs;
		for (int dimerNum = 0; dimerNum < 9; dimerNum++){
			dists[dimerNum] = new DistributionBean();
		}

		// Generate monomer constrained surrogates
		//System.out.println(termDegree+" " +treeMotifs.get("C").intValue());
		for (int i = 0; i < 100; i++){
			tree = NodeSequenceBinaryTreeGenerator.generateRandomMonomerConstrainedHardingTreeByDegree(termDegree, treeMotifs.get("C").intValue());
			sequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
			surrogateMotifs = MotifFinder.findMotifs(sequence,2);
			for (int dimerNum = 0; dimerNum < 9; dimerNum++){
				dists[dimerNum].addData(surrogateMotifs.get(dimers[dimerNum]));
			}
		}
		
		for (int dimerNum = 0; dimerNum < 9; dimerNum++){
			sb.append(","+nf.format(dists[dimerNum].getPercentileRankForValue(treeMotifs.get(dimers[dimerNum]).doubleValue())));
		}
		
		return(sb.toString());
	}

	private static String addKmers(int k, String[] kmers, int termDegree, String sequence) throws Exception{
		StringBuilder sb = new StringBuilder();
		DistributionBean[] dists = new DistributionBean[kmers.length];
		BinaryTreeNode tree;
		String surrogateSequence;
		Map<String,Number> surrogateMotifs, sequenceConstraints = MotifFinder.findMotifs(sequence,k-1);
		for (int kmerNum = 0; kmerNum < kmers.length; kmerNum++){
			dists[kmerNum] = new DistributionBean();
		}

		// Generate monomer constrained surrogates
		//System.out.println(termDegree+" " +treeMotifs.get("C").intValue());
		for (int i = 0; i < 100; i++){
			switch(k-1){
			case 0:
				tree = NodeSequenceBinaryTreeGenerator.generateRandomHardingTreeByDegree(termDegree);
				break;
			case 1:
				tree = NodeSequenceBinaryTreeGenerator.generateRandomMonomerConstrainedHardingTreeByDegree(termDegree, sequenceConstraints.get("C").intValue());	
				break;
			case 2:
				tree = NodeSequenceBinaryTreeGenerator.generateRandomDimerConstrainedNonuniformTree(sequenceConstraints, "STL");
				break;
			case 3:
				tree = NodeSequenceBinaryTreeGenerator.generateRandomTrimerConstrainedNonuniformTree(sequenceConstraints, "STL");
				break;
			case 4:
				tree = NodeSequenceBinaryTreeGenerator.generateRandomTetramerConstrainedNonuniformTree(sequenceConstraints, "STL");
				break;
			default:
				throw new Exception("k value "+k+" not allowed as constraint level only goes up to 4-mers");
			}
			
			surrogateSequence = MappingUtils.traverseTreeToString(tree, NSConstants.MAPPING_MODE_TOPO_ACT, "STL");
			surrogateMotifs = MotifFinder.findMotifs(surrogateSequence,k);
			for (int kmerNum = 0; kmerNum < kmers.length; kmerNum++){
				dists[kmerNum].addData(surrogateMotifs.get(kmers[kmerNum]));
			}
		}
		
		Map<String,Number> sequenceMotifs = MotifFinder.findMotifs(sequence,k);
		for (int kmerNum = 0; kmerNum < kmers.length; kmerNum++){
			sb.append(","+nf.format(dists[kmerNum].getPercentileRankForValue(sequenceMotifs.get(kmers[kmerNum]).doubleValue())));
		}
		
		return(sb.toString());
	}

	/*
	private static JSAPResult readParameters(String[] args) throws Exception{
		JSAP jsap = MotifConstants.getParameters();
        
        FlaggedOption opt;

        opt = new FlaggedOption(NSConstants.PARAM_DATASET)
    	.setStringParser(JSAP.STRING_PARSER)
        .setRequired(false)
        .setDefault(NSConstants.DEFAULT_DATASET)
        .setShortFlag('D') 
        .setLongFlag(NSConstants.PARAM_DATASET);
        jsap.registerParameter(opt);


    	JSAPResult config = jsap.parse(args);
        if (!config.success()) {
        	for (@SuppressWarnings("rawtypes")
			java.util.Iterator errs = config.getErrorMessageIterator();errs.hasNext();) {
        		System.err.println("Error: " + errs.next());
        	}
            System.err.println();
            System.err.println("Usage: java "
                                + TerminalSegmentalComparer.class.getName());
            System.err.println("                "
                                + jsap.getUsage());
            System.err.println();
            // show full help as well
            System.err.println(jsap.getHelp());
            System.exit(1);
        }
        
        return config;
	}
*/
	
}
