package org.cn3.ns.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import org.cn3.morpho.domain.BinaryTreeNode;
import org.cn3.ns.nodeencoding.JoinedTreeTopo1Encoder;
import org.cn3.ns.nodeencoding.EncodingMode;
import org.cn3.ns.nodeencoding.NodeEncoder;
import org.cn3.ns.nodeencoding.ParentRelativeLengthAngleTopo1Encoder;
import org.cn3.ns.nodeencoding.ParentRelativeLengthAngleTopo1SeparateEncoder;
import org.cn3.ns.nodeencoding.Topological0thOrderEncoder;
import org.cn3.ns.nodeencoding.Topological1stOrderEncoder;
import org.cn3.ns.nodeencoding.Topological2ndOrderEncoder;
import org.cn3.ns.nodeencoding.Topology1AndRemoteAngleEncoder;
import org.cn3.ns.nodeencoding.TreeRelativeLengthAngleTopo1Encoder;
import org.cn3.ns.resources.NSConstants;

public class MappingUtils {

	private static char[] charSetAC = {'A','C'};
	private static char[] charSetACT = {'A','C','T'};
	private static char[] charSetGACT = {'G','A','C','T'};
	private static char[] charSet20 = {'A','R','N','D','C','Q','E','G','H','I','L','K','M','F','P','S','T','W','Y','V'};
	/*private static char[] charSetAllResidues = 
		{'A','R','N','D','C','Q','E','G','H','I','L','K','M','F','P','S','T','W','Y','V','B','J','Z','X'};*/

	private static char[] getInternalCharSet(int mapping){
		switch(mapping){
		case NSConstants.MAPPING_MODE_TOPO_GACT:
			return charSetGACT;
		case NSConstants.MAPPING_MODE_TOPO_ACT:
			return charSetACT;
		case NSConstants.MAPPING_MODE_TOPO_CA:
			return charSetAC;
		default: 
			return charSet20;
		}
		
	}

	public static char[] getCharSet(int mapping){
		return getInternalCharSet(mapping).clone();
	}

	public static char[] getCharSetWithGap(int mapping){
		return getCharSet(mapping, true);
	}
	public static char[] getCharSet(int mapping, boolean includeGap){
		if (!includeGap) return getCharSet(mapping);
		char[] charSet = getInternalCharSet(mapping);
		char[] finalCharSet = new char[charSet.length+1];
		for (int i = 0; i < charSet.length; i++){
			finalCharSet[i] = charSet[i];
		}
		finalCharSet[charSet.length] = '-';
		return finalCharSet;
	}
	

	public static String traverseTreeToString(
			BinaryTreeNode root, int mappingMode) throws Exception{
		return traverseTreeToString(root,mappingMode,
				NSConstants.DEFAULT_TRAVERSAL,NSConstants.DEFAULT_ORDER);
	}
	public static String traverseTreeToString(
			BinaryTreeNode root, int mappingMode, String order)
	throws Exception{
		return traverseTreeToString(root, mappingMode, NSConstants.DEFAULT_TRAVERSAL, order);
	}

	public static String traverseTreeToString(
			BinaryTreeNode root, int mappingMode, String traversal, String order)
	throws Exception{
		NodeEncoder nodeEncoder;
		switch (mappingMode){
		case NSConstants.MAPPING_MODE_TOPO_ACT:
			nodeEncoder = new Topological1stOrderEncoder();
			break;
		case NSConstants.MAPPING_MODE_TOPO_LEN_ANG:
			nodeEncoder = new TreeRelativeLengthAngleTopo1Encoder(root);
			break;
		case NSConstants.MAPPING_MODE_TOPO_GACT:
			nodeEncoder = new JoinedTreeTopo1Encoder();
			break;
		case NSConstants.MAPPING_MODE_TOPO_LOCAL_LEN_ANG:
			nodeEncoder = new ParentRelativeLengthAngleTopo1Encoder();
			break;
		case NSConstants.MAPPING_MODE_TOPO_LTB_AOB_SEPARATE:
			nodeEncoder = new ParentRelativeLengthAngleTopo1SeparateEncoder();
			break;
		case NSConstants.MAPPING_MODE_TOPO_TWO:
			nodeEncoder = new Topological2ndOrderEncoder();
			break;
		default:
			throw new Exception("Unknown mapping mode: "+mappingMode);
		}
		return nodeEncoder.encodeTree(root, order);
		/*		
		switch (mappingMode){
		case NSConstants.MAPPING_MODE_TOPO_ACT:
			return traverseTreeToStringWithMappingMode2(root,order);
		case NSConstants.MAPPING_MODE_TOPO_LEN_ANG:
			return traverseTreeToStringWithMappingMode3(root,order);
		case NSConstants.MAPPING_MODE_TOPO_GACT:
			return traverseTreeToStringWithMappingMode4(root,order);
		case NSConstants.MAPPING_MODE_TOPO_LOCAL_LEN_ANG:
			return traverseTreeToStringWithMappingMode5(root,order);
		case NSConstants.MAPPING_MODE_TOPO_LTB_AOB_SEPARATE:
			return traverseTreeToStringWithMappingMode6(root,order);
		case NSConstants.MAPPING_MODE_TOPO_TWO:
			return traverseTreeToStringWithMappingMode7(root,order);
		default:
			throw new Exception("Unknown mapping mode: "+mappingMode);
		}
			*/
	}

	public static List<BinaryTreeNode> traverseTreeToNodeList(
			BinaryTreeNode root, String order)throws Exception{
		return traverseTreeToNodeList(root, false, order);
	}
	public static List<BinaryTreeNode> traverseTreeToNodeList(
			BinaryTreeNode root, boolean includeTerminations, String order) throws Exception{
		List<BinaryTreeNode> nodeList = new ArrayList<BinaryTreeNode>(
				root.getDegree()+(includeTerminations?root.getDegree():0));
    	Stack<BinaryTreeNode> stack = new Stack<BinaryTreeNode>();
    	BinaryTreeNode current;
    	stack.push(root);
    	while (!stack.isEmpty()){
    		current = stack.pop();
    		//System.out.println("Check: "+mapNodeMode2(current)+"");
    		if (current.hasChildren() || includeTerminations){
    			nodeList.add(current);

    			addChildrenToStack(stack, current, order);
    		}
    	}

    	return nodeList;
	}

	public static int getLengthFromSize(int size, int mapping){
		switch (NSConstants.mappingModeSequenceType[mapping]){
		case NSConstants.SEQUENCE_TOPOLOGY_ZERO:
			return size;
		case NSConstants.SEQUENCE_TOPOLOGY_ONE:
			return (size - 1)/2;
		case NSConstants.SEQUENCE_TOPOLOGY_TWO:
			return ((size - 1)/2 - 1)/2;
		default:
			return size;
		}
	}

	public static int getSizeFromLength(int length, int mapping){
		switch (NSConstants.mappingModeSequenceType[mapping]){
		case NSConstants.SEQUENCE_TOPOLOGY_ZERO:
			return length;
		case NSConstants.SEQUENCE_TOPOLOGY_ONE:
			return 2*length + 1;
		case NSConstants.SEQUENCE_TOPOLOGY_TWO:
			return 2*(2*length + 1) + 1;
		default:
			return length;
		}
	}

	public static void addChildrenToStack(
			Stack<BinaryTreeNode> stack, BinaryTreeNode node, String order) throws Exception{
		BinaryTreeNode right = node.getRight();
		BinaryTreeNode left = node.getLeft();
    	if (right != null){
    		if (left != null){
		    	if (right.getTreeSize() > left.getTreeSize()){
		    		addToStack(stack, right, left, order);
		    	}
		    	else if (left.getTreeSize() > right.getTreeSize()){
		    		addToStack(stack, left, right, order);
		    	}
		    	/* Larger shapeNum generally less asymmetric, less avgHeight, 
		    	 * so smaller shapeNum should go first */
		    	else if (left.getShapeNum().compareTo(right.getShapeNum()) > 0){
    	    		addToStack(stack, right, left, order);
    	    	}
    	    	else if (left.getShapeNum().compareTo(right.getShapeNum()) < 0){
    	    		addToStack(stack, left, right, order);
    	    	}
   	    		// no differences, default
	    		else{
	    			addToStack(stack, left, right, order);
    	    	}	
    		}
    		else{
    			stack.push(right);
    		}
    	}
    	else if (left != null){
    		stack.push(left);
    	}
	}

	private static void addToStack(
    		Stack<BinaryTreeNode> stack, 
    		BinaryTreeNode larger, 
    		BinaryTreeNode smaller,
    		String order){
    	if (order.equalsIgnoreCase("LTS")){
    		stack.push(smaller);
			stack.push(larger);
    	}
    	else{
    		stack.push(larger);
			stack.push(smaller);
    	}
    }

    public static int getTreeSizeFromSequenceLength(int length, int mappingMode){
		switch (mappingMode){
		case NSConstants.MAPPING_MODE_TOPO_ACT:
		case NSConstants.MAPPING_MODE_TOPO_GACT:
		case NSConstants.MAPPING_MODE_TOPO_LEN_ANG:
		case NSConstants.MAPPING_MODE_TOPO_LOCAL_LEN_ANG:
			return length*2+1;
		default:
			return length;
		}
    }
    
	public static String traverseTreeToStringWithMappingMode2(
			BinaryTreeNode root)throws Exception{
		return traverseTreeToStringWithMappingMode2(root, NSConstants.DEFAULT_ORDER);
	}
	public static String traverseTreeToStringWithMappingMode2(
			BinaryTreeNode root, String order)throws Exception{
		NodeEncoder nodeEncoder = new Topological1stOrderEncoder();
		return nodeEncoder.encodeTree(root, order);
	}
	
	// Part of conversion to using enum for mapping mode
	private static char[] getInternalCharSet(EncodingMode mapping){
		switch(mapping){
		case TOPO_GACT:
			return charSetGACT;
		case TOPO_ONE:
			return charSetACT;
		case TOPO_ZERO:
			return charSetAC;
		default: 
			return charSet20;
		}
	}
	public static char[] getCharSet(EncodingMode mapping){
		return getInternalCharSet(mapping).clone();
	}
	public static char[] getCharSetWithGap(EncodingMode mapping){
		return getCharSet(mapping, true);
	}
	public static char[] getCharSet(EncodingMode mapping, boolean includeGap){
		if (!includeGap) return getCharSet(mapping);
		char[] charSet = getInternalCharSet(mapping);
		char[] finalCharSet = new char[charSet.length+1];
		for (int i = 0; i < charSet.length; i++){
			finalCharSet[i] = charSet[i];
		}
		finalCharSet[charSet.length] = '-';
		return finalCharSet;
	}
	
	public static String traverseTreeToString(
			BinaryTreeNode root, EncodingMode mapping) throws Exception{
		return traverseTreeToString(root,mapping,
				NSConstants.DEFAULT_TRAVERSAL,NSConstants.DEFAULT_ORDER);
	}
	public static String traverseTreeToString(
			BinaryTreeNode root, EncodingMode mapping, String order)
	throws Exception{
		return traverseTreeToString(root, mapping, NSConstants.DEFAULT_TRAVERSAL, order);
	}

	public static String traverseTreeToString(
			BinaryTreeNode root, EncodingMode encoding, String traversal, String order)
	throws Exception{
		NodeEncoder nodeEncoder;
		switch (encoding){
		case TOPO_ZERO:
			nodeEncoder = new Topological0thOrderEncoder();
		case TOPO_ONE:
			nodeEncoder = new Topological1stOrderEncoder();
			break;
		case LENGTH_ANGLE:
			nodeEncoder = new TreeRelativeLengthAngleTopo1Encoder(root);
			break;
		case TOPO_GACT:
			nodeEncoder = new JoinedTreeTopo1Encoder();
			break;
		case LOCAL_LENGTH_ANGLE:
			nodeEncoder = new ParentRelativeLengthAngleTopo1Encoder();
			break;
		case LTB_AOB_SEPARATE:
			nodeEncoder = new ParentRelativeLengthAngleTopo1SeparateEncoder();
			break;
		case TOPO_TWO:
			nodeEncoder = new Topological2ndOrderEncoder();
			break;
		case TOPO1_REMOTE_ANGLE:
			nodeEncoder = new Topology1AndRemoteAngleEncoder();
			break;
		default:
			throw new Exception("Unknown mapping mode: "+encoding);
		}
		return nodeEncoder.encodeTree(root, order);
	}

	public static int getLengthFromSize(int size, EncodingMode encoding){
		switch (encoding.getTopologyLevel()){
		case 0:
			return size;
		case 1:
			return (size - 1)/2;
		case 2:
			return ((size - 1)/2 - 1)/2;
		default:
			return size;
		}
	}

	public static int getSizeFromLength(int length, EncodingMode mapping){
		switch (mapping.getTopologyLevel()){
		case 0:
			return length;
		case 1:
			return 2*length + 1;
		case 2:
			return 2*(2*length + 1) + 1;
		default:
			return length;
		}
	}
    
}
